@@ -27,12 +27,9 @@
 #include <tag_c.h>
 #endif
 #include "streamtuner.h"
-#include "art/local.h"
 
 /*** cpp *********************************************************************/
 
-#define COPYRIGHT		"Copyright \302\251 2002, 2003, 2004 Jean-Yves Lefort"
-
 #define POINTER_TO_STRING(ptr)	((ptr) ? (ptr) : "")
 
 /*** type definitions ********************************************************/
@@ -42,8 +39,6 @@ typedef struct
   STStream	stream;
 
   char		*pathname;	/* full pathname, in the fs locale */
-  int		type;		/* obsolete, now always 0 */
-
   char		*filename;	/* filename only, converted to UTF-8 */
   char		*title;
   char		*artist;
@@ -59,7 +54,6 @@ typedef struct
 
 enum {
   FIELD_PATHNAME,
-  FIELD_TYPE,
   FIELD_FILENAME,
   FIELD_TITLE,
   FIELD_ARTIST,
@@ -76,7 +70,8 @@ enum {
 
 /*** variables ***************************************************************/
 
-static STHandler *handler = NULL;
+static STPlugin *local_plugin = NULL;
+static STHandler *local_handler = NULL;
 
 /*** function declarations ***************************************************/
 
@@ -136,6 +131,9 @@ static gboolean metadata_write (LocalStream *stream,
 				GError **err);
 #endif /* WITH_LOCAL_METADATA */
 
+static void init_handler (void);
+static gboolean check_api_version (GError **err);
+
 /*** implementation **********************************************************/
 
 static LocalStream *
@@ -156,10 +154,6 @@ stream_field_get_cb (LocalStream *stream,
       g_value_set_string(value, stream->pathname);
       break;
 
-    case FIELD_TYPE:
-      g_value_set_int(value, 0); /* obsolete */
-      break;
-
     case FIELD_FILENAME:
       g_value_set_string(value, stream->filename);
       break;
@@ -205,9 +199,9 @@ stream_field_get_cb (LocalStream *stream,
       break;
 
     case FIELD_AUDIO:
-      g_value_set_string_take_ownership(value, st_format_audio_properties(stream->bitrate,
-									  stream->samplerate,
-									  stream->channels));
+      g_value_take_string(value, st_format_audio_properties(stream->bitrate,
+							    stream->samplerate,
+							    stream->channels));
       break;
 
     default:
@@ -227,10 +221,6 @@ stream_field_set_cb (LocalStream *stream,
       stream->pathname = g_value_dup_string(value);
       break;
 
-    case FIELD_TYPE:
-      /* obsolete, nop */
-      break;
-
     case FIELD_FILENAME:
       stream->filename = g_value_dup_string(value);
       break;
@@ -308,6 +298,43 @@ stream_stock_field_get_cb (LocalStream *stream,
     case ST_HANDLER_STOCK_FIELD_GENRE:
       g_value_set_string(value, stream->genre);
       break;
+
+    case ST_HANDLER_STOCK_FIELD_DESCRIPTION:
+      g_value_set_string(value, stream->comment);
+      break;
+
+    case ST_HANDLER_STOCK_FIELD_HOMEPAGE:
+      /* nop */
+      break;
+
+    case ST_HANDLER_STOCK_FIELD_URI_LIST:
+      {
+	char *uri;
+	GError *err = NULL;
+
+	uri = g_filename_to_uri(stream->pathname, NULL, &err);
+	if (uri)
+	  {
+	    GValueArray *value_array;
+	    GValue uri_value = { 0, };
+	    
+	    value_array = g_value_array_new(1);
+
+	    g_value_init(&uri_value, G_TYPE_STRING);
+	    g_value_take_string(&uri_value, uri);
+	    g_value_array_append(value_array, &uri_value);
+	    g_value_unset(&uri_value);
+
+	    g_value_take_boxed(value, value_array);
+	  }
+	else
+	  {
+	    st_handler_notice(local_handler, _("%s: unable to convert filename to URI: %s"), stream->pathname, err->message);
+	    g_error_free(err);
+	  }
+
+	break;
+      }
     }
 }
 
@@ -583,7 +610,7 @@ reload_categories (const char *music_dir, GNode *root, GError **err)
 	  category->label = g_filename_to_utf8(filename, -1, NULL, NULL, &tmp_err);
 	  if (! category->label)
 	    {
-	      st_handler_notice(handler, _("%s: unable to convert directory name to UTF-8 encoding: %s"), pathname, tmp_err->message);
+	      st_handler_notice(local_handler, _("%s: unable to convert directory name to UTF-8 encoding: %s"), pathname, tmp_err->message);
 	      g_clear_error(&tmp_err);
 	    }
 	
@@ -654,10 +681,10 @@ reload_streams (const char *music_dir,
       
       extension = strrchr(filename, '.');
       if (! (extension++
-	     && (! strcasecmp(extension, "mp3")
-		 || ! strcasecmp(extension, "ogg")
-		 || ! strcasecmp(extension, "m3u")
-		 || ! strcasecmp(extension, "pls"))))
+	     && (! g_ascii_strcasecmp(extension, "mp3")
+		 || ! g_ascii_strcasecmp(extension, "ogg")
+		 || ! g_ascii_strcasecmp(extension, "m3u")
+		 || ! g_ascii_strcasecmp(extension, "pls"))))
 	continue;		/* unhandled */
 
       stream = stream_new_cb(NULL);
@@ -668,7 +695,7 @@ reload_streams (const char *music_dir,
       stream->filename = g_filename_to_utf8(filename, -1, NULL, NULL, &tmp_err);
       if (! stream->filename)
 	{
-	  st_handler_notice(handler, _("%s: unable to convert filename to UTF-8 encoding: %s"), stream->pathname, tmp_err->message);
+	  st_handler_notice(local_handler, _("%s: unable to convert filename to UTF-8 encoding: %s"), stream->pathname, tmp_err->message);
 	  g_clear_error(&tmp_err);
 	}
       
@@ -700,7 +727,7 @@ metadata_read (LocalStream *stream)
   file = taglib_file_new(stream->pathname);
   if (! file)
     {
-      st_handler_notice(handler, _("unable to open %s"), stream->pathname);
+      st_handler_notice(local_handler, _("unable to open %s"), stream->pathname);
       return;
     }
 
@@ -742,7 +769,7 @@ metadata_read (LocalStream *stream)
       taglib_tag_free_strings();
     }
   else
-    st_handler_notice(handler, _("%s has no tag"), stream->pathname);
+    st_handler_notice(local_handler, _("%s has no tag"), stream->pathname);
   
   audio_properties = taglib_file_audioproperties(file);
   if (audio_properties)
@@ -758,7 +785,7 @@ metadata_read (LocalStream *stream)
       stream->channels = taglib_audioproperties_channels(audio_properties);
     }
   else
-    st_handler_notice(handler, _("%s has no audio properties"), stream->pathname);
+    st_handler_notice(local_handler, _("%s has no audio properties"), stream->pathname);
 
   taglib_file_free(file);
 }
@@ -859,14 +886,11 @@ init_handler (void)
 {
   GNode *stock_categories;
   STCategory *category;
+  STHandlerField *field;
 
-  handler = st_handler_new("local");
+  local_handler = st_handler_new_from_plugin(local_plugin);
 
-  st_handler_set_label(handler, _("Local"));
-  st_handler_set_copyright(handler, COPYRIGHT);
-  st_handler_set_description(handler, _("Local Music Collection"));
-
-  st_handler_set_icon_from_inline(handler, sizeof(art_local), art_local);
+  st_handler_set_description(local_handler, _("Local Music Collection"));
 
   stock_categories = g_node_new(NULL);
 
@@ -876,115 +900,160 @@ init_handler (void)
   
   g_node_append_data(stock_categories, category);
   
-  st_handler_set_stock_categories(handler, stock_categories);
-  st_handler_set_flags(handler, ST_HANDLER_CONFIRM_DELETION);
+  st_handler_set_stock_categories(local_handler, stock_categories);
+  st_handler_set_flags(local_handler, ST_HANDLER_CONFIRM_DELETION);
 
-  st_handler_bind(handler, ST_HANDLER_EVENT_RELOAD, reload_cb, NULL);
+  st_handler_bind(local_handler, ST_HANDLER_EVENT_RELOAD, reload_cb, NULL);
 
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_NEW, stream_new_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FIELD_GET, stream_field_get_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FIELD_SET, stream_field_set_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_STOCK_FIELD_GET, stream_stock_field_get_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_MODIFY, stream_modify_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_DELETE, stream_delete_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FREE, stream_free_cb, NULL);
+  st_handler_bind(local_handler, ST_HANDLER_EVENT_STREAM_NEW, stream_new_cb, NULL);
+  st_handler_bind(local_handler, ST_HANDLER_EVENT_STREAM_FIELD_GET, stream_field_get_cb, NULL);
+  st_handler_bind(local_handler, ST_HANDLER_EVENT_STREAM_FIELD_SET, stream_field_set_cb, NULL);
+  st_handler_bind(local_handler, ST_HANDLER_EVENT_STREAM_STOCK_FIELD_GET, stream_stock_field_get_cb, NULL);
+  st_handler_bind(local_handler, ST_HANDLER_EVENT_STREAM_MODIFY, stream_modify_cb, NULL);
+  st_handler_bind(local_handler, ST_HANDLER_EVENT_STREAM_DELETE, stream_delete_cb, NULL);
+  st_handler_bind(local_handler, ST_HANDLER_EVENT_STREAM_FREE, stream_free_cb, NULL);
 
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_TUNE_IN_MULTIPLE, stream_tune_in_multiple_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_BROWSE, stream_browse_cb, NULL);
+  st_handler_bind(local_handler, ST_HANDLER_EVENT_STREAM_TUNE_IN_MULTIPLE, stream_tune_in_multiple_cb, NULL);
+  st_handler_bind(local_handler, ST_HANDLER_EVENT_STREAM_BROWSE, stream_browse_cb, NULL);
 
-  st_handler_add_field(handler,
+  st_handler_add_field(local_handler,
 		       st_handler_field_new(FIELD_PATHNAME,
 					    _("Pathname"),
 					    G_TYPE_STRING,
 					    0));
-  st_handler_add_field(handler,
-		       st_handler_field_new(FIELD_TYPE,
-					    _("Type"),
-					    G_TYPE_INT,
-					    0));
-  st_handler_add_field(handler,
-		       st_handler_field_new(FIELD_FILENAME,
-					    _("Filename"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE));
-  st_handler_add_field(handler,
-		       st_handler_field_new(FIELD_TITLE,
-					    _("Title"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE));
-  st_handler_add_field(handler,
-		       st_handler_field_new(FIELD_ARTIST,
-					    _("Artist"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE));
-  st_handler_add_field(handler,
-		       st_handler_field_new(FIELD_ALBUM,
-					    _("Album"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE));
-  st_handler_add_field(handler,
-		       st_handler_field_new(FIELD_YEAR,
-					    _("Year"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE));
-  st_handler_add_field(handler,
-		       st_handler_field_new(FIELD_GENRE,
-					    _("Genre"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE
-					    | ST_HANDLER_FIELD_START_HIDDEN));
-  st_handler_add_field(handler,
-		       st_handler_field_new(FIELD_COMMENT,
-					    _("Comment"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE
-					    | ST_HANDLER_FIELD_START_HIDDEN));
-  st_handler_add_field(handler,
-		       st_handler_field_new(FIELD_DURATION,
-					    _("Duration"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE));
-  st_handler_add_field(handler,
-		       st_handler_field_new(FIELD_AUDIO,
-					    _("Audio"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_START_HIDDEN
-					    | ST_HANDLER_FIELD_VOLATILE));
 
+  field = st_handler_field_new(FIELD_FILENAME,
+			       _("Filename"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE);
+  st_handler_field_set_description(field, _("The song filename"));
+  st_handler_add_field(local_handler, field);
+
+  field = st_handler_field_new(FIELD_TITLE,
+			       _("Title"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE);
+  st_handler_field_set_description(field, _("The song title"));
+  st_handler_add_field(local_handler, field);
+
+  field = st_handler_field_new(FIELD_ARTIST,
+			       _("Artist"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE);
+  st_handler_field_set_description(field, _("The performing artist"));
+  st_handler_add_field(local_handler, field);
+
+  field = st_handler_field_new(FIELD_ALBUM,
+			       _("Album"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE);
+  st_handler_field_set_description(field, _("The album the song was released on"));
+  st_handler_add_field(local_handler, field);
+
+  field = st_handler_field_new(FIELD_YEAR,
+			       _("Year"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE);
+  st_handler_field_set_description(field, _("The song release year"));
+  st_handler_add_field(local_handler, field);
+
+  field = st_handler_field_new(FIELD_GENRE,
+			       _("Genre"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE
+			       | ST_HANDLER_FIELD_START_HIDDEN);
+  st_handler_field_set_description(field, _("The song genre"));
+  st_handler_add_field(local_handler, field);
+
+  field = st_handler_field_new(FIELD_COMMENT,
+			       _("Comment"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE
+			       | ST_HANDLER_FIELD_START_HIDDEN);
+  st_handler_field_set_description(field, _("The song comment"));
+  st_handler_add_field(local_handler, field);
+
+  field = st_handler_field_new(FIELD_DURATION,
+			       _("Duration"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE);
+  st_handler_field_set_description(field, _("The song duration"));
+  st_handler_add_field(local_handler, field);
+
+  field = st_handler_field_new(FIELD_AUDIO,
+			       _("Audio"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_START_HIDDEN
+			       | ST_HANDLER_FIELD_VOLATILE);
+  st_handler_field_set_description(field, _("The song audio properties"));
+  st_handler_add_field(local_handler, field);
+  
   /* invisible fields */
 
-  st_handler_add_field(handler, st_handler_field_new(FIELD_BITRATE,
-						     _("Bitrate"),
-						     G_TYPE_INT,
-						     0));
-  st_handler_add_field(handler, st_handler_field_new(FIELD_SAMPLERATE,
-						     _("Sample rate"),
-						     G_TYPE_INT,
-						     0));
-  st_handler_add_field(handler, st_handler_field_new(FIELD_CHANNELS,
-						     _("Channels"),
-						     G_TYPE_INT,
-						     0));
+  st_handler_add_field(local_handler, st_handler_field_new(FIELD_BITRATE,
+							   _("Bitrate"),
+							   G_TYPE_INT,
+							   0));
+  st_handler_add_field(local_handler, st_handler_field_new(FIELD_SAMPLERATE,
+							   _("Sample rate"),
+							   G_TYPE_INT,
+							   0));
+  st_handler_add_field(local_handler, st_handler_field_new(FIELD_CHANNELS,
+							   _("Channels"),
+							   G_TYPE_INT,
+							   0));
 
-  st_handlers_add(handler);
+  st_handlers_add(local_handler);
 }
 
-gboolean
-plugin_init (GError **err)
+static gboolean
+check_api_version (GError **err)
 {
-  if (! st_check_api_version(5, 7))
+  if (st_check_api_version(5, 8))
+    return TRUE;
+  else
     {
       g_set_error(err, 0, 0, _("API version mismatch"));
       return FALSE;
     }
+}
+
+G_MODULE_EXPORT gboolean
+plugin_get_info (STPlugin *plugin, GError **err)
+{
+  GdkPixbuf *pixbuf;
+
+  if (! check_api_version(err))
+    return FALSE;
+
+  local_plugin = plugin;
+
+  st_plugin_set_name(plugin, "local");
+  st_plugin_set_label(plugin, _("Local"));
+
+  pixbuf = st_pixbuf_new_from_file(UIDIR "/local.png");
+  if (pixbuf)
+    {
+      st_plugin_set_icon_from_pixbuf(plugin, pixbuf);
+      g_object_unref(pixbuf);
+    }
+
+  return TRUE;
+}
+
+G_MODULE_EXPORT gboolean
+plugin_init (GError **err)
+{
+  if (! check_api_version(err))
+    return FALSE;
 
   init_handler();
 
