@@ -35,18 +35,17 @@
 #include <glib/gi18n.h>
 #include <libxml/parser.h>
 #include "streamtuner.h"
-#include "art/xiph.h"
 
 /*** cpp *********************************************************************/
 
-#define COPYRIGHT		"Copyright \302\251 2003, 2004 Jean-Yves Lefort"
-
 #define XIPH_HOME		"http://dir.xiph.org/"
 #define XIPH_XML		"http://dir.xiph.org/yp.xml"
 
 #define PARSER_STATE_IS_IN_DIRECTORY(state) \
   ((state)->tags && ! (state)->tags->next && ! strcmp((state)->tags->data, "directory"))
 
+#define PARSE_ERROR		st_handler_notice(xiph_handler, _("parse error at %s"), G_STRLOC)
+
 /*** types *******************************************************************/
 
 typedef struct
@@ -72,7 +71,6 @@ enum {
   FIELD_SAMPLERATE,
   FIELD_GENRE,
   FIELD_CURRENT_SONG,
-  FIELD_SERVER_DESC,	/* obsolete, not given anymore by xiph */
   FIELD_AUDIO		/* meta field (bitrate, channels and samplerate) */
 };
 
@@ -102,7 +100,8 @@ typedef struct
   
 /*** variables ***************************************************************/
 
-static STHandler *handler = NULL;
+static STPlugin *xiph_plugin = NULL;
+static STHandler *xiph_handler = NULL;
 
 static struct
 {
@@ -147,7 +146,6 @@ static void stream_stock_field_get_cb (XiphStream *stream,
 static void stream_free_cb (XiphStream *stream, gpointer data);
 static XiphStream *stream_copy (XiphStream *stream);
 
-gboolean str_isnumeric (const char *str);
 static char *stream_get_audio (XiphStream *stream);
 
 static gboolean stream_tune_in_cb (XiphStream *stream,
@@ -187,6 +185,7 @@ static void reload_streams_error_cb (gpointer user_data,
 
 static gboolean search_url_cb (STCategory *category);
 
+static gboolean check_api_version (GError **err);
 static void init_handler (void);
 
 /*** implementation **********************************************************/
@@ -237,12 +236,8 @@ stream_field_get_cb (XiphStream *stream,
       g_value_set_string(value, stream->current_song);
       break;
 
-    case FIELD_SERVER_DESC:
-      /* obsolete, nop */
-      break;
-
     case FIELD_AUDIO:
-      g_value_set_string_take_ownership(value, stream_get_audio(stream));
+      g_value_take_string(value, stream_get_audio(stream));
       break;
 
     default:
@@ -290,10 +285,6 @@ stream_field_set_cb (XiphStream *stream,
       stream->current_song = g_value_dup_string(value);
       break;
 
-    case FIELD_SERVER_DESC:
-      /* obsolete, nop */
-      break;
-
     default:
       g_assert_not_reached();
     }
@@ -314,6 +305,28 @@ stream_stock_field_get_cb (XiphStream *stream,
     case ST_HANDLER_STOCK_FIELD_GENRE:
       g_value_set_string(value, stream->genre);
       break;
+
+    case ST_HANDLER_STOCK_FIELD_DESCRIPTION:
+    case ST_HANDLER_STOCK_FIELD_HOMEPAGE:
+      /* nop */
+      break;
+
+    case ST_HANDLER_STOCK_FIELD_URI_LIST:
+      {
+	GValueArray *value_array;
+	GValue uri_value = { 0, };
+
+	value_array = g_value_array_new(1);
+
+	g_value_init(&uri_value, G_TYPE_STRING);
+	g_value_set_string(&uri_value, stream->listen_url);
+	g_value_array_append(value_array, &uri_value);
+	g_value_unset(&uri_value);
+
+	g_value_take_boxed(value, value_array);
+
+	break;
+      }
     }
 }
 
@@ -349,20 +362,6 @@ stream_copy (XiphStream *stream)
   return copy;
 }
 
-gboolean
-str_isnumeric (const char *str)
-{
-  int i;
-
-  g_return_val_if_fail(str != NULL, FALSE);
-
-  for (i = 0; str[i]; i++)
-    if (! g_ascii_isdigit(str[i]))
-      return FALSE;
-
-  return TRUE;
-}
-
 static char *
 stream_get_audio (XiphStream *stream)
 {
@@ -375,9 +374,9 @@ stream_get_audio (XiphStream *stream)
 
   if (stream->bitrate)
     {
-      if (! strncmp(stream->bitrate, "Quality", 7))
+      if (g_str_has_prefix(stream->bitrate, "Quality"))
 	g_string_append(audio, stream->bitrate);
-      else if (str_isnumeric(stream->bitrate))
+      else if (st_str_like(stream->bitrate, ST_NUMERIC))
 	{
 	  int bitrate = atoi(stream->bitrate);
 
@@ -579,7 +578,7 @@ reload_streams (GList **streams, GError **err)
     {
       g_hash_table_destroy(state.stream_properties);
       if (status) /* only display warning if the parsing was successful */
-	st_handler_notice(handler, _("EOF: found unterminated stream"));
+	PARSE_ERROR;
     }
 
   if (status)
@@ -654,7 +653,7 @@ reload_streams_start_element_cb (gpointer user_data,
     {
       if (state->stream_properties)
 	{
-	  st_handler_notice(handler, _("found unterminated stream"));
+	  PARSE_ERROR;
 	  g_hash_table_destroy(state->stream_properties);
 	}
       state->stream_properties = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
@@ -676,7 +675,7 @@ reload_streams_end_element_cb (gpointer user_data, const xmlChar *name)
       state->tags = g_slist_delete_link(state->tags, state->tags);
     }
   else
-    st_handler_notice(handler, _("end tag mismatch in XML document"));
+    PARSE_ERROR;
 
   if (PARSER_STATE_IS_IN_DIRECTORY(state) && ! strcmp(name, "entry"))
     {
@@ -702,7 +701,7 @@ reload_streams_end_element_cb (gpointer user_data, const xmlChar *name)
 	  state->streams = g_list_append(state->streams, stream);
 	}
       else
-	st_handler_notice(handler, _("found stream without URL"));
+	PARSE_ERROR;
 
       g_hash_table_destroy(state->stream_properties);
       state->stream_properties = NULL;
@@ -749,7 +748,7 @@ reload_streams_warning_cb (gpointer user_data, const char *format, ...)
   message = g_strdup_vprintf(format, args);
   va_end(args);
 
-  st_handler_notice(handler, _("XML document: %s"), message);
+  st_handler_notice(xiph_handler, _("XML document: %s"), message);
   g_free(message);
 }
 
@@ -768,7 +767,7 @@ reload_streams_error_cb (gpointer user_data, const char *format, ...)
     state->error = g_strdup(message);
 
   /* ...and display them all */
-  st_handler_notice(handler, _("XML document: unrecoverable error: %s"), message);
+  st_handler_notice(xiph_handler, _("XML document: unrecoverable error: %s"), message);
   g_free(message);
 }
 
@@ -798,13 +797,12 @@ init_handler (void)
   GNode *stock_categories;
   STCategory *category;
   int i;
+  STHandlerField *field;
 
-  handler = st_handler_new("xiph");
+  xiph_handler = st_handler_new_from_plugin(xiph_plugin);
 
-  st_handler_set_label(handler, "Xiph");
-  st_handler_set_copyright(handler, COPYRIGHT);
-  st_handler_set_description(handler, _("Xiph.org Streaming Directory"));
-  st_handler_set_home(handler, XIPH_HOME);
+  st_handler_set_description(xiph_handler, _("Xiph.org Streaming Directory"));
+  st_handler_set_home(xiph_handler, XIPH_HOME);
 
   stock_categories = g_node_new(NULL);
 
@@ -836,85 +834,129 @@ init_handler (void)
       g_node_append_data(stock_categories, category);
     }
   
-  st_handler_set_icon(handler, sizeof(art_xiph), art_xiph);
-  st_handler_set_stock_categories(handler, stock_categories);
+  st_handler_set_stock_categories(xiph_handler, stock_categories);
 
-  st_handler_bind(handler, ST_HANDLER_EVENT_RELOAD_MULTIPLE, reload_multiple_cb, NULL);
+  st_handler_bind(xiph_handler, ST_HANDLER_EVENT_RELOAD_MULTIPLE, reload_multiple_cb, NULL);
 
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_NEW, stream_new_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FIELD_GET, stream_field_get_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FIELD_SET, stream_field_set_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_STOCK_FIELD_GET, stream_stock_field_get_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FREE, stream_free_cb, NULL);
+  st_handler_bind(xiph_handler, ST_HANDLER_EVENT_STREAM_NEW, stream_new_cb, NULL);
+  st_handler_bind(xiph_handler, ST_HANDLER_EVENT_STREAM_FIELD_GET, stream_field_get_cb, NULL);
+  st_handler_bind(xiph_handler, ST_HANDLER_EVENT_STREAM_FIELD_SET, stream_field_set_cb, NULL);
+  st_handler_bind(xiph_handler, ST_HANDLER_EVENT_STREAM_STOCK_FIELD_GET, stream_stock_field_get_cb, NULL);
+  st_handler_bind(xiph_handler, ST_HANDLER_EVENT_STREAM_FREE, stream_free_cb, NULL);
 
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_TUNE_IN, stream_tune_in_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_RECORD, stream_record_cb, NULL);
+  st_handler_bind(xiph_handler, ST_HANDLER_EVENT_STREAM_TUNE_IN, stream_tune_in_cb, NULL);
+  st_handler_bind(xiph_handler, ST_HANDLER_EVENT_STREAM_RECORD, stream_record_cb, NULL);
 
   /* visible fields */
 
-  st_handler_add_field(handler, st_handler_field_new(FIELD_SERVER_NAME,
-						     _("Name"),
-						     G_TYPE_STRING,
-						     ST_HANDLER_FIELD_VISIBLE));
-  st_handler_add_field(handler, st_handler_field_new(FIELD_SERVER_DESC,
-						     _("Description"),
-						     G_TYPE_STRING,
-						     0)); /* obsolete, invisible */
-  st_handler_add_field(handler, st_handler_field_new(FIELD_GENRE,
-						     _("Genre"),
-						     G_TYPE_STRING,
-						     ST_HANDLER_FIELD_VISIBLE));
-  st_handler_add_field(handler, st_handler_field_new(FIELD_CURRENT_SONG,
-						     _("Current song"),
-						     G_TYPE_STRING,
-						     ST_HANDLER_FIELD_VISIBLE));
-  st_handler_add_field(handler, st_handler_field_new(FIELD_SERVER_TYPE,
-						     _("Type"),
-						     G_TYPE_STRING,
-						     ST_HANDLER_FIELD_VISIBLE));
-  st_handler_add_field(handler, st_handler_field_new(FIELD_AUDIO,
-						     _("Audio"),
-						     G_TYPE_STRING,
-						     ST_HANDLER_FIELD_VISIBLE
-						     | ST_HANDLER_FIELD_VOLATILE));
-  st_handler_add_field(handler, st_handler_field_new(FIELD_LISTEN_URL,
-						     _("URL"),
-						     G_TYPE_STRING,
-						     ST_HANDLER_FIELD_VISIBLE
-						     | ST_HANDLER_FIELD_START_HIDDEN));
+  field = st_handler_field_new(FIELD_SERVER_NAME,
+			       _("Name"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE);
+  st_handler_field_set_description(field, _("The stream name"));
+  st_handler_add_field(xiph_handler, field);
+
+  field = st_handler_field_new(FIELD_GENRE,
+			       _("Genre"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE);
+  st_handler_field_set_description(field, _("The stream genre"));
+  st_handler_add_field(xiph_handler, field);
+
+  field = st_handler_field_new(FIELD_CURRENT_SONG,
+			       _("Current song"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE);
+  st_handler_field_set_description(field, _("The currently playing song"));
+  st_handler_add_field(xiph_handler, field);
+
+  field = st_handler_field_new(FIELD_SERVER_TYPE,
+			       _("Type"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE);
+  st_handler_field_set_description(field, _("The stream type"));
+  st_handler_add_field(xiph_handler, field);
+
+  field = st_handler_field_new(FIELD_AUDIO,
+			       _("Audio"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_VOLATILE);
+  st_handler_field_set_description(field, _("The stream audio properties"));
+  st_handler_add_field(xiph_handler, field);
+
+  field = st_handler_field_new(FIELD_LISTEN_URL,
+			       _("URL"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_START_HIDDEN);
+  st_handler_field_set_description(field, _("The stream listen URL"));
+  st_handler_add_field(xiph_handler, field);
   
   /* invisible fields */
 
-  st_handler_add_field(handler, st_handler_field_new(FIELD_BITRATE,
-						     _("Bitrate"),
-						     G_TYPE_STRING,
-						     0));
-  st_handler_add_field(handler, st_handler_field_new(FIELD_CHANNELS,
-						     _("Channels"),
-						     G_TYPE_INT,
-						     0));
-  st_handler_add_field(handler, st_handler_field_new(FIELD_SAMPLERATE,
-						     _("Sample rate"),
-						     G_TYPE_INT,
-						     0));
+  st_handler_add_field(xiph_handler, st_handler_field_new(FIELD_BITRATE,
+							  _("Bitrate"),
+							  G_TYPE_STRING,
+							  0));
+  st_handler_add_field(xiph_handler, st_handler_field_new(FIELD_CHANNELS,
+							  _("Channels"),
+							  G_TYPE_INT,
+							  0));
+  st_handler_add_field(xiph_handler, st_handler_field_new(FIELD_SAMPLERATE,
+							  _("Sample rate"),
+							  G_TYPE_INT,
+							  0));
 
-  st_handlers_add(handler);
+  st_handlers_add(xiph_handler);
 }
 
-gboolean
-plugin_init (GError **err)
+static gboolean
+check_api_version (GError **err)
 {
-  if (! st_check_api_version(5, 7))
+  if (st_check_api_version(5, 8))
+    return TRUE;
+  else
     {
       g_set_error(err, 0, 0, _("API version mismatch"));
       return FALSE;
     }
+}
+
+G_MODULE_EXPORT gboolean
+plugin_get_info (STPlugin *plugin, GError **err)
+{
+  GdkPixbuf *pixbuf;
+
+  if (! check_api_version(err))
+    return FALSE;
+
+  xiph_plugin = plugin;
+
+  st_plugin_set_name(plugin, "xiph");
+  st_plugin_set_label(plugin, "Xiph");
+
+  pixbuf = st_pixbuf_new_from_file(UIDIR "/xiph.png");
+  if (pixbuf)
+    {
+      st_plugin_set_icon_from_pixbuf(plugin, pixbuf);
+      g_object_unref(pixbuf);
+    }
+
+  return TRUE;
+}
+
+G_MODULE_EXPORT gboolean
+plugin_init (GError **err)
+{
+  if (! check_api_version(err))
+    return FALSE;
 
   xmlInitParser();
 
   init_handler();
 
-  st_action_register("record-stream", _("Record a stream"), "xterm -hold -e streamripper %q");
+  st_action_register("record-stream", _("Record a stream"), "xterm -e streamripper %q");
   st_action_register("play-stream", _("Listen to a stream"), "xmms %q");
 
   return TRUE;
