@@ -35,7 +35,7 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include "sg-parser.h"
-#include "sg-stack.h"
+#include "sg-util.h"
 
 #define EXPECT_EOF			(1 << 0)
 #define EXPECT_SYMBOL			(1 << 1)
@@ -50,9 +50,6 @@
 static int	sg_parser_expect_mask		(GTokenType	type);
 static void	sg_parser_skip_statement	(SGParser	*parser);
 static gboolean	sg_parser_statement_validate	(SGParserStatement *statement);
-static void	sg_parser_scanner_msg_func	(GScanner	*scanner,
-						 char		*message,
-						 gboolean	error);
 
 /*** implementation **********************************************************/
 
@@ -77,8 +74,7 @@ sg_parser_new (const char *filename, GError **err)
 
   parser->filename = g_strdup(filename);
   parser->scanner = g_scanner_new(NULL);
-  parser->scope_stack = sg_stack_new();
-  parser->message_cb = NULL;
+  parser->scope_stack = NULL;
   
   parser->scanner->input_name = parser->filename;
   parser->scanner->user_data = parser;
@@ -99,21 +95,11 @@ sg_parser_free (SGParser *parser)
 
   g_free(parser->filename);
   g_scanner_destroy(parser->scanner);
-  sg_stack_free(parser->scope_stack);
+  g_slist_free(parser->scope_stack);
 
   g_free(parser);
 }
 
-void
-sg_parser_set_message_cb (SGParser *parser, SGParserMessageCallback *cb)
-{
-  g_return_if_fail(parser != NULL);
-  g_return_if_fail(cb != NULL);
-
-  parser->scanner->msg_handler = sg_parser_scanner_msg_func;
-  parser->message_cb = cb;
-}
-
 void
 sg_parser_define (SGParser *parser, SGParserDefinition *definition)
 {
@@ -163,7 +149,7 @@ sg_parser_get_statement (SGParser *parser)
       if (type == G_TOKEN_EOF)
 	{
 	  if (SG_PARSER_SCOPE(parser) != 0)
-	    sg_parser_warn(parser, _("EOF not in root scope"));
+	    g_scanner_warn(parser->scanner, _("EOF not in root scope"));
 	  
 	  sg_parser_statement_free(statement);
 	  
@@ -186,23 +172,40 @@ sg_parser_get_statement (SGParser *parser)
 	  goto loop;
 	}
 
+#define NO_MINUS		\
+      {				\
+	if (minus)		\
+	  {			\
+	    g_scanner_warn(parser->scanner, _("unexpected minus sign")); \
+	    minus = FALSE;	\
+	  }			\
+      }
+#define MINUS(var)		\
+      {				\
+	if (minus)		\
+	  {			\
+	    (var) = 0 - (var);	\
+	    minus = FALSE;	\
+	  }			\
+      }
+      
       switch (type)
 	{
 	case G_TOKEN_SYMBOL:
+	  NO_MINUS;
+
 	  statement->definition = parser->scanner->value.v_symbol;
 	  
 	  expect = EXPECT_VALUE | EXPECT_LEFT_CURLY | EXPECT_SEMICOLON;
 
-	  if (statement->definition->value_type == G_TYPE_INT)
+	  if (statement->definition->value_type == G_TYPE_INT
+	      || statement->definition->value_type == G_TYPE_DOUBLE)
 	    expect |= EXPECT_MINUS;
-	  
+
 	  break;
 	  
 	case '-':
-	  /*
-	   * FIXME: we should handle the result of a minus sign
-	   * prefixing something else than an integer.
-	   */
+	  NO_MINUS;
 	  minus = TRUE;
 	  break;
 	  
@@ -210,6 +213,8 @@ sg_parser_get_statement (SGParser *parser)
 	  {
 	    gboolean v_gboolean;
 
+	    NO_MINUS;
+
 	    v_gboolean = ! strcmp(parser->scanner->value.v_identifier, "yes");
 
 	    g_value_init(&statement->value, G_TYPE_BOOLEAN);
@@ -225,11 +230,13 @@ sg_parser_get_statement (SGParser *parser)
 	    if (statement->definition && statement->definition->value_type == G_TYPE_BOOLEAN)
 	      {
 		/* support for old configuration files using integers instead of booleans */
+		NO_MINUS;
 		g_value_init(&statement->value, G_TYPE_BOOLEAN);
 		g_value_set_boolean(&statement->value, parser->scanner->value.v_int);
 	      }
 	    else if (statement->definition && statement->definition->value_type == G_TYPE_UINT)
 	      {
+		NO_MINUS;
 		g_value_init(&statement->value, G_TYPE_UINT);
 		g_value_set_uint(&statement->value, parser->scanner->value.v_int);
 	      }
@@ -237,11 +244,7 @@ sg_parser_get_statement (SGParser *parser)
 	      {
 		int v_int = parser->scanner->value.v_int;
 
-		if (minus)
-		  {
-		    v_int = 0 - v_int;
-		    minus = FALSE;
-		  }
+		MINUS(v_int);
 
 		g_value_init(&statement->value, G_TYPE_INT);
 		g_value_set_int(&statement->value, v_int);
@@ -253,17 +256,25 @@ sg_parser_get_statement (SGParser *parser)
 	  }
 
 	case G_TOKEN_FLOAT:
-	  g_value_init(&statement->value, G_TYPE_DOUBLE);
-	  g_value_set_double(&statement->value, parser->scanner->value.v_float);
+	  {
+	    double v_double = parser->scanner->value.v_float;
+
+	    MINUS(v_double);
+
+	    g_value_init(&statement->value, G_TYPE_DOUBLE);
+	    g_value_set_double(&statement->value, v_double);
 	  
-	  expect = EXPECT_LEFT_CURLY | EXPECT_SEMICOLON;
+	    expect = EXPECT_LEFT_CURLY | EXPECT_SEMICOLON;
 	  
-	  break;
+	    break;
+	  }
 
 	case G_TOKEN_STRING:
 	  {
 	    char *unescaped;
 
+	    NO_MINUS;
+
 	    unescaped = *parser->scanner->value.v_string
 	      ? g_strcompress(parser->scanner->value.v_string)
 	      : NULL;
@@ -280,6 +291,8 @@ sg_parser_get_statement (SGParser *parser)
 	case G_TOKEN_LEFT_CURLY:
 	case G_TOKEN_RIGHT_CURLY:
 	case ';':
+	  NO_MINUS;
+
 	  statement->terminator = type;
 	  
 	  break;
@@ -288,6 +301,9 @@ sg_parser_get_statement (SGParser *parser)
 	  break;
 	}
       
+#undef NO_MINUS
+#undef MINUS
+
       if (statement->terminator)
 	{			/* got a complete statement */
 	  if (sg_parser_statement_validate(statement))
@@ -318,13 +334,13 @@ sg_parser_statement_evaluate (SGParserStatement *statement)
       g_return_if_fail(statement->definition != NULL);
 
       scope = statement->definition->id;
-      sg_stack_push(statement->parser->scope_stack, GINT_TO_POINTER(scope));
+      sg_slist_push(&statement->parser->scope_stack, GINT_TO_POINTER(scope));
       g_scanner_set_scope(statement->parser->scanner, scope);
     }
   else if (statement->terminator == '}')
     {
-      sg_stack_pop(statement->parser->scope_stack);
-      scope = GPOINTER_TO_INT(sg_stack_peek(statement->parser->scope_stack));
+      sg_slist_pop(&statement->parser->scope_stack);
+      scope = GPOINTER_TO_INT(sg_slist_peek(statement->parser->scope_stack));
       g_scanner_set_scope(statement->parser->scanner, scope);
     }
 }
@@ -340,24 +356,6 @@ sg_parser_statement_free (SGParserStatement *statement)
   g_free(statement);
 }
 
-void
-sg_parser_warn (SGParser *parser, const char *format, ...)
-{
-  va_list args;
-  char *message;
-
-  g_return_if_fail(parser != NULL);
-  g_return_if_fail(format != NULL);
-
-  va_start(args, format);
-  message = g_strdup_vprintf(format, args);
-  va_end(args);
-
-  g_scanner_warn(parser->scanner, message);
-
-  g_free(message);
-}
-
 static int
 sg_parser_expect_mask (GTokenType type)
 {
@@ -401,42 +399,29 @@ sg_parser_statement_validate (SGParserStatement *statement)
       if ((statement->definition->scope && statement->terminator != '{')
 	  || (! statement->definition->scope && statement->terminator != ';'))
 	{
-	  sg_parser_warn(statement->parser, _("wrong terminator"));
+	  g_scanner_warn(statement->parser->scanner, _("wrong terminator"));
 	  return FALSE;
 	}
       else if (statement->definition->value_type == G_TYPE_NONE)
 	{
 	  if (G_IS_VALUE(&statement->value))
 	    {
-	      sg_parser_warn(statement->parser, _("value encountered, but none expected"));
+	      g_scanner_warn(statement->parser->scanner, _("value encountered, but none expected"));
 	      return FALSE;
 	    }
 	}
       else if (! G_VALUE_HOLDS(&statement->value, statement->definition->value_type))
 	{
-	  sg_parser_warn(statement->parser, _("incorrect value type"));
+	  g_scanner_warn(statement->parser->scanner, _("incorrect value type"));
 	  return FALSE;
 	}
     }
 
-  if (statement->terminator == '}' && ! sg_stack_peek(statement->parser->scope_stack))
+  if (statement->terminator == '}' && ! sg_slist_peek(statement->parser->scope_stack))
     {
-      sg_parser_warn(statement->parser, _("unmatched start of scope"));
+      g_scanner_warn(statement->parser->scanner, _("unmatched start of scope"));
       return FALSE;
     }
 
   return TRUE;
 }
-
-static void
-sg_parser_scanner_msg_func (GScanner *scanner,
-			    char *message,
-			    gboolean error)
-{
-  SGParser *parser;
-
-  parser = scanner->user_data;
-  g_return_if_fail(parser->message_cb != NULL);
-
-  parser->message_cb(parser, message);
-}
