@@ -32,6 +32,10 @@
 #ifndef _SG_UTIL_H
 #define _SG_UTIL_H
 
+#include <glib.h>
+
+#ifndef ST_REGRESSION_TEST
+
 #include <stdarg.h>
 #include <glib-object.h>
 
@@ -40,13 +44,13 @@ for ((var) = (head);				\
      (var);					\
      (var) = (var)->next)
 
+#define SG_SET_FLAGS(flags, set)	((flags) |= (set))
+#define SG_UNSET_FLAGS(flags, set)	((flags) &= ~(set))
+
 char		*sg_str_sentencize		(const char	*str);
 void		sg_pointers_free		(GSList		*pointers);
 void		sg_objects_free			(GSList		*objects);
 void		sg_objects_free_node		(GNode		*objects);
-GSList		*sg_objects_copy		(GSList		*objects);
-gboolean	sg_value_equal			(const GValue	*value1,
-						 const GValue	*value2);
 int		sg_utf8_strcmp			(const char	*s1,
 						 const char	*s2);
 gboolean	sg_utf8_strcontains		(const char	*big,
@@ -62,10 +66,21 @@ void		sg_string_append_printf_escaped	(GString	*string,
 void		sg_string_append_vprintf_escaped (GString	*string,
 						  const char	*format,
 						  va_list	args);
-void		sg_object_link_int		(GObject	*object,
-						 const char	*property_name,
-						 int		*var);
 gboolean	sg_str_slist_contains		(GSList		*list,
 						 const char	*str);
+GValue		*sg_value_new			(GType		type);
+void		sg_value_free			(GValue		*value);
+void		sg_slist_push			(GSList		**list,
+						 gpointer	data);
+gpointer	sg_slist_peek			(GSList		*list);
+gpointer	sg_slist_pop			(GSList		**list);
+gboolean	sg_double_equal			(double		a,
+						 double		b);
+
+#endif /* ! ST_REGRESSION_TEST */
+
+gboolean	sg_ascii_validate		(const char	*str);
+char		*sg_ascii_strcasestr		(const char	*big,
+						 const char	*little);
 
 #endif /* _SG_UTIL_H */
