@@ -62,18 +62,15 @@ typedef struct
 GType		sgtk_auth_dialog_get_type (void);
 
 GtkWidget	*sgtk_auth_dialog_new     (GtkWindow		*parent,
+					   const char		*default_name,
+					   const char		*default_password,
 					   const char		*primary,
 					   const char		*format,
-					   ...) G_GNUC_PRINTF(3, 4);
+					   ...) G_GNUC_PRINTF(5, 6);
 
 void sgtk_auth_dialog_set_name_sensitive  (sGtkAuthDialog	*dialog,
 					   gboolean		sensitive);
 
-void sgtk_auth_dialog_set_name		  (sGtkAuthDialog	*dialog,
-					   const char		*name);
-void sgtk_auth_dialog_set_password	  (sGtkAuthDialog	*dialog,
-					   const char		*password);
-
 const char *sgtk_auth_dialog_get_name     (sGtkAuthDialog	*dialog);
 const char *sgtk_auth_dialog_get_password (sGtkAuthDialog	*dialog);
 
