@@ -94,13 +94,15 @@ sgtk_blinking_image_class_init (sGtkBlinkingImageClass *class)
 
   parent_class = g_type_class_peek_parent(class);
 
+  g_type_class_add_private(class, sizeof(sGtkBlinkingImagePrivate));
+
   object_class->finalize = sgtk_blinking_image_finalize;
 }
 
 static void
 sgtk_blinking_image_init (sGtkBlinkingImage *image)
 {
-  image->priv = g_new0(sGtkBlinkingImagePrivate, 1);
+  image->priv = G_TYPE_INSTANCE_GET_PRIVATE(image, SGTK_TYPE_BLINKING_IMAGE, sGtkBlinkingImagePrivate);
   image->priv->is_on = TRUE;
 }
 
@@ -109,15 +111,16 @@ sgtk_blinking_image_finalize (GObject *object)
 {
   sGtkBlinkingImage *image = SGTK_BLINKING_IMAGE(object);
 
+  if (image->priv->blinking)
+    g_source_remove(image->priv->timeout_id);
+    
   if (image->priv->on_pixbuf)
     {
       g_object_unref(image->priv->on_pixbuf);
       g_object_unref(image->priv->off_pixbuf);
     }
 
-  g_free(image->priv);
-
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  parent_class->finalize(object);
 }
 
 static void
@@ -159,8 +162,7 @@ sgtk_blinking_image_set_from_pixbuf (sGtkBlinkingImage *image,
   width = gdk_pixbuf_get_width(pixbuf);
   height = gdk_pixbuf_get_height(pixbuf);
 
-  g_object_ref(pixbuf);
-  image->priv->on_pixbuf = pixbuf;
+  image->priv->on_pixbuf = g_object_ref(pixbuf);
   image->priv->off_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, width, height);
   gdk_pixbuf_fill(image->priv->off_pixbuf, 0); /* fill with transparency */
 
