@@ -36,8 +36,6 @@
 #include "sgtk-dialog.h"
 #include "sgtk-hig.h"
 #include "st-about-dialog.h"
-#include "st-plugins.h"
-#include "st-handlers.h"
 #include "st-link.h"
 #include "st-settings.h"
 #include "sg-util.h"
@@ -48,8 +46,6 @@
 
 static void      st_about_dialog_init		(STAboutDialog *dialog);
 
-static char      *st_about_dialog_make_info	(void);
-
 /*** implementation **********************************************************/
 
 GType
@@ -88,11 +84,10 @@ st_about_dialog_init (STAboutDialog *dialog)
   GtkWidget *logo;
   GtkWidget *version;
   GtkWidget *link;
-  GtkWidget *info;
   char *tmp;
 
   close = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
-  gtk_window_set_default(GTK_WINDOW(dialog), close);
+  gtk_widget_grab_focus(close);
 
   gtk_window_set_title(GTK_WINDOW(dialog), _("About streamtuner"));
   gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
@@ -116,16 +111,6 @@ st_about_dialog_init (STAboutDialog *dialog)
   st_link_set_text(ST_LINK(link), ST_HOME);
   st_link_set_uri(ST_LINK(link), ST_HOME);
 
-  /* create the dynamic info */
-
-  tmp = st_about_dialog_make_info();
-  info = gtk_label_new(tmp);
-  g_free(tmp);
-
-  gtk_label_set_selectable(GTK_LABEL(info), TRUE);
-  gtk_label_set_use_markup(GTK_LABEL(info), TRUE);
-  gtk_label_set_justify(GTK_LABEL(info), GTK_JUSTIFY_CENTER);
-
   /* pack the widgets */
 
   box = gtk_vbox_new(FALSE, 0);
@@ -133,50 +118,11 @@ st_about_dialog_init (STAboutDialog *dialog)
   gtk_box_pack_start(GTK_BOX(box), logo, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(box), version, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(box), link, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(box), info, FALSE, FALSE, 0);
 
   gtk_container_add(GTK_CONTAINER(SGTK_DIALOG(dialog)->contents), box);
   gtk_widget_show_all(box);
 }
 
-static char *
-st_about_dialog_make_info (void)
-{
-  GString *string;
-  GSList *l;
-
-  string = g_string_new(NULL);
-
-  if (st_plugins_loaded_list)
-    {
-      g_string_append(string, _("\n<span weight=\"bold\">Loaded plugins:</span>\n"));
-
-      SG_LIST_FOREACH(l, st_plugins_loaded_list)
-	{
-	  char *filename = l->data;
-	  sg_string_append_printf_escaped(string, "\n%s", filename);
-	}
-    }
-  else
-    g_string_append(string, _("\nThere are no loaded plugins."));
-  
-  g_string_append(string, _("\n\n<span weight=\"bold\">Registered handlers:</span>\n"));
-      
-  SG_LIST_FOREACH(l, st_handlers_list)
-    {
-      STHandler *handler = l->data;
-      const char *copyright;
-
-      sg_string_append_printf_escaped(string, "\n%s", st_handler_get_label(handler));
-	  
-      copyright = st_handler_get_copyright(handler);
-      if (copyright)
-	sg_string_append_printf_escaped(string, " - %s", copyright);
-    }
-      
-  return g_string_free(string, FALSE);
-}
-
 GtkWidget *
 st_about_dialog_new (GtkWindow *parent)
 {
