@@ -39,58 +39,79 @@
 /*** variable declarations ***************************************************/
 
 static GHashTable *actions;
-G_LOCK_DEFINE_STATIC(actions);
 
 /*** function declarations ***************************************************/
 
+static void st_action_register_real (const char *id,
+				     const char *label,
+				     const char *command);
+
 static void st_action_list_cb (gpointer key,
 			       gpointer value,
 			       gpointer user_data);
 
-static STAction *st_action_new (const char *label, const char *command);
-static void st_action_free (STAction *action);
-
 static char *st_action_subst (const char *command, const char *uri);
 
 /*** API implementation ******************************************************/
 
+/**
+ * st_action_register:
+ * @id: an unique id for the action.
+ * @label: a human-readable label for the action.
+ * @command: the default command for the action, or %NULL.
+ *
+ * Registers a new action. The action will appear in the applications
+ * page of the streamtuner preferences, and the user will be able to
+ * modify its associated command.
+ **/
 void
 st_action_register (const char *id, const char *label, const char *command)
 {
   g_return_if_fail(id != NULL);
   g_return_if_fail(label != NULL);
-  g_return_if_fail(command != NULL);
 
-  G_LOCK(actions);
-  if (! g_hash_table_lookup(actions, id))
-    {				/* don't register an action twice */
-      STAction *action;
-
-      action = st_action_new(label, command);
-      g_hash_table_insert(actions, g_strdup(id), action);
-    }
-  G_UNLOCK(actions);
+  st_action_register_real(id, label, command);
 }
 
+/**
+ * st_action_run:
+ * @id: the id of the action to run.
+ * @uri: the URI to run the action on.
+ * @err: a location to report errors, or %NULL.
+ *
+ * Runs action @id with @uri as parameter. The action must have been
+ * previously registered with st_action_register().
+ *
+ * Return value: %TRUE on success, %FALSE on failure.
+ **/
 gboolean
 st_action_run (const char *id, const char *uri, GError **err)
 {
   STAction *action;
-  char *subst;
   gboolean status;
+  char *subst;
 
   g_return_val_if_fail(id != NULL, FALSE);
   g_return_val_if_fail(uri != NULL, FALSE);
 
-  G_LOCK(actions);
   action = g_hash_table_lookup(actions, id);
-  G_UNLOCK(actions);
-
   g_return_val_if_fail(action != NULL, FALSE);
+  g_return_val_if_fail(action->label != NULL, FALSE);
 
-  subst = st_action_subst(action->command, uri);
-  status = g_spawn_command_line_async(subst, err);
-  g_free(subst);
+  g_mutex_lock(action->mutex);
+  subst = action->command ? st_action_subst(action->command, uri) : NULL;
+  g_mutex_unlock(action->mutex);
+
+  if (subst)
+    {
+      status = g_spawn_command_line_async(subst, err);
+      g_free(subst);
+    }
+  else
+    {
+      status = FALSE;
+      g_set_error(err, 0, 0, _("no command is set for \"%s\""), action->label);
+    }
 
   return status;
 }
@@ -100,46 +121,49 @@ st_action_run (const char *id, const char *uri, GError **err)
 void
 st_action_init (void)
 {
-  actions = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify) st_action_free);
+  actions = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
 
   st_action_register("view-web", _("Open a web page"), "epiphany %q");
   st_action_register("view-ghelp", _("Open a GNOME help document"), "yelp %q");
 }
 
-gboolean
-st_action_exists (const char *id)
+static void
+st_action_register_real (const char *id,
+			 const char *label,
+			 const char *command)
 {
-  gboolean exists;
+  g_return_if_fail(id != NULL);
 
-  g_return_val_if_fail(id != NULL, FALSE);
+  if (! g_hash_table_lookup(actions, id))
+    {				/* don't register an action twice */
+      STAction *action;
 
-  G_LOCK(actions);
-  exists = g_hash_table_lookup(actions, id) != NULL;
-  G_UNLOCK(actions);
+      action = g_new(STAction, 1);
+      action->mutex = g_mutex_new();
+      action->label = g_strdup(label);
+      action->command = g_strdup(command);
 
-  return exists;
+      g_hash_table_insert(actions, g_strdup(id), action);
+    }
 }
 
 void
 st_action_associate (const char *id, const char *command)
 {
   STAction *action;
-  gboolean status;
 
   g_return_if_fail(id != NULL);
-  g_return_if_fail(command != NULL);
 
-  G_LOCK(actions);
   action = g_hash_table_lookup(actions, id);
   if (action)
     {
+      g_mutex_lock(action->mutex);
       g_free(action->command);
       action->command = g_strdup(command);
+      g_mutex_unlock(action->mutex);
     }
-  status = action != NULL;
-  G_UNLOCK(actions);
-
-  g_return_if_fail(status == TRUE);
+  else
+    st_action_register_real(id, NULL, command);	/* dummy action */
 }
 
 GSList *
@@ -147,9 +171,7 @@ st_action_list (void)
 {
   GSList *list = NULL;
 
-  G_LOCK(actions);
   g_hash_table_foreach(actions, st_action_list_cb, &list);
-  G_UNLOCK(actions);
 
   return list;
 }
@@ -165,40 +187,9 @@ st_action_list_cb (gpointer key, gpointer value, gpointer user_data)
 STAction *
 st_action_get (const char *id)
 {
-  STAction *action;
-
   g_return_val_if_fail(id != NULL, NULL);
 
-  G_LOCK(actions);
-  action = g_hash_table_lookup(actions, id);
-  G_UNLOCK(actions);
-
-  return action;
-}
-
-static STAction *
-st_action_new (const char *label, const char *command)
-{
-  STAction *action;
-
-  g_return_val_if_fail(label != NULL, NULL);
-  g_return_val_if_fail(command != NULL, NULL);
-
-  action = g_new(STAction, 1);
-  action->label = g_strdup(label);
-  action->command = g_strdup(command);
-
-  return action;
-}
-
-static void
-st_action_free (STAction *action)
-{
-  g_return_if_fail(action != NULL);
-
-  g_free(action->label);
-  g_free(action->command);
-  g_free(action);
+  return g_hash_table_lookup(actions, id);
 }
 
 static char *
@@ -255,7 +246,8 @@ st_action_convert_old_style_command (const char *command)
   int i;
   gboolean quoted = FALSE;
 
-  g_return_val_if_fail(command != NULL, NULL);
+  if (! command)
+    return NULL;
 
   string = g_string_new(NULL);
   
