@@ -37,12 +37,12 @@
 
 typedef struct
 {
-  char	*label;
-  char	*command;
+  GMutex	*mutex;
+  char		*label;
+  char		*command;
 } STAction;
 
 void		st_action_init		(void);
-gboolean	st_action_exists	(const char	*id);
 void		st_action_associate	(const char	*id,
 					 const char	*command);
 GSList		*st_action_list		(void);
