@@ -33,13 +33,14 @@
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include "art/bookmarks.h"
 #include "sg-util.h"
 #include "sgtk-util.h"
 #include "st-handler.h"
 #include "st-handlers.h"
 #include "st-handler-field.h"
 #include "st-stream-store.h"
+#include "st-util.h"
+#include "st-util-api.h"
 
 /*** cpp *********************************************************************/
 
@@ -52,12 +53,14 @@ typedef struct
 {
   STStream	stream;
 
+  char		*handler_name;
   STHandler	*handler;
   STStreamBag	*stream_bag;
 
   GdkPixbuf	*handler_icon;
   char		*name;
   char		*genre;
+  char		*description;
 } STBookmarksStream;
 
 enum {
@@ -67,6 +70,9 @@ enum {
   FIELD_HANDLER_LABEL,
   FIELD_NAME,
   FIELD_GENRE,
+  FIELD_DESCRIPTION,
+  FIELD_HOMEPAGE,
+  FIELD_URI_LIST
 };
 
 typedef struct
@@ -146,12 +152,8 @@ st_bookmarks_stream_set_handler (STBookmarksStream *stream, STHandler *handler)
       GdkPixbuf *pixbuf;
 
       pixbuf = st_handler_get_pixbuf(stream->handler);
-
       if (pixbuf)
-	{
-	  stream->handler_icon = sgtk_pixbuf_scale(pixbuf, GTK_ICON_SIZE_MENU);
-	  g_object_unref(pixbuf);
-	}
+	stream->handler_icon = sgtk_pixbuf_scale(pixbuf, GTK_ICON_SIZE_MENU);
     }
 }
 
@@ -172,7 +174,7 @@ st_bookmarks_stream_field_get_cb (STBookmarksStream *stream,
 	GValueArray *value_array;
 
 	value_array = st_bookmarks_value_array_new(stream->stream_bag);
-	g_value_set_boxed_take_ownership(value, value_array);
+	g_value_take_boxed(value, value_array);
 
 	break;
       }
@@ -182,7 +184,7 @@ st_bookmarks_stream_field_get_cb (STBookmarksStream *stream,
       break;
       
     case FIELD_HANDLER_LABEL:
-      g_value_set_string(value, stream->handler ? st_handler_get_label(stream->handler) : _("None"));
+      g_value_set_string(value, stream->handler ? st_handler_get_label(stream->handler) : stream->handler_name);
       break;
 
     case FIELD_NAME:
@@ -193,6 +195,20 @@ st_bookmarks_stream_field_get_cb (STBookmarksStream *stream,
       g_value_set_string(value, stream->genre);
       break;
 
+    case FIELD_DESCRIPTION:
+      g_value_set_string(value, stream->description);
+      break;
+
+    case FIELD_HOMEPAGE:
+      if (stream->stream_bag)
+	st_stream_bag_get_stock_field_initialized(stream->stream_bag, ST_HANDLER_STOCK_FIELD_HOMEPAGE, value);
+      break;
+
+    case FIELD_URI_LIST:
+      if (stream->stream_bag)
+	st_stream_bag_get_stock_field_initialized(stream->stream_bag, ST_HANDLER_STOCK_FIELD_URI_LIST, value);
+      break;
+
     default:
       g_return_if_reached();
     }
@@ -207,7 +223,8 @@ st_bookmarks_stream_field_set_cb (STBookmarksStream *stream,
   switch (field->id)
     {
     case FIELD_HANDLER_NAME:
-      st_bookmarks_stream_set_handler(stream, st_handlers_find_by_name(g_value_get_string(value)));
+      stream->handler_name = g_value_dup_string(value);
+      st_bookmarks_stream_set_handler(stream, st_handlers_find_by_name(stream->handler_name));
       break;
 
     case FIELD_STREAM_BAG:
@@ -221,15 +238,20 @@ st_bookmarks_stream_field_set_cb (STBookmarksStream *stream,
       break;
 
     case FIELD_NAME:
-      g_free(stream->name);	/* field is editable */
+      g_free(stream->name);		/* field is editable */
       stream->name = g_value_dup_string(value);
       break;
 
     case FIELD_GENRE:
-      g_free(stream->genre);	/* field is editable */
+      g_free(stream->genre);		/* field is editable */
       stream->genre = g_value_dup_string(value);
       break;
 
+    case FIELD_DESCRIPTION:
+      g_free(stream->description);	/* field is editable */
+      stream->description = g_value_dup_string(value);
+      break;
+
     default:
       g_return_if_reached();
     }
@@ -251,15 +273,19 @@ st_bookmarks_stream_stock_field_get_cb (STBookmarksStream *stream,
       g_value_set_string(value, stream->genre);
       break;
 
-    default:
-      /*
-       * This can not happen.
-       *
-       * However, handlers not shipped with streamtuner MUST ignore
-       * unknown stock fields, to stay compatible with future versions
-       * of streamtuner.
-       */
-      g_return_if_reached();
+    case ST_HANDLER_STOCK_FIELD_DESCRIPTION:
+      g_value_set_string(value, stream->description);
+      break;
+
+    case ST_HANDLER_STOCK_FIELD_HOMEPAGE:
+      if (stream->stream_bag)
+	st_stream_bag_get_stock_field_initialized(stream->stream_bag, ST_HANDLER_STOCK_FIELD_HOMEPAGE, value);
+      break;
+
+    case ST_HANDLER_STOCK_FIELD_URI_LIST:
+      if (stream->stream_bag)
+	st_stream_bag_get_stock_field_initialized(stream->stream_bag, ST_HANDLER_STOCK_FIELD_URI_LIST, value);
+      break;
     }
 }
 
@@ -376,7 +402,10 @@ st_bookmarks_stream_free_cb (STBookmarksStream *stream, gpointer data)
   if (stream->handler_icon)
     g_object_unref(stream->handler_icon);
 
+  g_free(stream->handler_name);
   g_free(stream->name);
+  g_free(stream->genre);
+  g_free(stream->description);
 
   st_stream_free((STStream *) stream);
 }
@@ -410,7 +439,7 @@ st_bookmarks_stream_tune_in_cb (STBookmarksStream *stream,
     }
   else
     {
-      g_set_error(err, 0, 0, _("bookmark contains no stream"));
+      g_set_error(err, 0, 0, _("the %s handler is not available"), stream->handler_name);
       return FALSE;
     }
 }
@@ -433,7 +462,7 @@ st_bookmarks_stream_record_cb (STBookmarksStream *stream,
     }
   else
     {
-      g_set_error(err, 0, 0, _("bookmark contains no stream"));
+      g_set_error(err, 0, 0, _("the %s handler is not available"), stream->handler_name);
       return FALSE;
     }
 }
@@ -456,7 +485,7 @@ st_bookmarks_stream_browse_cb (STBookmarksStream *stream,
     }
   else
     {
-      g_set_error(err, 0, 0, _("bookmark contains no stream"));
+      g_set_error(err, 0, 0, _("the %s handler is not available"), stream->handler_name);
       return FALSE;
     }
 }
@@ -464,16 +493,24 @@ st_bookmarks_stream_browse_cb (STBookmarksStream *stream,
 void
 st_bookmarks_init (void)
 {
+  GdkPixbuf *pixbuf;
   STStreamStore *bookmarks;
+  STHandlerField *field;
 
   st_bookmarks_handler = st_handler_new("bookmarks");
 
   st_handler_set_label(st_bookmarks_handler, _("Bookmarks"));
   st_handler_set_copyright(st_bookmarks_handler, "Copyright \302\251 2004 Jean-Yves Lefort");
   st_handler_set_description(st_bookmarks_handler, _("Favourite Streams"));
-  st_handler_set_icon_from_inline(st_bookmarks_handler, sizeof(art_bookmarks), art_bookmarks);
   st_handler_set_flags(st_bookmarks_handler, ST_HANDLER_NO_CATEGORIES);
-  
+
+  pixbuf = st_pixbuf_new_from_file(ST_PNG("bookmarks"));
+  if (pixbuf)
+    {
+      st_handler_set_icon_from_pixbuf(st_bookmarks_handler, pixbuf);
+      g_object_unref(pixbuf);
+    }
+
   st_handler_bind(st_bookmarks_handler, ST_HANDLER_EVENT_STREAM_NEW, st_bookmarks_stream_new_cb, NULL);
   st_handler_bind(st_bookmarks_handler, ST_HANDLER_EVENT_STREAM_FIELD_GET, st_bookmarks_stream_field_get_cb, NULL);
   st_handler_bind(st_bookmarks_handler, ST_HANDLER_EVENT_STREAM_FIELD_SET, st_bookmarks_stream_field_set_cb, NULL);
@@ -491,11 +528,13 @@ st_bookmarks_init (void)
 					    _("STHandler"),
 					    G_TYPE_STRING,
 					    0));
+
   st_handler_add_field(st_bookmarks_handler,
 		       st_handler_field_new(FIELD_STREAM_BAG,
 					    _("STStreamBag"),
 					    G_TYPE_VALUE_ARRAY,
 					    0));
+
   st_handler_add_field(st_bookmarks_handler,
 		       st_handler_field_new(FIELD_HANDLER_ICON,
 					    _("Handler icon"),
@@ -503,24 +542,56 @@ st_bookmarks_init (void)
 					    ST_HANDLER_FIELD_VISIBLE
 					    | ST_HANDLER_FIELD_VOLATILE
 					    | ST_HANDLER_FIELD_NO_DEDICATED_COLUMN));
-  st_handler_add_field(st_bookmarks_handler,
-		       st_handler_field_new(FIELD_HANDLER_LABEL,
-					    _("Handler"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_VOLATILE));
-  st_handler_add_field(st_bookmarks_handler,
-		       st_handler_field_new(FIELD_NAME,
-					    _("Name"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE));
-  st_handler_add_field(st_bookmarks_handler,
-		       st_handler_field_new(FIELD_GENRE,
-					    _("Genre"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE));
+  
+  field = st_handler_field_new(FIELD_HANDLER_LABEL,
+			       _("Handler"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_VOLATILE);
+  st_handler_field_set_description(field, _("The stream handler"));
+  st_handler_add_field(st_bookmarks_handler, field);
+
+  field = st_handler_field_new(FIELD_NAME,
+			       _("Name"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE);
+  st_handler_field_set_description(field, _("The stream name"));
+  st_handler_add_field(st_bookmarks_handler, field);
+
+  field = st_handler_field_new(FIELD_GENRE,
+			       _("Genre"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE);
+  st_handler_field_set_description(field, _("The stream genre"));
+  st_handler_add_field(st_bookmarks_handler, field);
+
+  field = st_handler_field_new(FIELD_DESCRIPTION,
+			       _("Description"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE);
+  st_handler_field_set_description(field, _("The stream description"));
+  st_handler_add_field(st_bookmarks_handler, field);
+
+  field = st_handler_field_new(FIELD_HOMEPAGE,
+			       _("Homepage"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_START_HIDDEN
+			       | ST_HANDLER_FIELD_VOLATILE);
+  st_handler_field_set_description(field, _("The stream homepage URL"));
+  st_handler_add_field(st_bookmarks_handler, field);
+
+  field = st_handler_field_new(FIELD_URI_LIST,
+			       _("URI list"),
+			       G_TYPE_VALUE_ARRAY,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_START_HIDDEN
+			       | ST_HANDLER_FIELD_VOLATILE);
+  st_handler_field_set_description(field, _("The stream listen URI list"));
+  st_handler_add_field(st_bookmarks_handler, field);
 
   st_handlers_add(st_bookmarks_handler);
 
@@ -582,6 +653,10 @@ st_bookmarks_add (STStreamBag *stream_bag)
       st_stream_bag_get_stock_field(stream_bag, ST_HANDLER_STOCK_FIELD_GENRE, &value);
       ST_BOOKMARKS_STREAM(bookmark)->genre = g_value_dup_string(&value);
       g_value_unset(&value);
+
+      st_stream_bag_get_stock_field(stream_bag, ST_HANDLER_STOCK_FIELD_DESCRIPTION, &value);
+      ST_BOOKMARKS_STREAM(bookmark)->description = g_value_dup_string(&value);
+      g_value_unset(&value);
     }
   else
     {
@@ -616,10 +691,10 @@ st_bookmarks_add (STStreamBag *stream_bag)
       ST_BOOKMARKS_STREAM(bookmark)->name = g_string_free(name, FALSE);
     }
   
-  g_object_ref(stream_bag);
-  ST_BOOKMARKS_STREAM(bookmark)->stream_bag = stream_bag;
+  ST_BOOKMARKS_STREAM(bookmark)->stream_bag = g_object_ref(stream_bag);
 
   st_stream_store_append(bookmarks, bookmark);
+  st_stream_store_touch(bookmarks);
 
   g_object_unref(bookmark);
   g_object_unref(bookmarks);
