@@ -29,19 +29,26 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include "config.h"
 #include <string.h>
+#include <glib/gi18n.h>
 #include <glib-object.h>
+#include "sg-util.h"
 #include "st-category-bag.h"
 #include "st-category-view.h"
 #include "st-handler.h"
+#include "st-statusbar.h"
 
 /*** type definitions ********************************************************/
 
+enum {
+  PROP_0,
+  PROP_TASK_THREAD
+};
+
 struct _STCategoryBagPrivate
 {
-  GMutex		*mutex;
-
-  unsigned int		flags;
+  STThread		*task_thread;
 };
 
 /*** variable declarations ***************************************************/
@@ -53,8 +60,14 @@ static GObjectClass *parent_class = NULL;
 static void st_category_bag_class_init (STCategoryBagClass *class);
 static void st_category_bag_init (STCategoryBag *bag);
 static void st_category_bag_finalize (GObject *object);
-
-static void st_category_bag_set_or_unset_remarkable (STCategoryBag *bag);
+static void st_category_bag_set_property (GObject *object,
+					  unsigned int prop_id,
+					  const GValue *value,
+					  GParamSpec *pspec);
+static void st_category_bag_get_property (GObject *object,
+					  unsigned int prop_id,
+					  GValue *value,
+					  GParamSpec *pspec);
 
 /*** implementation **********************************************************/
 
@@ -93,7 +106,18 @@ st_category_bag_class_init (STCategoryBagClass *class)
 
   parent_class = g_type_class_peek_parent(class);
 
+  g_type_class_add_private(class, sizeof(STCategoryBagPrivate));
+
   object_class->finalize = st_category_bag_finalize;
+  object_class->set_property = st_category_bag_set_property;
+  object_class->get_property = st_category_bag_get_property;
+
+  g_object_class_install_property(object_class,
+				  PROP_TASK_THREAD,
+				  g_param_spec_pointer("task-thread",
+						       NULL,
+						       NULL,
+						       G_PARAM_READWRITE));
 }
 
 static void
@@ -102,9 +126,14 @@ st_category_bag_init (STCategoryBag *bag)
   bag->category = NULL;
   bag->handler = NULL;
   memset(&bag->iter, 0, sizeof(bag->iter));
+
+  bag->statusbar = st_statusbar_new();
+  g_object_ref(bag->statusbar);
+
+  bag->statusbar_context_count = gtk_statusbar_get_context_id(GTK_STATUSBAR(bag->statusbar), _("Count messages"));
+  bag->statusbar_context_task = gtk_statusbar_get_context_id(GTK_STATUSBAR(bag->statusbar), _("Task messages"));
   
-  bag->priv = g_new0(STCategoryBagPrivate, 1);
-  bag->priv->mutex = g_mutex_new();
+  bag->priv = G_TYPE_INSTANCE_GET_PRIVATE(bag, ST_TYPE_CATEGORY_BAG, STCategoryBagPrivate);
 }
 
 static void
@@ -112,11 +141,52 @@ st_category_bag_finalize (GObject *object)
 {
   STCategoryBag *bag = ST_CATEGORY_BAG(object);
 
-  st_handler_event_category_free(bag->handler, bag->category);
-  g_mutex_free(bag->priv->mutex);
-  g_free(bag->priv);
+  if (bag->category)
+    st_handler_event_category_free(bag->handler, bag->category);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  g_object_unref(bag->statusbar);
+
+  parent_class->finalize(object);
+}
+
+static void
+st_category_bag_set_property (GObject *object,
+			      unsigned int prop_id,
+			      const GValue *value,
+			      GParamSpec *pspec)
+{
+  STCategoryBag *bag = ST_CATEGORY_BAG(object);
+
+  switch (prop_id)
+    {
+    case PROP_TASK_THREAD:
+      bag->priv->task_thread = g_value_get_pointer(value);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+st_category_bag_get_property (GObject *object,
+			      unsigned int prop_id,
+			      GValue *value,
+			      GParamSpec *pspec)
+{
+  STCategoryBag *bag = ST_CATEGORY_BAG(object);
+
+  switch (prop_id)
+    {
+    case PROP_TASK_THREAD:
+      g_value_set_pointer(value, bag->priv->task_thread);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+      break;
+    }
 }
 
 STCategoryBag *
@@ -143,42 +213,6 @@ st_category_bag_new_from_category (STHandler *handler, STCategory *category)
   return bag;
 }
 
-void
-st_category_bag_set_flags (STCategoryBag *bag, unsigned int flags)
-{
-  g_return_if_fail(ST_IS_CATEGORY_BAG(bag));
-
-  g_mutex_lock(bag->priv->mutex);
-  bag->priv->flags |= flags;
-  st_category_bag_set_or_unset_remarkable(bag);
-  g_mutex_unlock(bag->priv->mutex);
-}
-
-void
-st_category_bag_unset_flags (STCategoryBag *bag, unsigned int flags)
-{
-  g_return_if_fail(ST_IS_CATEGORY_BAG(bag));
-
-  g_mutex_lock(bag->priv->mutex);
-  bag->priv->flags &= ~flags;
-  st_category_bag_set_or_unset_remarkable(bag);
-  g_mutex_unlock(bag->priv->mutex);
-}
-
-unsigned int
-st_category_bag_get_flags (STCategoryBag *bag)
-{
-  unsigned int flags;
-
-  g_return_val_if_fail(ST_IS_CATEGORY_BAG(bag), 0);
-
-  g_mutex_lock(bag->priv->mutex);
-  flags = bag->priv->flags;
-  g_mutex_unlock(bag->priv->mutex);
-
-  return flags;
-}
-
 gboolean
 st_category_bag_apply_url_cb (STCategoryBag *bag)
 {
@@ -186,9 +220,6 @@ st_category_bag_apply_url_cb (STCategoryBag *bag)
 
   if (ST_CATEGORY(bag)->url_cb)
     {
-      g_free(ST_CATEGORY(bag)->url_postfix);
-      ST_CATEGORY(bag)->url_postfix = NULL;
-
       if (! ST_CATEGORY(bag)->url_cb(ST_CATEGORY(bag)))
 	return FALSE;
     }
@@ -196,20 +227,6 @@ st_category_bag_apply_url_cb (STCategoryBag *bag)
   return TRUE;
 }
 
-/*
- * Must be called with mutex held.
- */
-static void
-st_category_bag_set_or_unset_remarkable (STCategoryBag *bag)
-{
-  g_return_if_fail(ST_IS_CATEGORY_BAG(bag));
-
-  if (bag->priv->flags != 0)
-    st_handler_add_remarkable_category(bag->handler, bag);
-  else
-    st_handler_remove_remarkable_category(bag->handler, bag);
-}
-
 void
 st_category_bag_select (STCategoryBag *bag)
 {
@@ -220,9 +237,25 @@ st_category_bag_select (STCategoryBag *bag)
   selected_bag = st_handler_get_selected_category(bag->handler);
   if (selected_bag)
     {
-      st_category_bag_unset_flags(selected_bag, ST_CATEGORY_BAG_SELECTED);
+      SG_UNSET_FLAGS(selected_bag->flags, ST_CATEGORY_BAG_SELECTED);
       g_object_unref(selected_bag);
     }
 
-  st_category_bag_set_flags(bag, ST_CATEGORY_BAG_SELECTED);
+  SG_SET_FLAGS(bag->flags, ST_CATEGORY_BAG_SELECTED);
+}
+
+void
+st_category_bag_set_task_thread (STCategoryBag *bag, STThread *task_thread)
+{
+  g_return_if_fail(ST_IS_CATEGORY_BAG(bag));
+
+  g_object_set(bag, "task-thread", task_thread, NULL);
+}
+
+STThread *
+st_category_bag_get_task_thread (STCategoryBag *bag)
+{
+  g_return_val_if_fail(ST_IS_CATEGORY_BAG(bag), NULL);
+
+  return bag->priv->task_thread;
 }
