@@ -34,6 +34,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include "sg-util.h"
+#include "sgtk-cell-renderer-pixbuf.h"
 #include "st-category-view.h"
 #include "st-stock.h"
 #include "st-category-bag.h"
@@ -50,7 +51,6 @@ enum {
 struct _STCategoryViewPrivate
 {
   unsigned int		selection_changed_hid;
-  GtkCellRenderer	*pixbuf_renderer;
 };
 
 /*** variable declarations ***************************************************/
@@ -62,8 +62,6 @@ static unsigned int category_view_signals[LAST_SIGNAL] = { 0 };
 
 static void st_category_view_init	(STCategoryView		*view);
 static void st_category_view_class_init	(STCategoryViewClass	*class);
-static void st_category_view_finalize	(GObject		*object);
-static void st_category_view_realize	(GtkWidget		*widget);
 
 static void st_category_view_row_expanded_h	(GtkTreeView	*treeview,
 						 GtkTreeIter	*iter,
@@ -121,14 +119,9 @@ st_category_view_get_type (void)
 static void
 st_category_view_class_init (STCategoryViewClass *class)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS(class);
-  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(class);
-
   parent_class = g_type_class_peek_parent(class);
 
-  object_class->finalize = st_category_view_finalize;
-
-  widget_class->realize = st_category_view_realize;
+  g_type_class_add_private(class, sizeof(STCategoryViewPrivate));
 
   category_view_signals[SELECTION_CHANGED] = g_signal_new("selection-changed",
 							  ST_TYPE_CATEGORY_VIEW,
@@ -148,7 +141,7 @@ st_category_view_init (STCategoryView *view)
   GtkCellRenderer *renderer;
   GtkTreeSelection *selection;
 
-  view->priv = g_new0(STCategoryViewPrivate, 1);
+  view->priv = G_TYPE_INSTANCE_GET_PRIVATE(view, ST_TYPE_CATEGORY_VIEW, STCategoryViewPrivate);
 
   /* configure the view */
 
@@ -164,11 +157,13 @@ st_category_view_init (STCategoryView *view)
 
   column = gtk_tree_view_column_new();
 
-  view->priv->pixbuf_renderer = gtk_cell_renderer_pixbuf_new();
+  renderer = sgtk_cell_renderer_pixbuf_new();
 
-  gtk_tree_view_column_pack_start(column, view->priv->pixbuf_renderer, FALSE);
-  gtk_tree_view_column_set_attributes(column, view->priv->pixbuf_renderer,
+  gtk_tree_view_column_pack_start(column, renderer, FALSE);
+  gtk_tree_view_column_set_attributes(column, renderer,
 				      "stock-id", ST_CATEGORY_STORE_COLUMN_STOCK_ID,
+				      "stock-id-closed", ST_CATEGORY_STORE_COLUMN_STOCK_ID_CLOSED,
+				      "stock-id-open", ST_CATEGORY_STORE_COLUMN_STOCK_ID_OPEN,
 				      NULL);
 
   renderer = gtk_cell_renderer_text_new();
@@ -193,44 +188,6 @@ st_category_view_init (STCategoryView *view)
 						       view);
 }
 
-static void
-st_category_view_finalize (GObject *object)
-{
-  STCategoryView *view = ST_CATEGORY_VIEW(object);
-
-  g_free(view->priv);
-
-  G_OBJECT_CLASS(parent_class)->finalize(object);
-}
-
-static void
-st_category_view_realize (GtkWidget *widget)
-{
-  STCategoryView *view = ST_CATEGORY_VIEW(widget);
-  GdkPixbuf *pixbuf_expander_closed;
-  GdkPixbuf *pixbuf_expander_open;
-
-  pixbuf_expander_closed = gtk_widget_render_icon(widget,
-						  ST_STOCK_CATEGORY,
-						  GTK_ICON_SIZE_MENU,
-						  NULL);
-  pixbuf_expander_open = gtk_widget_render_icon(widget,
-						ST_STOCK_CATEGORY_OPEN,
-						GTK_ICON_SIZE_MENU,
-						NULL);
-  
-  /* those are used only for cells where is-expander is true */
-  g_object_set(view->priv->pixbuf_renderer,
-	       "pixbuf-expander-closed", pixbuf_expander_closed,
-	       "pixbuf-expander-open", pixbuf_expander_open,
-	       NULL);
-
-  g_object_unref(pixbuf_expander_closed);
-  g_object_unref(pixbuf_expander_open);
-
-  GTK_WIDGET_CLASS(parent_class)->realize(widget);
-}
-
 static void
 st_category_view_selection_changed_h (GtkTreeSelection *selection,
 				      gpointer data)
@@ -262,7 +219,7 @@ st_category_view_row_expanded_h (GtkTreeView *treeview,
   STCategoryBag *bag;
 
   gtk_tree_model_get(gtk_tree_view_get_model(treeview), iter, ST_CATEGORY_STORE_COLUMN_BAG, &bag, -1);
-  st_category_bag_set_flags(bag, ST_CATEGORY_BAG_EXPANDED);
+  SG_SET_FLAGS(bag->flags, ST_CATEGORY_BAG_EXPANDED);
   g_object_unref(bag);
 }
 
@@ -275,7 +232,7 @@ st_category_view_row_collapsed_h (GtkTreeView *treeview,
   STCategoryBag *bag;
 
   gtk_tree_model_get(gtk_tree_view_get_model(treeview), iter, ST_CATEGORY_STORE_COLUMN_BAG, &bag, -1);
-  st_category_bag_unset_flags(bag, ST_CATEGORY_BAG_EXPANDED);
+  SG_UNSET_FLAGS(bag->flags, ST_CATEGORY_BAG_EXPANDED);
   g_object_unref(bag);
 }
 
@@ -358,10 +315,6 @@ st_category_view_model_category_appended_h (STCategoryStore *store,
     }
 }
 
-/*
- * Select the iter belonging to category BAG, but does not trigger the
- * emission of a selection-changed signal.
- */
 void
 st_category_view_select_category (STCategoryView *view, STCategoryBag *bag)
 {
@@ -371,8 +324,5 @@ st_category_view_select_category (STCategoryView *view, STCategoryBag *bag)
   g_return_if_fail(ST_IS_CATEGORY_BAG(bag));
 
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
-
-  g_signal_handler_block(selection, view->priv->selection_changed_hid);
   gtk_tree_selection_select_iter(selection, &bag->iter);
-  g_signal_handler_unblock(selection, view->priv->selection_changed_hid);
 }
