@@ -38,9 +38,9 @@
 #include "st-handler.h"
 #include "st-handler-field.h"
 #include "st-handlers.h"
-#include "st-parser.h"
 #include "st-action.h"
 #include "st-settings.h"
+#include "st-util.h"
 
 /*** type definitions ********************************************************/
 
@@ -63,8 +63,12 @@ enum {
   STATEMENT_OPTIONS_WINDOW_DIVIDER,		/* deprecated */
   STATEMENT_OPTIONS_PREFERENCES_WINDOW_WIDTH,
   STATEMENT_OPTIONS_PREFERENCES_WINDOW_HEIGHT,
+  STATEMENT_OPTIONS_PREFERENCES_SELECTED_PAGE,
+  STATEMENT_OPTIONS_PREFERENCES_PLUGINS_EXPANDED,
   STATEMENT_OPTIONS_STREAM_PROPERTIES_WINDOW_WIDTH,
-  STATEMENT_OPTIONS_SELECTED_PREFERENCES_PAGE,
+  STATEMENT_OPTIONS_STREAM_COLUMNS_WINDOW_WIDTH,
+  STATEMENT_OPTIONS_STREAM_COLUMNS_WINDOW_HEIGHT,
+  STATEMENT_OPTIONS_SELECTED_PREFERENCES_PAGE,	/* obsolete */
   STATEMENT_OPTIONS_PROXY_ENABLED,
   STATEMENT_OPTIONS_PROXY_TYPE,
   STATEMENT_OPTIONS_PROXY_URL,			/* deprecated */
@@ -99,6 +103,8 @@ enum {
   STATEMENT_HANDLER_FIELD,
   STATEMENT_HANDLER_FIELD_VISIBLE,
   STATEMENT_HANDLER_FIELD_WIDTH,
+  STATEMENT_HANDLER_FIELD_POSITION,
+  STATEMENT_HANDLER_KEY,
   STATEMENT_HANDLER_EXPANDED_CATEGORIES,	/* obsolete */
   STATEMENT_HANDLER_SELECTED_STREAMS		/* obsolete */
 };
@@ -113,7 +119,11 @@ typedef struct
   GSList		*handlers;
   STHandler		*handler;
   GSList		*handler_field_iter;
-} STConfigLoadInfo;
+  STHandlerField	*handler_field;
+  GParamSpec		*handler_config_pspec;
+  char			*handler_config_key;
+  GValueArray		*handler_config_values;
+} LoadInfo;
 
 /*** constant definitions ****************************************************/
 
@@ -194,11 +204,27 @@ static SGParserDefinition config_definitions[] = {
     STATEMENT_OPTIONS,	STATEMENT_OPTIONS_PREFERENCES_WINDOW_HEIGHT,
     "preferences_window_height", FALSE,	G_TYPE_INT
   },
+  {
+    STATEMENT_OPTIONS,	STATEMENT_OPTIONS_PREFERENCES_SELECTED_PAGE,
+    "preferences_selected_page", FALSE,	G_TYPE_STRING
+  },
+  {
+    STATEMENT_OPTIONS,	STATEMENT_OPTIONS_PREFERENCES_PLUGINS_EXPANDED,
+    "preferences_plugins_expanded", FALSE, G_TYPE_BOOLEAN
+  },
   {
     STATEMENT_OPTIONS,	STATEMENT_OPTIONS_STREAM_PROPERTIES_WINDOW_WIDTH,
     "stream_properties_window_width", FALSE, G_TYPE_INT
   },
   {
+    STATEMENT_OPTIONS,	STATEMENT_OPTIONS_STREAM_COLUMNS_WINDOW_WIDTH,
+    "stream_columns_window_width", FALSE, G_TYPE_INT
+  },
+  {
+    STATEMENT_OPTIONS,	STATEMENT_OPTIONS_STREAM_COLUMNS_WINDOW_HEIGHT,
+    "stream_columns_window_height", FALSE, G_TYPE_INT
+  },
+  {				/* obsolete */
     STATEMENT_OPTIONS,	STATEMENT_OPTIONS_SELECTED_PREFERENCES_PAGE,
     "selected_preferences_page", FALSE,	G_TYPE_INT
   },
@@ -330,6 +356,14 @@ static SGParserDefinition config_definitions[] = {
     STATEMENT_HANDLER_FIELD,	STATEMENT_HANDLER_FIELD_WIDTH,
     "width",		FALSE,		G_TYPE_INT
   },
+  {
+    STATEMENT_HANDLER_FIELD,	STATEMENT_HANDLER_FIELD_POSITION,
+    "position",		FALSE,		G_TYPE_INT
+  },
+  {
+    STATEMENT_HANDLER,	STATEMENT_HANDLER_KEY,
+    "key",		TRUE,		G_TYPE_STRING
+  },
   {				/* obsolete */
     STATEMENT_HANDLER,	STATEMENT_HANDLER_EXPANDED_CATEGORIES,
     "expanded_categories",	TRUE,	G_TYPE_NONE
@@ -343,24 +377,28 @@ static SGParserDefinition config_definitions[] = {
   
 /*** function declarations ***************************************************/
 
-static void st_config_load_find_history              (STConfigLoadInfo *info);
+static void st_config_load_find_history              (LoadInfo *info);
 
-static void st_config_load_handler_begin	     (STConfigLoadInfo *info);
-static void st_config_load_handler_end		     (STConfigLoadInfo *info);
+static void st_config_load_handler_begin	     (LoadInfo *info);
+static void st_config_load_handler_end		     (LoadInfo *info);
 
-static void st_config_load_handler_field_begin	     (STConfigLoadInfo *info);
-static void st_config_load_handler_field_end	     (STConfigLoadInfo *info);
+static void st_config_load_handler_field_begin	     (LoadInfo *info);
+static void st_config_load_handler_field_end	     (LoadInfo *info);
 
-static void st_config_load_command		     (STConfigLoadInfo *info);
-static void st_config_load_program		     (STConfigLoadInfo *info);
-static void st_config_load_handler_field_width	     (STConfigLoadInfo *info);
+static void st_config_load_command		     (LoadInfo *info);
+static void st_config_load_program		     (LoadInfo *info);
+static void st_config_load_handler_field_width	     (LoadInfo *info);
+
+static void st_config_load_handler_key_begin         (LoadInfo *info);
+static void st_config_load_handler_key_value         (LoadInfo *info);
+static void st_config_load_handler_key_end           (LoadInfo *info);
 
 /*** implementation **********************************************************/
 
 gboolean
 st_config_load (const char *filename, GError **err)
 { 
-  STConfigLoadInfo info = { NULL, };
+  LoadInfo info = { NULL, };
 
   g_return_val_if_fail(filename != NULL, FALSE);
 
@@ -368,7 +406,7 @@ st_config_load (const char *filename, GError **err)
   if (! info.parser)
     return FALSE;
   
-  sg_parser_set_message_cb(info.parser, st_parser_message_cb);
+  info.parser->scanner->msg_handler = st_parser_msg_cb;
   sg_parser_definev(info.parser, config_definitions);
 
   while ((info.statement = sg_parser_get_statement(info.parser)))
@@ -384,6 +422,10 @@ st_config_load (const char *filename, GError **err)
 	    case STATEMENT_HANDLER_FIELD:
 	      st_config_load_handler_field_end(&info);
 	      break;
+
+	    case STATEMENT_HANDLER_KEY:
+	      st_config_load_handler_key_end(&info);
+	      break;
 	    }
 	}
     
@@ -425,10 +467,6 @@ st_config_load (const char *filename, GError **err)
 	      st_settings.main_window_height = g_value_get_int(&info.statement->value);
 	      break;
 
-	    case STATEMENT_OPTIONS_SELECTED_PREFERENCES_PAGE:
-	      st_settings.selected_preferences_page = g_value_get_int(&info.statement->value);
-	      break;
-
 	    case STATEMENT_OPTIONS_PREFERENCES_WINDOW_WIDTH:
 	      st_settings.preferences_window_width = g_value_get_int(&info.statement->value);
 	      break;
@@ -437,10 +475,26 @@ st_config_load (const char *filename, GError **err)
 	      st_settings.preferences_window_height = g_value_get_int(&info.statement->value);
 	      break;
 
+	    case STATEMENT_OPTIONS_PREFERENCES_SELECTED_PAGE:
+	      st_settings.preferences_selected_page = g_value_dup_string(&info.statement->value);
+	      break;
+
+	    case STATEMENT_OPTIONS_PREFERENCES_PLUGINS_EXPANDED:
+	      st_settings.preferences_plugins_expanded = g_value_get_boolean(&info.statement->value);
+	      break;
+
 	    case STATEMENT_OPTIONS_STREAM_PROPERTIES_WINDOW_WIDTH:
 	      st_settings.stream_properties_window_width = g_value_get_int(&info.statement->value);
 	      break;
 
+	    case STATEMENT_OPTIONS_STREAM_COLUMNS_WINDOW_WIDTH:
+	      st_settings.stream_columns_window_width = g_value_get_int(&info.statement->value);
+	      break;
+
+	    case STATEMENT_OPTIONS_STREAM_COLUMNS_WINDOW_HEIGHT:
+	      st_settings.stream_columns_window_height = g_value_get_int(&info.statement->value);
+	      break;
+
 	    case STATEMENT_OPTIONS_PROXY_ENABLED:
 	      st_settings.proxy_enabled = g_value_get_boolean(&info.statement->value);
 	      break;
@@ -519,7 +573,8 @@ st_config_load (const char *filename, GError **err)
 	    case STATEMENT_HANDLER_EXPANDED_CATEGORIES:
 	    case STATEMENT_HANDLER_SELECTED_STREAMS:
 	    case STATEMENT_OPTIONS_SAVE_AUTOMATIC:
-	      sg_parser_warn(info.parser, _("obsolete statement, ignored"));
+	    case STATEMENT_OPTIONS_SELECTED_PREFERENCES_PAGE:
+	      g_scanner_warn(info.parser->scanner, _("obsolete statement, ignored"));
 	      break;
 
 	    case STATEMENT_OPTIONS_SELECTED_HANDLER:
@@ -551,7 +606,10 @@ st_config_load (const char *filename, GError **err)
 	      break;
 
 	    case STATEMENT_ACTION:
+	      g_free(info.action);
 	      info.action = g_value_dup_string(&info.statement->value);
+	      if (! info.action)
+		g_scanner_warn(info.parser->scanner, _("action name missing"));
 	      break;
 
 	    case STATEMENT_ACTION_COMMAND:
@@ -571,8 +629,8 @@ st_config_load (const char *filename, GError **err)
 		{
 		  int sort_index = g_value_get_int(&info.statement->value);
 		  
-		  if (sort_index > st_handler_count_fields(info.handler, ST_HANDLER_FIELD_VISIBLE))
-		    sg_parser_warn(info.parser, _("index too big, ignored"));
+		  if (! st_handler_is_dummy(info.handler) && sort_index > st_handler_count_fields(info.handler, ST_HANDLER_FIELD_VISIBLE))
+		    g_scanner_warn(info.parser->scanner, _("index too big, ignored"));
 		  else
 		    st_handler_set_fields_sort_index(info.handler, sort_index);
 		}
@@ -593,19 +651,22 @@ st_config_load (const char *filename, GError **err)
 	      break;
 
 	    case STATEMENT_HANDLER_FIELD_VISIBLE:
-	      if (info.handler_field_iter)
-		{
-		  STHandlerField *field = info.handler_field_iter->data;
-		  st_handler_field_set_user_visible(field, g_value_get_boolean(&info.statement->value));
-		}
+	      if (info.handler_field)
+		st_handler_field_set_user_visible(info.handler_field, g_value_get_boolean(&info.statement->value));
 	      break;
 
 	    case STATEMENT_HANDLER_FIELD_WIDTH:
-	      if (info.handler_field_iter)
-		{
-		  STHandlerField *field = info.handler_field_iter->data;
-		  st_handler_field_set_width(field, g_value_get_int(&info.statement->value));
-		}
+	      if (info.handler_field)
+		st_handler_field_set_width(info.handler_field, g_value_get_int(&info.statement->value));
+	      break;
+
+	    case STATEMENT_HANDLER_FIELD_POSITION:
+	      if (info.handler_field)
+		st_handler_field_set_position(info.handler_field, g_value_get_int(&info.statement->value));
+	      break;
+
+	    case STATEMENT_HANDLER_KEY:
+	      st_config_load_handler_key_begin(&info);
 	      break;
 	    }
 	}
@@ -626,8 +687,12 @@ st_config_load (const char *filename, GError **err)
 	      /* obsolete */
 	      break;
 
+	    case STATEMENT_HANDLER_KEY:
+	      st_config_load_handler_key_value(&info);
+	      break;
+
 	    default:
-	      sg_parser_warn(info.parser, _("unexpected value"));
+	      g_scanner_warn(info.parser->scanner, _("unexpected value"));
 	    }
 	}
 
@@ -636,121 +701,133 @@ st_config_load (const char *filename, GError **err)
     }
   
   sg_parser_free(info.parser);
+  g_free(info.action);
   st_handlers_reorder(info.handlers);
+  g_free(info.handler_config_key);
+  if (info.handler_config_values)
+    g_value_array_free(info.handler_config_values);
   
   return TRUE;
 }
 
 static void
-st_config_load_find_history (STConfigLoadInfo *info)
+st_config_load_find_history (LoadInfo *info)
 {
   if (! G_VALUE_HOLDS_STRING(&info->statement->value))
     {
-      sg_parser_warn(info->parser, _("expected string value"));
+      g_scanner_warn(info->parser->scanner, _("expected string value"));
       return;
     }
 
   if (g_slist_length(st_settings.find_history) < ST_SETTINGS_FIND_HISTORY_MAX_LENGTH)
     st_settings.find_history = g_slist_append(st_settings.find_history, g_value_dup_string(&info->statement->value));
   else
-    sg_parser_warn(info->parser, _("too many find history tokens"));
+    g_scanner_warn(info->parser->scanner, _("too many find history tokens"));
 }
 
 static void
-st_config_load_handler_begin (STConfigLoadInfo *info)
+st_config_load_handler_begin (LoadInfo *info)
 {
   const char *name;
 
   name = g_value_get_string(&info->statement->value);
-  info->handler = st_handlers_find_by_name(name);
-  
-  if (info->handler)
+  if (name)
     {
-      info->handler_field_iter = st_handler_get_fields(info->handler);
+      info->handler = st_handlers_find_by_name(name);
+  
+      if (info->handler)
+	info->handler_field_iter = st_handler_get_fields(info->handler);
+      else
+	{
+	  info->handler = st_handler_new_dummy(name);
+	  g_scanner_warn(info->parser->scanner, _("%s: no such handler"), name);
+	}
+
       info->handlers = g_slist_append(info->handlers, info->handler);
     }
   else
-    sg_parser_warn(info->parser, _("%s: no such handler"), name);
+    {
+      info->handler = NULL;
+      g_scanner_warn(info->parser->scanner, _("handler name missing"));
+    }
 }
 
 static void
-st_config_load_handler_end (STConfigLoadInfo *info)
+st_config_load_handler_end (LoadInfo *info)
 {
   info->handler = NULL;
   info->handler_field_iter = NULL;
+  info->handler_field = NULL;
 }
 
 static void
-st_config_load_handler_field_begin (STConfigLoadInfo *info)
+st_config_load_handler_field_begin (LoadInfo *info)
 {
   if (! info->handler)
     return;
   
-  while (info->handler_field_iter)
+  if (st_handler_is_dummy(info->handler))
     {
-      STHandlerField *field = info->handler_field_iter->data;
-      
-      if (ST_HANDLER_FIELD_IS_VISIBLE(field))
-	break;
-      
-      info->handler_field_iter = info->handler_field_iter->next;
+      info->handler_field = st_handler_field_new_dummy();
+      st_handler_add_field(info->handler, info->handler_field);
     }
+  else
+    {
+      while (info->handler_field_iter)
+	{
+	  STHandlerField *field = info->handler_field_iter->data;
+      
+	  if (ST_HANDLER_FIELD_IS_VISIBLE(field))
+	    break;
+      
+	  info->handler_field_iter = info->handler_field_iter->next;
+	}
 		  
-  if (! info->handler_field_iter)
-    sg_parser_warn(info->parser, _("too many fields"));
+      if (info->handler_field_iter)
+	info->handler_field = info->handler_field_iter->data;
+      else
+	{
+	  info->handler_field = NULL;
+	  g_scanner_warn(info->parser->scanner, _("too many fields"));
+	}
+    }
 }
 
 static void
-st_config_load_handler_field_end (STConfigLoadInfo *info)
+st_config_load_handler_field_end (LoadInfo *info)
 {
   if (info->handler_field_iter)
     info->handler_field_iter = info->handler_field_iter->next;
-  /* otherwise there has been a "too many fields" error -> nop */
+  /* otherwise it's a dummy handler, or there has been a "too many fields" error -> nop */
 }
 
 static void
-st_config_load_command (STConfigLoadInfo *info)
+st_config_load_command (LoadInfo *info)
 {
   if (! info->action)
-    {
-      sg_parser_warn(info->parser, _("command already specified"));
-      return;
-    }
+    return;
 
-  if (st_action_exists(info->action))
-    st_action_associate(info->action, g_value_get_string(&info->statement->value));
-
-  g_free(info->action);
-  info->action = NULL;
+  st_action_associate(info->action, g_value_get_string(&info->statement->value));
 }
 
 static void
-st_config_load_program (STConfigLoadInfo *info)
+st_config_load_program (LoadInfo *info)
 {
+  const char *command;
+  char *new_command;
+
   if (! info->action)
-    {
-      sg_parser_warn(info->parser, _("program already specified"));
-      return;
-    }
+    return;
 
-  if (st_action_exists(info->action))
-    {
-      const char *command;
-      char *new_command;
+  command = g_value_get_string(&info->statement->value);
+  new_command = st_action_convert_old_style_command(command);
 
-      command = g_value_get_string(&info->statement->value);
-      new_command = st_action_convert_old_style_command(command);
-
-      st_action_associate(info->action, new_command);
-      g_free(new_command);
-    }
-
-  g_free(info->action);
-  info->action = NULL;
+  st_action_associate(info->action, new_command);
+  g_free(new_command);
 }
 
 static void
-st_config_load_handler_field_width (STConfigLoadInfo *info)
+st_config_load_handler_field_width (LoadInfo *info)
 {
   STHandlerField *field = NULL;
 
@@ -759,24 +836,167 @@ st_config_load_handler_field_width (STConfigLoadInfo *info)
 
   if (! G_VALUE_HOLDS_INT(&info->statement->value))
     {
-      sg_parser_warn(info->parser, _("expected integer value"));
+      g_scanner_warn(info->parser->scanner, _("expected integer value"));
       return;
     }
 
-  while (info->handler_field_iter)
+  if (st_handler_is_dummy(info->handler))
     {
-      field = info->handler_field_iter->data;
-
-      info->handler_field_iter = info->handler_field_iter->next;
-
-      if (ST_HANDLER_FIELD_IS_VISIBLE(field))
-	break;
-      else
-	field = NULL;
+      field = st_handler_field_new_dummy();
+      st_handler_add_field(info->handler, field);
     }
+  else
+    while (info->handler_field_iter)
+      {
+	field = info->handler_field_iter->data;
+
+	info->handler_field_iter = info->handler_field_iter->next;
+	
+	if (ST_HANDLER_FIELD_IS_VISIBLE(field))
+	  break;
+	else
+	  field = NULL;
+      }
   
   if (field)
     st_handler_field_set_width(field, g_value_get_int(&info->statement->value));
   else
-    sg_parser_warn(info->parser, _("too many fields"));
+    g_scanner_warn(info->parser->scanner, _("too many fields"));
+}
+
+static void
+st_config_load_handler_key_begin (LoadInfo *info)
+{
+  if (! info->handler)
+    return;
+
+  if (st_handler_is_dummy(info->handler))
+    {
+      g_free(info->handler_config_key);
+      info->handler_config_key = g_value_dup_string(&info->statement->value);
+    }
+  else
+    {
+      info->handler_config_pspec = st_handler_config_lookup(info->handler, g_value_get_string(&info->statement->value));
+      if (! info->handler_config_pspec)
+	{
+	  g_scanner_warn(info->parser->scanner, _("unknown configuration key"));
+	  return;
+	}
+    }
+  
+  if (info->handler_config_values)
+    g_value_array_free(info->handler_config_values);
+  info->handler_config_values = g_value_array_new(0);
+}
+
+static void
+st_config_load_handler_key_value (LoadInfo *info)
+{
+  if (! info->handler)
+    return;
+
+  g_value_array_append(info->handler_config_values, &info->statement->value);
+}
+
+static void
+st_config_load_handler_key_end (LoadInfo *info)
+{
+  GParamSpec *pspec;
+
+  if (! info->handler)
+    return;
+
+  if (st_handler_is_dummy(info->handler))
+    {
+      if (info->handler_config_values->n_values == 0
+	  || info->handler_config_values->n_values > 1)
+	pspec = g_param_spec_value_array(info->handler_config_key,
+					 NULL,
+					 NULL,
+					 NULL,
+					 G_PARAM_READWRITE);
+      else
+	{
+	  GValue *value = g_value_array_get_nth(info->handler_config_values, 0);
+
+	  /* keep in sync with st_handler_config_register() */
+	  if (G_VALUE_HOLDS_BOOLEAN(value))
+	    pspec = g_param_spec_boolean(info->handler_config_key,
+					 NULL,
+					 NULL,
+					 FALSE,
+					 G_PARAM_READWRITE);
+	  else if (G_VALUE_HOLDS_INT(value))
+	    pspec = g_param_spec_int(info->handler_config_key,
+				     NULL,
+				     NULL,
+				     G_MININT,
+				     G_MAXINT,
+				     0,
+				     G_PARAM_READWRITE);
+	  else if (G_VALUE_HOLDS_UINT(value))
+	    pspec = g_param_spec_uint(info->handler_config_key,
+				      NULL,
+				      NULL,
+				      0,
+				      G_MAXUINT,
+				      0,
+				      G_PARAM_READWRITE);
+	  else if (G_VALUE_HOLDS_DOUBLE(value))
+	    pspec = g_param_spec_double(info->handler_config_key,
+					NULL,
+					NULL,
+					G_MINDOUBLE,
+					G_MAXDOUBLE,
+					0,
+					G_PARAM_READWRITE);
+	  else if (G_VALUE_HOLDS_STRING(value))
+	    pspec = g_param_spec_string(info->handler_config_key,
+					NULL,
+					NULL,
+					NULL,
+					G_PARAM_READWRITE);
+	  else
+	    {
+	      g_scanner_warn(info->parser->scanner, _("invalid value type"));
+	      return;
+	    }
+	}
+
+      st_handler_config_register(info->handler, pspec);
+    }
+  else
+    {
+      pspec = info->handler_config_pspec;
+      if (! pspec)
+	return;
+    }
+
+  if (G_PARAM_SPEC_VALUE_TYPE(pspec) == G_TYPE_VALUE_ARRAY)
+    {
+      GValue value = { 0, };
+
+      g_value_init(&value, G_TYPE_VALUE_ARRAY);
+      g_value_set_boxed(&value, info->handler_config_values);
+      st_handler_config_set_value(info->handler, g_param_spec_get_name(pspec), &value);
+      g_value_unset(&value);
+    }
+  else
+    {
+      GValue *value;
+
+      if (info->handler_config_values->n_values != 1)
+	{
+	  g_scanner_warn(info->parser->scanner, _("wrong number of values"));
+	  return;
+	}
+      
+      value = g_value_array_get_nth(info->handler_config_values, 0);
+      
+      if (G_VALUE_HOLDS(value, G_PARAM_SPEC_VALUE_TYPE(pspec)))
+	st_handler_config_set_value(info->handler, g_param_spec_get_name(pspec), value);
+      else
+	g_scanner_warn(info->parser->scanner, _("invalid value type"));
+    }
 }
