@@ -41,9 +41,16 @@
 #include "st-category-store.h"
 #include "st-stream-store.h"
 #include "st-dialog-api.h"
+#include "st-plugin.h"
 
 /*** type definitions ********************************************************/
 
+enum {
+  PROP_0,
+  PROP_NAME,
+  PROP_DUMMY
+};
+    
 typedef struct
 {
   gpointer cb;
@@ -52,9 +59,9 @@ typedef struct
 
 struct _STHandlerPrivate
 {
+  gboolean		dummy;
   char			*name;
   char			*label;
-  char			*copyright;
   char			*description;
   char			*home;
 
@@ -63,7 +70,8 @@ struct _STHandlerPrivate
   GSList		*fields;
   GNode			*stock_categories;
   
-  unsigned int		flags;
+  STHandlerFlags	flags;
+  int			stream_version;
 
   STHandlerCallback	callbacks[ST_HANDLER_N_EVENTS];
 
@@ -76,13 +84,13 @@ struct _STHandlerPrivate
   GHashTable		*streams;
   GMutex		*streams_mutex;
 
-  GSList		*remarkable_categories;
-  GMutex		*remarkable_categories_mutex;
-
   int			fields_sort_index;
   int			fields_sort_order;
 
   int			paned_position;
+
+  GHashTable		*config;
+  GMutex		*config_mutex;
 };
 
 typedef struct
@@ -97,9 +105,29 @@ typedef struct
   GHashTable		*stream_bag_lists;
 } StreamListsToBagsInfo;
 
+typedef struct
+{
+  STHandlerConfigForeachCallback	cb;
+  gpointer				data;
+} ConfigForeachInfo;
+
+typedef struct
+{
+  GParamSpec		*pspec;
+  GValue		value;
+} ConfigEntry;
+
 /*** function declarations ***************************************************/
 
+static void        st_handler_class_init		(STHandlerClass *class);
 static void        st_handler_init			(STHandler   *handler);
+static GObject     *st_handler_constructor		(GType       type,
+							 unsigned int n_construct_properties,
+							 GObjectConstructParam *construct_params);
+static void        st_handler_set_property		(GObject      *object,
+							 unsigned int prop_id,
+							 const GValue *value,
+							 GParamSpec   *pspec);
 
 static STCategory  *st_handler_category_new_default_cb	(gpointer    data);
 static void	   st_handler_category_free_default_cb	(STCategory  *category,
@@ -153,8 +181,18 @@ static void        st_handler_stream_lists_free_cb	(gpointer    key,
 							 gpointer    value,
 							 gpointer    user_data);
 
-static void        st_handler_restore_category_flags	(STHandler     *handler,
-							 STCategoryBag *category_bag);
+static gboolean    st_handler_get_selected_category_cb	(STCategoryStore *store,
+							 STCategoryBag   *bag,
+							 STCategoryBag   *parent,
+							 gpointer        data);
+
+static void        st_handler_config_foreach_cb		(gpointer    key,
+							 gpointer    value,
+							 gpointer    user_data);
+
+/*** variable declarations ***************************************************/
+
+static GObjectClass *parent_class = NULL;
 
 /*** API implementation ******************************************************/
 
@@ -169,7 +207,7 @@ st_handler_get_type (void)
 	sizeof(STHandlerClass),
 	NULL,
 	NULL,
-	NULL,
+	(GClassInitFunc) st_handler_class_init,
 	NULL,
 	NULL,
 	sizeof(STHandler),
@@ -186,15 +224,54 @@ st_handler_get_type (void)
   return handler_type;
 }
 
+/**
+ * st_handler_new:
+ * @name: the name of the handler.
+ *
+ * Creates a new #STHandler. After creating the handler, you must set
+ * its label using st_handler_set_label(), add your stream fields
+ * using st_handler_add_field(), and bind functions to events using
+ * st_handler_bind().
+ *
+ * Functions must be bound to #ST_HANDLER_EVENT_STREAM_FIELD_GET and
+ * #ST_HANDLER_EVENT_STREAM_FIELD_SET. Additionally, if one of the
+ * fields added with st_handler_add_field() has the
+ * #ST_HANDLER_FIELD_EDITABLE flag set, a function must be bound to
+ * #ST_HANDLER_EVENT_STREAM_MODIFY.
+ *
+ * Return value: the newly created #STHandler.
+ **/
 STHandler *
 st_handler_new (const char *name)
 {
-  STHandler *handler;
-
   g_return_val_if_fail(name != NULL, NULL);
 
-  handler = g_object_new(ST_TYPE_HANDLER, NULL);
-  handler->priv->name = g_strdup(name);
+  return g_object_new(ST_TYPE_HANDLER, "name", name, NULL);
+}
+
+/**
+ * st_handler_new_from_plugin:
+ * @plugin: a plugin to take the name, label and icon from.
+ *
+ * Creates a new #STHandler, using the name, label and icon of
+ * @plugin. You must set the name and label of @plugin before calling
+ * this function.
+ *
+ * Return value: the newly created #STHandler.
+ **/
+STHandler *
+st_handler_new_from_plugin (STPlugin *plugin)
+{
+  STHandler *handler;
+
+  g_return_val_if_fail(ST_IS_PLUGIN(plugin), NULL);
+
+  handler = st_handler_new(st_plugin_get_name(plugin));
+
+  handler->priv->label = g_strdup(st_plugin_get_label(plugin));
+  handler->priv->pixbuf = st_plugin_get_pixbuf(plugin);
+  if (handler->priv->pixbuf)
+    g_object_ref(handler->priv->pixbuf);
 
   return handler;
 }
@@ -223,44 +300,71 @@ st_handler_stream_free_default_cb (STStream *stream, gpointer data)
   st_stream_free(stream);
 }
 
+/**
+ * st_handler_set_label:
+ * @handler: a handler.
+ * @label: the handler label.
+ *
+ * Sets the handler label, which will be used in the handler tab, and
+ * in various other places of the user interface.
+ **/
 void
 st_handler_set_label (STHandler *handler, const char *label)
 {
   g_return_if_fail(ST_IS_HANDLER(handler));
   g_return_if_fail(label != NULL);
-  g_return_if_fail(handler->priv->label == NULL);
 
+  g_free(handler->priv->label);
   handler->priv->label = g_strdup(label);
 }
 
+/**
+ * st_handler_set_description:
+ * @handler: a handler.
+ * @description: the handler description.
+ *
+ * Sets the handler description, which will be displayed in the
+ * toolbar when the handler is selected.
+ **/
 void
 st_handler_set_description (STHandler *handler, const char *description)
 {
   g_return_if_fail(ST_IS_HANDLER(handler));
   g_return_if_fail(description != NULL);
-  g_return_if_fail(handler->priv->description == NULL);
 
+  g_free(handler->priv->description);
   handler->priv->description = g_strdup(description);
 }
 
+/**
+ * st_handler_set_home:
+ * @handler: a handler.
+ * @home: the handler home page URL.
+ *
+ * Sets the home page URL of the directory handled by
+ * @handler. Setting the home page URL will allow the user to visit
+ * the page by clicking on the toolbar link.
+ **/
 void
 st_handler_set_home (STHandler *handler, const char *home)
 {
   g_return_if_fail(ST_IS_HANDLER(handler));
   g_return_if_fail(home != NULL);
-  g_return_if_fail(handler->priv->home == NULL);
 
+  g_free(handler->priv->home);
   handler->priv->home = g_strdup(home);
 }
 
+/*
+ * Deprecated
+ */
 void
 st_handler_set_copyright (STHandler *handler, const char *copyright)
 {
   g_return_if_fail(ST_IS_HANDLER(handler));
   g_return_if_fail(copyright != NULL);
-  g_return_if_fail(handler->priv->copyright == NULL);
 
-  handler->priv->copyright = g_strdup(copyright);
+  /* nop */
 }
 
 /*
@@ -273,14 +377,38 @@ st_handler_set_info (STHandler *handler,
 {
   g_return_if_fail(ST_IS_HANDLER(handler));
   g_return_if_fail(label != NULL);
-  g_return_if_fail(handler->priv->label == NULL);
   g_return_if_fail(copyright != NULL);
-  g_return_if_fail(handler->priv->copyright == NULL);
 
   st_handler_set_label(handler, label);
-  st_handler_set_copyright(handler, copyright);
 }
 
+/**
+ * st_handler_set_icon_from_pixbuf:
+ * @handler: a handler.
+ * @pixbuf: an icon.
+ *
+ * Sets the handler icon from a #GdkPixbuf.
+ **/
+void
+st_handler_set_icon_from_pixbuf (STHandler *handler, GdkPixbuf *pixbuf)
+{
+  g_return_if_fail(ST_IS_HANDLER(handler));
+  g_return_if_fail(GDK_IS_PIXBUF(pixbuf));
+  
+  if (handler->priv->pixbuf)
+    g_object_unref(handler->priv->pixbuf);
+  handler->priv->pixbuf = g_object_ref(pixbuf);
+}
+
+/**
+ * st_handler_set_icon_from_inline:
+ * @handler: a handler.
+ * @size: the size of the inline data.
+ * @data: the inline data.
+ *
+ * Sets the handler icon using inline data. See the
+ * gdk_pixbuf_new_from_inline() documentation.
+ **/
 void
 st_handler_set_icon_from_inline (STHandler *handler,
 				 int size,
@@ -289,12 +417,25 @@ st_handler_set_icon_from_inline (STHandler *handler,
   g_return_if_fail(ST_IS_HANDLER(handler));
   g_return_if_fail(size > 0);
   g_return_if_fail(data != NULL);
-  g_return_if_fail(handler->priv->pixbuf == NULL);
 
+  if (handler->priv->pixbuf)
+    g_object_unref(handler->priv->pixbuf);
   handler->priv->pixbuf = gdk_pixbuf_new_from_inline(size, data, FALSE, NULL);
   g_return_if_fail(GDK_IS_PIXBUF(handler->priv->pixbuf));
 }
 
+/**
+ * st_handler_set_icon_from_file:
+ * @handler: a handler.
+ * @filename: an icon file.
+ * @err: a location to store errors, or %NULL.
+ *
+ * Sets the handler icon using an external file. See the
+ * gdk_pixbuf_new_from_file() documentation.
+ *
+ * Return value: %FALSE if the file could not be loaded. In such case
+ * @err will be set.
+ **/
 gboolean
 st_handler_set_icon_from_file (STHandler *handler,
 			       const char *filename,
@@ -304,8 +445,10 @@ st_handler_set_icon_from_file (STHandler *handler,
   g_return_val_if_fail(filename != NULL, FALSE);
   g_return_val_if_fail(handler->priv->pixbuf == NULL, FALSE);
 
+  if (handler->priv->pixbuf)
+    g_object_unref(handler->priv->pixbuf);
   handler->priv->pixbuf = gdk_pixbuf_new_from_file(filename, err);
-  return GDK_IS_PIXBUF(handler->priv->pixbuf);
+  return handler->priv->pixbuf != NULL;
 }
 
 /*
@@ -317,11 +460,22 @@ st_handler_set_icon (STHandler *handler, int size, const guint8 *data)
   g_return_if_fail(ST_IS_HANDLER(handler));
   g_return_if_fail(size > 0);
   g_return_if_fail(data != NULL);
-  g_return_if_fail(handler->priv->pixbuf == NULL);
 
   st_handler_set_icon_from_inline(handler, size, data);
 }
 
+/**
+ * st_handler_set_stock_categories:
+ * @handler: a handler.
+ * @categories: a tree of stock categories.
+ *
+ * Sets the handler stock categories. The "__main" category is
+ * mandatory.
+ *
+ * Special category names: __main and __search.
+ *
+ * The stock categories will not be saved to permanent storage.
+ **/
 void
 st_handler_set_stock_categories (STHandler *handler, GNode *categories)
 {
@@ -353,14 +507,43 @@ st_handler_set_stock_categories_cb (GNode *node, gpointer data)
   return FALSE;			/* continue */
 }
 
+/**
+ * st_handler_set_flags:
+ * @handler: a handler.
+ * @flags: the handler flags.
+ *
+ * Sets the handler flags.
+ **/
 void
-st_handler_set_flags (STHandler *handler, unsigned int flags)
+st_handler_set_flags (STHandler *handler, STHandlerFlags flags)
 {
   g_return_if_fail(ST_IS_HANDLER(handler));
 
   handler->priv->flags = flags;
 }
 
+/**
+ * st_handler_set_stream_version:
+ * @handler: a handler:
+ * @version: the stream version.
+ *
+ * Sets the handler stream version.
+ **/
+void
+st_handler_set_stream_version (STHandler *handler, int version)
+{
+  g_return_if_fail(ST_IS_HANDLER(handler));
+
+  handler->priv->stream_version = version;
+}
+
+/**
+ * st_handler_add_field:
+ * @handler: a handler.
+ * @field: a field to add to @handler.
+ *
+ * Add a stream field to @handler.
+ **/
 void
 st_handler_add_field (STHandler *handler, STHandlerField *field)
 {
@@ -370,6 +553,17 @@ st_handler_add_field (STHandler *handler, STHandlerField *field)
   handler->priv->fields = g_slist_append(handler->priv->fields, field);
 }
 
+/**
+ * st_handler_bind:
+ * @handler: a handler.
+ * @event: an event to bind @cb to.
+ * @cb: a callback function to bind to @event.
+ * @data: data to pass to @cb.
+ *
+ * Binds callback function @cb to event @event. If a function is
+ * already bound to @cb, the previous binding will be overriden (only
+ * one function can be bound to an event).
+ **/
 void
 st_handler_bind (STHandler *handler,
 		 STHandlerEvent event,
@@ -384,6 +578,14 @@ st_handler_bind (STHandler *handler,
   handler->priv->callbacks[event].data = data;
 }
 
+/**
+ * st_handler_notice:
+ * @handler: a handler.
+ * @format: the message format. See the printf() documentation.
+ * @...: the parameters to insert into the format string.
+ *
+ * Outputs a formatted handler notice to the standard error output.
+ **/
 void
 st_handler_notice (STHandler *handler, const char *format, ...)
 {
@@ -401,25 +603,350 @@ st_handler_notice (STHandler *handler, const char *format, ...)
   g_free(notice);
 }
 
+/**
+ * st_handler_config_register:
+ * @handler: a handler.
+ * @pspec: a param spec defining the configuration key to
+ * register. The param spec value type must be #G_TYPE_BOOLEAN,
+ * #G_TYPE_INT, #G_TYPE_UINT, #G_TYPE_DOUBLE, #G_TYPE_STRING or
+ * #G_TYPE_VALUE_ARRAY. The param spec flags must be
+ * #G_PARAM_READWRITE.
+ *
+ * Registers a new configuration key.
+ **/
+void
+st_handler_config_register (STHandler *handler, GParamSpec *pspec)
+{
+  ConfigEntry *entry;
+
+  /* keep in sync with st_config_load_handler_key_value() */
+
+  g_return_if_fail(ST_IS_HANDLER(handler));
+  g_return_if_fail(G_IS_PARAM_SPEC(pspec));
+  g_return_if_fail(G_PARAM_SPEC_VALUE_TYPE(pspec) == G_TYPE_BOOLEAN
+		   || G_PARAM_SPEC_VALUE_TYPE(pspec) == G_TYPE_INT
+		   || G_PARAM_SPEC_VALUE_TYPE(pspec) == G_TYPE_UINT
+		   || G_PARAM_SPEC_VALUE_TYPE(pspec) == G_TYPE_DOUBLE
+		   || G_PARAM_SPEC_VALUE_TYPE(pspec) == G_TYPE_STRING
+		   || G_PARAM_SPEC_VALUE_TYPE(pspec) == G_TYPE_VALUE_ARRAY);
+  g_return_if_fail((pspec->flags & G_PARAM_READWRITE) != 0);
+  
+  g_mutex_lock(handler->priv->config_mutex);
+
+  entry = g_hash_table_lookup(handler->priv->config, g_param_spec_get_name(pspec));
+  g_return_if_fail(entry == NULL);
+
+  entry = g_new0(ConfigEntry, 1);
+  entry->pspec = g_param_spec_ref(pspec);
+  g_param_spec_sink(pspec);
+  g_value_init(&entry->value, G_PARAM_SPEC_VALUE_TYPE(entry->pspec));
+  g_param_value_set_default(entry->pspec, &entry->value);
+
+  g_hash_table_insert(handler->priv->config, (gpointer) g_param_spec_get_name(pspec), entry);
+
+  g_mutex_unlock(handler->priv->config_mutex);
+}
+
+/**
+ * st_handler_config_lookup:
+ * @handler: a handler.
+ * @key: a configuration key.
+ *
+ * Looks up the #GParamSpec of a configuration key.
+ *
+ * Return value: the #GParamSpec of @key, or %NULL if no such
+ * configuration key exists.
+ **/
+GParamSpec *
+st_handler_config_lookup (STHandler *handler, const char *key)
+{
+  const ConfigEntry *entry;
+  GParamSpec *pspec;
+
+  g_return_val_if_fail(ST_IS_HANDLER(handler), NULL);
+  g_return_val_if_fail(key != NULL, NULL);
+
+  g_mutex_lock(handler->priv->config_mutex);
+  entry = g_hash_table_lookup(handler->priv->config, key);
+  pspec = entry ? entry->pspec : NULL;
+  g_mutex_unlock(handler->priv->config_mutex);
+
+  return pspec;
+}
+
+/**
+ * st_handler_config_foreach:
+ * @handler: a handler.
+ * @cb: a function to call for each key/value pair.
+ * @data: data to pass to @cb.
+ *
+ * Calls the given function for each key/value pair in the
+ * configuration database of @handler. You must not call any other
+ * st_handler_config_* function while iterating over the database.
+ **/
+void
+st_handler_config_foreach (STHandler *handler,
+			   STHandlerConfigForeachCallback cb,
+			   gpointer data)
+{
+  ConfigForeachInfo info;
+
+  g_return_if_fail(ST_IS_HANDLER(handler));
+  g_return_if_fail(cb != NULL);
+
+  info.cb = cb;
+  info.data = data;
+
+  g_mutex_lock(handler->priv->config_mutex);
+  g_hash_table_foreach(handler->priv->config, st_handler_config_foreach_cb, &info);
+  g_mutex_unlock(handler->priv->config_mutex);
+}
+
+static void
+st_handler_config_foreach_cb (gpointer key,
+			      gpointer value,
+			      gpointer user_data)
+{
+  ConfigForeachInfo *info = user_data;
+  const ConfigEntry *entry = value;
+
+  info->cb(entry->pspec, &entry->value, info->data);
+}
+
+/**
+ * st_handler_config_get_value:
+ * @handler: a handler.
+ * @key: a configuration key previously registered with
+ * st_handler_config_register().
+ * @value: a location to copy the value of @key to.
+ *
+ * Gets a configuration value. @value must be zero-filled (but not
+ * initialized), and should be unset with g_value_unset() when no
+ * longer needed.
+ **/
+void
+st_handler_config_get_value (STHandler *handler,
+			     const char *key,
+			     GValue *value)
+{
+  const ConfigEntry *entry;
+
+  g_return_if_fail(ST_IS_HANDLER(handler));
+  g_return_if_fail(key != NULL);
+
+  g_mutex_lock(handler->priv->config_mutex);
+
+  entry = g_hash_table_lookup(handler->priv->config, key);
+  g_return_if_fail(entry != NULL);
+
+  g_value_init(value, G_PARAM_SPEC_VALUE_TYPE(entry->pspec));
+  g_value_copy(&entry->value, value);
+
+  g_mutex_unlock(handler->priv->config_mutex);
+}
+
+/**
+ * st_handler_config_set_value:
+ * @handler: a handler.
+ * @key: a configuration key previously registered with
+ * st_handler_config_register().
+ * @value: the value to set.
+ *
+ * Sets a configuration value. @value must match the value type of the
+ * param spec of @key.
+ **/
+void
+st_handler_config_set_value (STHandler *handler,
+			     const char *key,
+			     const GValue *value)
+{
+  ConfigEntry *entry;
+
+  g_return_if_fail(ST_IS_HANDLER(handler));
+  g_return_if_fail(key != NULL);
+  g_return_if_fail(G_IS_VALUE(value));
+
+  g_mutex_lock(handler->priv->config_mutex);
+
+  entry = g_hash_table_lookup(handler->priv->config, key);
+  g_return_if_fail(entry != NULL);
+
+  g_value_copy(value, &entry->value);
+  g_param_value_validate(entry->pspec, &entry->value);
+
+  g_mutex_unlock(handler->priv->config_mutex);
+}
+
+#define CONFIG_GETTER(_name, _gtype, _ctype, _getter, _failret)		\
+  _ctype st_handler_config_get_ ## _name (STHandler *handler,		\
+					  const char *key)		\
+  {									\
+    const ConfigEntry *entry;						\
+    _ctype value;							\
+									\
+    g_return_val_if_fail(ST_IS_HANDLER(handler), _failret);		\
+    g_return_val_if_fail(key != NULL, _failret);			\
+									\
+    g_mutex_lock(handler->priv->config_mutex);				\
+									\
+    entry = g_hash_table_lookup(handler->priv->config, key);		\
+    g_return_val_if_fail(entry != NULL, _failret);			\
+    g_return_val_if_fail(G_VALUE_HOLDS(&entry->value, _gtype), _failret); \
+									\
+    value = _getter(&entry->value);					\
+									\
+    g_mutex_unlock(handler->priv->config_mutex);			\
+									\
+    return value;							\
+  }
+
+#define CONFIG_SETTER(_name, _gtype, _ctype, _setter)		\
+  void st_handler_config_set_ ## _name (STHandler *handler,	\
+					const char *key,	\
+					_ctype value)		\
+  {								\
+    ConfigEntry *entry;						\
+    								\
+    g_return_if_fail(ST_IS_HANDLER(handler));			\
+    g_return_if_fail(key != NULL);				\
+    								\
+    g_mutex_lock(handler->priv->config_mutex);			\
+    								\
+    entry = g_hash_table_lookup(handler->priv->config, key);	\
+    g_return_if_fail(entry != NULL);				\
+    g_return_if_fail(G_VALUE_HOLDS(&entry->value, _gtype));	\
+    								\
+    _setter(&entry->value, value);				\
+    g_param_value_validate(entry->pspec, &entry->value);	\
+    								\
+    g_mutex_unlock(handler->priv->config_mutex);		\
+  }
+
+CONFIG_GETTER(boolean, G_TYPE_BOOLEAN, gboolean, g_value_get_boolean, FALSE)
+CONFIG_SETTER(boolean, G_TYPE_BOOLEAN, gboolean, g_value_set_boolean)
+
+CONFIG_GETTER(int, G_TYPE_INT, int, g_value_get_int, 0)
+CONFIG_SETTER(int, G_TYPE_INT, int, g_value_set_int)
+
+CONFIG_GETTER(uint, G_TYPE_UINT, unsigned int, g_value_get_uint, 0)
+CONFIG_SETTER(uint, G_TYPE_UINT, unsigned int, g_value_set_uint)
+
+CONFIG_GETTER(double, G_TYPE_DOUBLE, double, g_value_get_double, 0)
+CONFIG_SETTER(double, G_TYPE_DOUBLE, double, g_value_set_double)
+
+CONFIG_GETTER(string, G_TYPE_STRING, char *, g_value_dup_string, NULL)
+CONFIG_SETTER(string, G_TYPE_STRING, const char *, g_value_set_string)
+
+CONFIG_GETTER(value_array, G_TYPE_VALUE_ARRAY, GValueArray *, g_value_dup_boxed, NULL)
+CONFIG_SETTER(value_array, G_TYPE_VALUE_ARRAY, const GValueArray *, g_value_set_boxed)
+
+#undef CONFIG_GETTER
+#undef CONFIG_SETTER
+
 /*** private implementation **************************************************/
 
+static void
+st_handler_class_init (STHandlerClass *class)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS(class);
+
+  parent_class = g_type_class_peek_parent(class);
+
+  g_type_class_add_private(class, sizeof(STHandlerPrivate));
+
+  object_class->constructor = st_handler_constructor;
+  object_class->set_property = st_handler_set_property;
+
+  g_object_class_install_property(object_class,
+				  PROP_NAME,
+				  g_param_spec_string("name",
+						      _("Name"),
+						      _("The handler name"),
+						      NULL,
+						      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+
+  /* private properties */
+
+  g_object_class_install_property(object_class,
+				  PROP_DUMMY,
+				  g_param_spec_boolean("dummy",
+						       NULL,
+						       NULL,
+						       FALSE,
+						       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+}
+
 static void
 st_handler_init (STHandler *handler)
 {
-  handler->priv = g_new0(STHandlerPrivate, 1);
+  handler->priv = G_TYPE_INSTANCE_GET_PRIVATE(handler, ST_TYPE_HANDLER, STHandlerPrivate);
+}
 
-  handler->priv->streams = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_object_unref);
-  handler->priv->streams_mutex = g_mutex_new();
-  handler->priv->remarkable_categories_mutex = g_mutex_new();
+static GObject *
+st_handler_constructor (GType type,
+			unsigned int n_construct_properties,
+			GObjectConstructParam *construct_params)
+{
+  GObject *object; 
+  STHandler *handler;
+
+  object = parent_class->constructor(type, n_construct_properties, construct_params);
+  handler = ST_HANDLER(object);
 
   handler->priv->paned_position = 150;
 
-  /* provide handy defaults for some mandatory callbacks */
+  handler->priv->config = g_hash_table_new(g_str_hash, g_str_equal);
+  handler->priv->config_mutex = g_mutex_new();
 
-  st_handler_bind(handler, ST_HANDLER_EVENT_CATEGORY_NEW, st_handler_category_new_default_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_CATEGORY_FREE, st_handler_category_free_default_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_NEW, st_handler_stream_new_default_cb, NULL);
-  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FREE, st_handler_stream_free_default_cb, NULL);
+  if (! handler->priv->dummy)
+    {
+      handler->priv->streams = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_object_unref);
+      handler->priv->streams_mutex = g_mutex_new();
+
+      /* provide handy defaults for some mandatory callbacks */
+
+      st_handler_bind(handler, ST_HANDLER_EVENT_CATEGORY_NEW, st_handler_category_new_default_cb, NULL);
+      st_handler_bind(handler, ST_HANDLER_EVENT_CATEGORY_FREE, st_handler_category_free_default_cb, NULL);
+      st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_NEW, st_handler_stream_new_default_cb, NULL);
+      st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FREE, st_handler_stream_free_default_cb, NULL);
+    }
+  
+  return object;
+}
+
+static void
+st_handler_set_property (GObject *object,
+			 unsigned int prop_id,
+			 const GValue *value,
+			 GParamSpec *pspec)
+{
+  STHandler *handler = ST_HANDLER(object);
+
+  switch (prop_id)
+    {
+    case PROP_NAME:
+      handler->priv->name = g_value_dup_string(value);
+      break;
+
+    case PROP_DUMMY:
+      handler->priv->dummy = g_value_get_boolean(value);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+      break;
+    }
+}
+
+STHandler *
+st_handler_new_dummy (const char *name)
+{
+  g_return_val_if_fail(name != NULL, NULL);
+
+  return g_object_new(ST_TYPE_HANDLER,
+		      "name", name,
+		      "dummy", TRUE,
+		      NULL);
 }
 
 void
@@ -563,6 +1090,7 @@ st_handler_reload (STHandler *handler,
 	  st_stream_store_append_list(*stream_store, streams);
 	  sg_objects_free(streams);
 	  
+	  st_stream_store_touch(*stream_store);
 	  st_handler_set_streams(handler, ST_CATEGORY(category_bag)->name, *stream_store);
 	}
     }
@@ -582,6 +1110,7 @@ st_handler_reload_multiple_cb (gpointer key,
 
   stream_store = st_stream_store_new(handler);
   st_stream_store_append_list(stream_store, streams);
+  st_stream_store_touch(stream_store);
 
   st_handler_set_streams(handler, category_name, stream_store);
   g_object_unref(stream_store);
@@ -689,15 +1218,7 @@ static gboolean
 st_handler_categories_to_bags_cb (GNode *node, gpointer data)
 {
   if (node->data)
-    {
-      STHandler *handler = data;
-      STCategoryBag *bag;
-
-      bag = st_category_bag_new_from_category(handler, node->data);
-      st_handler_restore_category_flags(handler, bag);
-
-      node->data = bag;
-    }
+    node->data = st_category_bag_new_from_category(data, node->data);
 
   return FALSE;			/* continue */
 }
@@ -823,8 +1344,7 @@ st_handler_get_categories (STHandler *handler)
   g_return_val_if_fail(ST_IS_HANDLER(handler), NULL);
   g_return_val_if_fail(handler->priv->categories != NULL, NULL);
 
-  g_object_ref(handler->priv->categories);
-  return handler->priv->categories;
+  return g_object_ref(handler->priv->categories);
 }
 
 void
@@ -836,12 +1356,10 @@ st_handler_set_streams (STHandler *handler,
   g_return_if_fail(category_name != NULL);
   g_return_if_fail(ST_IS_STREAM_STORE(streams));
 
-  g_object_ref(streams);
-
   g_mutex_lock(handler->priv->streams_mutex);
   g_hash_table_insert(handler->priv->streams,
 		      g_strdup(category_name),
-		      streams);
+		      g_object_ref(streams));
   g_mutex_unlock(handler->priv->streams_mutex);
 }
 
@@ -917,99 +1435,39 @@ st_handler_select_category (STHandler *handler)
       category_bag = st_handler_get_stock_category(handler, ST_CATEGORY_BAG_MAIN);
 
       g_assert(category_bag != NULL);
-      st_category_bag_set_flags(category_bag, ST_CATEGORY_BAG_SELECTED);
+      SG_SET_FLAGS(category_bag->flags, ST_CATEGORY_BAG_SELECTED);
     }
 }
 
-void
-st_handler_add_remarkable_category (STHandler *handler,
-				    STCategoryBag *category_bag)
-{
-  g_return_if_fail(ST_IS_HANDLER(handler));
-  g_return_if_fail(ST_IS_CATEGORY_BAG(category_bag));
-
-  g_mutex_lock(handler->priv->remarkable_categories_mutex);
-  if (! g_slist_find(handler->priv->remarkable_categories, category_bag))
-    {
-      g_object_ref(category_bag);
-      handler->priv->remarkable_categories = g_slist_append(handler->priv->remarkable_categories, category_bag);
-    }
-  g_mutex_unlock(handler->priv->remarkable_categories_mutex);
-}
-
-void
-st_handler_remove_remarkable_category (STHandler *handler,
-				       STCategoryBag *category_bag)
-{
-  GSList *elem;
-
-  g_return_if_fail(ST_IS_HANDLER(handler));
-  g_return_if_fail(ST_IS_CATEGORY_BAG(category_bag));
-
-  g_mutex_lock(handler->priv->remarkable_categories_mutex);
-  elem = g_slist_find(handler->priv->remarkable_categories, category_bag);
-  if (elem)
-    {
-      g_object_unref(elem->data);
-      handler->priv->remarkable_categories = g_slist_delete_link(handler->priv->remarkable_categories, elem);
-    }
-  g_mutex_unlock(handler->priv->remarkable_categories_mutex);
-}
-
 STCategoryBag *
 st_handler_get_selected_category (STHandler *handler)
 {
-  GSList *l;
   STCategoryBag *selected_category = NULL;
 
   g_return_val_if_fail(ST_IS_HANDLER(handler), NULL);
 
-  g_mutex_lock(handler->priv->remarkable_categories_mutex);
-  SG_LIST_FOREACH(l, handler->priv->remarkable_categories)
-    if (st_category_bag_get_flags(l->data) & ST_CATEGORY_BAG_SELECTED)
-      {
-	g_object_ref(l->data);
-	selected_category = l->data;
-	break;
-      }
-  g_mutex_unlock(handler->priv->remarkable_categories_mutex);
+  st_category_store_foreach(handler->priv->categories,
+			    st_handler_get_selected_category_cb,
+			    &selected_category);
   
   return selected_category;
 }
 
-static void
-st_handler_restore_category_flags (STHandler *handler, STCategoryBag *bag)
+static gboolean
+st_handler_get_selected_category_cb (STCategoryStore *store,
+				     STCategoryBag *bag,
+				     STCategoryBag *parent,
+				     gpointer data)
 {
-  GSList *l;
-  STCategoryBag *match = NULL;
-
-  g_return_if_fail(ST_IS_HANDLER(handler));
-  g_return_if_fail(ST_IS_CATEGORY_BAG(bag));
-
-  g_mutex_lock(handler->priv->remarkable_categories_mutex);
-
-  SG_LIST_FOREACH(l, handler->priv->remarkable_categories)
+  if (bag->flags & ST_CATEGORY_BAG_SELECTED)
     {
-      STCategoryBag *remarkable_bag = l->data;
-	
-      if (! strcmp(ST_CATEGORY(bag)->name, ST_CATEGORY(remarkable_bag)->name))
-	{
-	  g_object_ref(remarkable_bag);
-	  match = remarkable_bag;
-	  break;
-	}
-    }
+      STCategoryBag **selected = data;
 
-  g_mutex_unlock(handler->priv->remarkable_categories_mutex);
-
-  if (match && match != bag)
-    {
-      st_category_bag_unset_flags(bag, ~0); /* reset flags */
-      st_category_bag_set_flags(bag, st_category_bag_get_flags(match));
-
-      st_category_bag_unset_flags(match, ~0); /* reset flags */
-      g_object_unref(match);
+      *selected = g_object_ref(bag);
+      return TRUE;		/* stop */
     }
+  else
+    return FALSE;		/* continue */
 }
 
 gboolean
@@ -1021,6 +1479,14 @@ st_handler_event_is_bound (STHandler *handler, STHandlerEvent event)
   return handler->priv->callbacks[event].cb != NULL;
 }
 
+gboolean
+st_handler_is_dummy (STHandler *handler)
+{
+  g_return_val_if_fail(ST_IS_HANDLER(handler), FALSE);
+
+  return handler->priv->dummy;
+}
+
 const char *
 st_handler_get_name (STHandler *handler)
 {
@@ -1037,14 +1503,6 @@ st_handler_get_label (STHandler *handler)
   return handler->priv->label;
 }
 
-const char *
-st_handler_get_copyright (STHandler *handler)
-{
-  g_return_val_if_fail(ST_IS_HANDLER(handler), NULL);
-
-  return handler->priv->copyright;
-}
-
 const char *
 st_handler_get_description (STHandler *handler)
 {
@@ -1066,9 +1524,6 @@ st_handler_get_pixbuf (STHandler *handler)
 {
   g_return_val_if_fail(ST_IS_HANDLER(handler), NULL);
 
-  if (handler->priv->pixbuf)
-    g_object_ref(handler->priv->pixbuf);
-  
   return handler->priv->pixbuf;
 }
 
@@ -1080,6 +1535,14 @@ st_handler_get_flags (STHandler *handler)
   return handler->priv->flags;
 }
 
+int
+st_handler_get_stream_version (STHandler *handler)
+{
+  g_return_val_if_fail(ST_IS_HANDLER(handler), 0);
+
+  return handler->priv->stream_version;
+}
+
 void
 st_handler_set_paned_position (STHandler *handler, int paned_position)
 {
@@ -1159,7 +1622,7 @@ st_handler_event_reload (STHandler *handler,
 			 GError **err)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_RELOAD];
-  STHandlerReloadCallback *cb = callback->cb;
+  STHandlerReloadCallback cb = callback->cb;
 
   g_return_val_if_fail(cb != NULL, FALSE);
   return cb(category, categories, streams, callback->data, err);
@@ -1172,7 +1635,7 @@ st_handler_event_reload_multiple (STHandler *handler,
 				  GError **err)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_RELOAD_MULTIPLE];
-  STHandlerReloadMultipleCallback *cb = callback->cb;
+  STHandlerReloadMultipleCallback cb = callback->cb;
 
   g_return_val_if_fail(cb != NULL, FALSE);
   return cb(categories, streams, callback->data, err);
@@ -1182,7 +1645,7 @@ STCategory *
 st_handler_event_category_new (STHandler *handler)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_CATEGORY_NEW];
-  STCategoryNewCallback *cb = callback->cb;
+  STCategoryNewCallback cb = callback->cb;
 
   g_return_val_if_fail(cb != NULL, NULL);
   return cb(callback->data);
@@ -1192,7 +1655,7 @@ void
 st_handler_event_category_free (STHandler *handler, STCategory *category)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_CATEGORY_FREE];
-  STCategoryFreeCallback *cb = callback->cb;
+  STCategoryFreeCallback cb = callback->cb;
 
   g_return_if_fail(cb != NULL);
   cb(category, callback->data);
@@ -1202,7 +1665,7 @@ STStream *
 st_handler_event_stream_new (STHandler *handler)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_NEW];
-  STStreamNewCallback *cb = callback->cb;
+  STStreamNewCallback cb = callback->cb;
 
   g_return_val_if_fail(cb != NULL, NULL);
   return cb(callback->data);
@@ -1215,7 +1678,7 @@ st_handler_event_stream_field_get (STHandler *handler,
 				   GValue *value)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_FIELD_GET];
-  STStreamFieldGetCallback *cb = callback->cb;
+  STStreamFieldGetCallback cb = callback->cb;
 
   g_return_if_fail(cb != NULL);
   cb(stream, field, value, callback->data);
@@ -1228,7 +1691,7 @@ st_handler_event_stream_field_set (STHandler *handler,
 				   const GValue *value)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_FIELD_SET];
-  STStreamFieldSetCallback *cb = callback->cb;
+  STStreamFieldSetCallback cb = callback->cb;
 
   g_return_if_fail(cb != NULL);
   cb(stream, field, value, callback->data);
@@ -1241,7 +1704,7 @@ st_handler_event_stream_stock_field_get (STHandler *handler,
 					 GValue *value)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_STOCK_FIELD_GET];
-  STStreamStockFieldGetCallback *cb = callback->cb;
+  STStreamStockFieldGetCallback cb = callback->cb;
 
   g_return_if_fail(cb != NULL);
   cb(stream, stock_field, value, callback->data);
@@ -1255,7 +1718,7 @@ st_handler_event_stream_modify (STHandler *handler,
 				GError **err)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_MODIFY];
-  STStreamModifyCallback *cb = callback->cb;
+  STStreamModifyCallback cb = callback->cb;
 
   g_return_val_if_fail(cb != NULL, FALSE);
   return cb(stream, fields, values, callback->data, err);
@@ -1267,7 +1730,7 @@ st_handler_event_stream_delete (STHandler *handler,
 				GError **err)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_DELETE];
-  STStreamDeleteCallback *cb = callback->cb;
+  STStreamDeleteCallback cb = callback->cb;
 
   g_return_val_if_fail(cb != NULL, FALSE);
   return cb(stream, callback->data, err);
@@ -1277,7 +1740,7 @@ void
 st_handler_event_stream_free (STHandler *handler, STStream *stream)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_FREE];
-  STStreamFreeCallback *cb = callback->cb;
+  STStreamFreeCallback cb = callback->cb;
 
   g_return_if_fail(cb != NULL);
   cb(stream, callback->data);
@@ -1289,7 +1752,7 @@ st_handler_event_stream_resolve (STHandler *handler,
 				 GError **err)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_RESOLVE];
-  STStreamResolveCallback *cb = callback->cb;
+  STStreamResolveCallback cb = callback->cb;
   
   g_return_val_if_fail(cb != NULL, FALSE);
   return cb(stream, callback->data, err);
@@ -1301,7 +1764,7 @@ st_handler_event_stream_tune_in (STHandler *handler,
 				 GError **err)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_TUNE_IN];
-  STStreamTuneInCallback *cb = callback->cb;
+  STStreamTuneInCallback cb = callback->cb;
   
   g_return_val_if_fail(cb != NULL, FALSE);
   return cb(stream, callback->data, err);
@@ -1313,7 +1776,7 @@ st_handler_event_stream_tune_in_multiple (STHandler *handler,
 					  GError **err)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_TUNE_IN_MULTIPLE];
-  STStreamTuneInMultipleCallback *cb = callback->cb;
+  STStreamTuneInMultipleCallback cb = callback->cb;
   
   g_return_val_if_fail(cb != NULL, FALSE);
   return cb(streams, callback->data, err);
@@ -1325,7 +1788,7 @@ st_handler_event_stream_record (STHandler *handler,
 				GError **err)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_RECORD];
-  STStreamRecordCallback *cb = callback->cb;
+  STStreamRecordCallback cb = callback->cb;
 
   g_return_val_if_fail(cb != NULL, FALSE);
   return cb(stream, callback->data, err);
@@ -1337,7 +1800,7 @@ st_handler_event_stream_browse (STHandler *handler,
 				GError **err)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_STREAM_BROWSE];
-  STStreamBrowseCallback *cb = callback->cb;
+  STStreamBrowseCallback cb = callback->cb;
 
   g_return_val_if_fail(cb != NULL, FALSE);
   return cb(stream, callback->data, err);
@@ -1347,7 +1810,7 @@ gpointer
 st_handler_event_thread_begin (STHandler *handler)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_THREAD_BEGIN];
-  STThreadBeginCallback *cb = callback->cb;
+  STThreadBeginCallback cb = callback->cb;
 
   g_return_val_if_fail(cb != NULL, NULL);
   return cb(callback->data);
@@ -1357,8 +1820,18 @@ void
 st_handler_event_thread_end (STHandler *handler, gpointer thread_data)
 {
   STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_THREAD_END];
-  STThreadEndCallback *cb = callback->cb;
+  STThreadEndCallback cb = callback->cb;
 
   g_return_if_fail(cb != NULL);
   cb(thread_data, callback->data);
 }
+
+GtkWidget *
+st_handler_event_preferences_widget_new (STHandler *handler)
+{
+  STHandlerCallback *callback = &handler->priv->callbacks[ST_HANDLER_EVENT_PREFERENCES_WIDGET_NEW];
+  STHandlerPreferencesWidgetNewCallback cb = callback->cb;
+
+  g_return_val_if_fail(cb != NULL, NULL);
+  return cb(callback->data);
+}
