@@ -51,6 +51,8 @@
   (st_handler_event_is_bound((handler), ST_HANDLER_EVENT_STREAM_TUNE_IN) 	\
    || st_handler_event_is_bound((handler), ST_HANDLER_EVENT_STREAM_TUNE_IN_MULTIPLE))
 
+STHandler	*st_handler_new_dummy		(const char	*name);
+
 void		st_handler_complete		(STHandler	*handler);
 gboolean	st_handler_validate		(STHandler	*handler,
 						 GError		**err);
@@ -77,13 +79,14 @@ void		st_handler_select_category	(STHandler	*handler);
 gboolean	st_handler_event_is_bound	(STHandler	*handler,
 						 STHandlerEvent	event);
 
+gboolean	st_handler_is_dummy		(STHandler	*handler);
 const char	*st_handler_get_name		(STHandler	*handler);
 const char	*st_handler_get_label		(STHandler	*handler);
-const char	*st_handler_get_copyright	(STHandler	*handler);
 const char	*st_handler_get_description	(STHandler	*handler);
 const char	*st_handler_get_home		(STHandler	*handler);
 GdkPixbuf	*st_handler_get_pixbuf		(STHandler	*handler);
 unsigned int	st_handler_get_flags		(STHandler	*handler);
+int		st_handler_get_stream_version	(STHandler	*handler);
 
 void		st_handler_set_paned_position	(STHandler	*handler,
 						 int		paned_position);
@@ -99,11 +102,6 @@ void		st_handler_set_fields_sort_order (STHandler	*handler,
 						  int		sort_order);
 int		st_handler_get_fields_sort_order (STHandler	*handler);
 
-void		st_handler_add_remarkable_category	(STHandler     *handler,
-							 STCategoryBag *category_bag);
-void		st_handler_remove_remarkable_category	(STHandler     *handler,
-							 STCategoryBag *category_bag);
-
 STCategoryBag	*st_handler_get_selected_category	(STHandler     *handler);
 
 /* events */
@@ -161,5 +159,6 @@ gboolean	st_handler_event_stream_browse	   (STHandler	*handler,
 gpointer	st_handler_event_thread_begin	   (STHandler	*handler);
 void		st_handler_event_thread_end	   (STHandler	*handler,
 						    gpointer	data);
+GtkWidget	*st_handler_event_preferences_widget_new (STHandler *handler);
 
 #endif /* _ST_HANDLER_H */
