@@ -43,9 +43,17 @@
 /*** variable declarations ***************************************************/
 
 GSList *st_handlers_list = NULL;
+GSList *st_handlers_full_list = NULL;
 
 /*** API implementation ******************************************************/
 
+/**
+ * st_handlers_add:
+ * @handler: a handler to add.
+ *
+ * Adds @handler to streamtuner. The reference count of @handler will
+ * not be incremented.
+ **/
 void
 st_handlers_add (STHandler *handler)
 {
@@ -54,7 +62,9 @@ st_handlers_add (STHandler *handler)
   if (st_handler_validate(handler, &err))
     {
       st_handler_complete(handler);
+
       st_handlers_list = g_slist_append(st_handlers_list, handler);
+      st_handlers_full_list = g_slist_append(st_handlers_full_list, handler);
     }
   else
     {
@@ -63,7 +73,9 @@ st_handlers_add (STHandler *handler)
 			_("In %s, handler %s is malformed and cannot be registered (%s).\n"
 			  "\n"
 			  "You should report this problem to the plugin maintainer."),
-			st_plugin_current, st_handler_get_name(handler), err->message);
+			st_plugin_get_filename(st_plugin_current),
+			st_handler_get_name(handler),
+			err->message);
       else
 	st_error_dialog(_("A handler error has occurred"),
 			_("Handler %s is malformed and cannot be registered (%s).\n"
@@ -107,6 +119,13 @@ st_handlers_reorder (GSList *ordered_list)
     if (! g_slist_find(ordered_list, l->data))
       ordered_list = g_slist_append(ordered_list, l->data);
 
+  g_slist_free(st_handlers_full_list);
+  st_handlers_full_list = ordered_list;
+
   g_slist_free(st_handlers_list);
-  st_handlers_list = ordered_list;
+  st_handlers_list = NULL;
+
+  SG_LIST_FOREACH(l, st_handlers_full_list)
+    if (! st_handler_is_dummy(l->data))
+      st_handlers_list = g_slist_append(st_handlers_list, l->data);
 }
