@@ -37,13 +37,19 @@
 #include "st-action.h"
 #include "st-dialog-api.h"
 #include "st-stock.h"
+#include "st-util-api.h"
 
 /*** type definitions ********************************************************/
 
+enum {
+  ACTIVATE,
+  LAST_SIGNAL
+};
+
 struct _STLinkPrivate
 {
   GtkWidget		*label;
-  GtkItemFactory	*factory;
+  GtkWidget		*menu;
 
   char			*text;
   char			*uri;
@@ -53,19 +59,21 @@ struct _STLinkPrivate
 /*** variable declarations ***************************************************/
 
 static GObjectClass *parent_class = NULL;
-
-static GtkTooltips *tooltips = NULL;
+static unsigned int link_signals[LAST_SIGNAL] = { 0 };
 
 /*** function declarations ***************************************************/
 
-static void st_link_init (STLink *link);
 static void st_link_class_init (STLinkClass *class);
-
+static void st_link_init (STLink *link);
 static void st_link_finalize (GObject *object);
 
 static void st_link_update_text (STLink *link);
 static void st_link_update_cursor (STLink *link);
 
+static void st_link_activate (STLink *link);
+static gboolean st_link_expose_event_h (GtkWidget *widget,
+					GdkEventExpose *event,
+					gpointer user_data);
 static gboolean st_link_enter_notify_event_h (STLink *link,
 					      GdkEventCrossing *event,
 					      gpointer user_data);
@@ -76,9 +84,6 @@ static void st_link_open_activate_h (GtkMenuItem *item,
 				     gpointer user_data);
 static void st_link_copy_address_activate_h (GtkMenuItem *item,
 					     gpointer user_data);
-static gboolean st_link_button_press_event_h (STLink *link,
-					      GdkEventButton *event,
-					      gpointer user_data);
 static gboolean st_link_button_release_event_h (STLink *link,
 						GdkEventButton *event,
 						gpointer user_data);
@@ -120,51 +125,59 @@ static void
 st_link_class_init (STLinkClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS(class);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(class);
 
   parent_class = g_type_class_peek_parent(class);
 
+  g_type_class_add_private(class, sizeof(STLinkPrivate));
+
   object_class->finalize = st_link_finalize;
 
-  tooltips = gtk_tooltips_new();
+  link_signals[ACTIVATE] = g_signal_new ("activate",
+					 G_OBJECT_CLASS_TYPE(object_class),
+					 G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+					 G_STRUCT_OFFSET(STLinkClass, activate),
+					 NULL,
+					 NULL,
+					 g_cclosure_marshal_VOID__VOID,
+					 G_TYPE_NONE,
+					 0);
+  widget_class->activate_signal = link_signals[ACTIVATE];
+
+  class->activate = st_link_activate;
 }
 
 static void
 st_link_init (STLink *link)
 {
   GtkWidget *item;
-  static GtkItemFactoryEntry entries[] = {
-    {
-      N_("/_Open Link"),			NULL,
-      NULL,					0,
-      "<StockItem>",				ST_STOCK_OPEN_LINK
-    },
-    {
-      N_("/_Copy Link Address"),		NULL,
-      NULL,					0,
-      "<StockItem>",				ST_STOCK_COPY_LINK_ADDRESS
-    }
-  };
 
-  link->priv = g_new0(STLinkPrivate, 1);
+  link->priv = G_TYPE_INSTANCE_GET_PRIVATE(link, ST_TYPE_LINK, STLinkPrivate);
 
-  link->priv->factory = gtk_item_factory_new(GTK_TYPE_MENU, "<streamtuner-Link>", NULL);
-  gtk_item_factory_set_translate_func(link->priv->factory, sgtk_translate_func, NULL, NULL);
-  gtk_item_factory_create_items(link->priv->factory, G_N_ELEMENTS(entries), entries, NULL);
+  gtk_event_box_set_visible_window(GTK_EVENT_BOX(link), FALSE);
 
-  item = gtk_item_factory_get_item(link->priv->factory, N_("/Open Link"));
+  link->priv->menu = gtk_menu_new();
+
+  item = gtk_image_menu_item_new_from_stock(ST_STOCK_OPEN_LINK, NULL);
+  gtk_widget_show(item);
+  gtk_menu_shell_append(GTK_MENU_SHELL(link->priv->menu), item);
   g_signal_connect(item, "activate", G_CALLBACK(st_link_open_activate_h), link);
 
-  item = gtk_item_factory_get_item(link->priv->factory, N_("/Copy Link Address"));
+  item = gtk_image_menu_item_new_from_stock(ST_STOCK_COPY_LINK_ADDRESS, NULL);
+  gtk_widget_show(item);
+  gtk_menu_shell_append(GTK_MENU_SHELL(link->priv->menu), item);
   g_signal_connect(item, "activate", G_CALLBACK(st_link_copy_address_activate_h), link);
-
+  
   link->priv->label = gtk_label_new(NULL);
   gtk_container_add(GTK_CONTAINER(link), link->priv->label);
   gtk_widget_show(link->priv->label);
 
+  sgtk_widget_set_popup_menu(GTK_WIDGET(link), GTK_MENU(link->priv->menu));
+
   g_object_connect(link,
+		   "signal::expose-event", st_link_expose_event_h, NULL,
 		   "signal::enter-notify-event", st_link_enter_notify_event_h, NULL,
 		   "signal::leave-notify-event", st_link_leave_notify_event_h, NULL,
-		   "signal::button-press-event", st_link_button_press_event_h, NULL,
 		   "signal::button-release-event", st_link_button_release_event_h, NULL,
 		   NULL);
 }
@@ -174,12 +187,11 @@ st_link_finalize (GObject *object)
 {
   STLink *link = ST_LINK(object);
 
-  g_object_unref(link->priv->factory);
+  gtk_widget_destroy(link->priv->menu);
   g_free(link->priv->text);
   g_free(link->priv->uri);
-  g_free(link->priv);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  parent_class->finalize(object);
 }
 
 static void
@@ -207,11 +219,11 @@ st_link_update_text (STLink *link)
       char *tip;
 
       tip = g_strdup_printf(_("Visit %s"), link->priv->uri);
-      gtk_tooltips_set_tip(tooltips, GTK_WIDGET(link), tip, NULL);
+      st_set_tooltip(GTK_WIDGET(link), tip);
       g_free(tip);
     }
   else
-    gtk_tooltips_set_tip(tooltips, GTK_WIDGET(link), NULL, NULL);
+    st_set_tooltip(GTK_WIDGET(link), NULL);
 }
 
 static void
@@ -236,6 +248,37 @@ st_link_update_cursor (STLink *link)
     }
 }
 
+static void
+st_link_activate (STLink *link)
+{
+  if (link->priv->uri)
+    st_link_open(link);
+}
+
+static gboolean
+st_link_expose_event_h (GtkWidget *widget,
+			GdkEventExpose *event,
+			gpointer user_data)
+{
+  if (GTK_WIDGET_DRAWABLE(widget) && GTK_WIDGET_HAS_FOCUS(widget))
+    {
+      int border_width = GTK_CONTAINER(widget)->border_width;
+
+      gtk_paint_focus(widget->style,
+		      widget->window,
+		      GTK_WIDGET_STATE(widget),
+		      &event->area,
+		      widget,
+		      "link",
+		      widget->allocation.x + border_width,
+		      widget->allocation.y + border_width,
+		      widget->allocation.width - border_width * 2,
+		      widget->allocation.height - border_width * 2);
+    }
+  
+  return FALSE;			/* propagate event */
+}
+
 static gboolean
 st_link_enter_notify_event_h (STLink *link,
 			      GdkEventCrossing *event,
@@ -282,17 +325,6 @@ st_link_copy_address_activate_h (GtkMenuItem *item, gpointer user_data)
   st_link_copy_address(link);
 }
 
-static gboolean
-st_link_button_press_event_h (STLink *link,
-			      GdkEventButton *event,
-			      gpointer user_data)
-{
-  if (event->button == 3 && link->priv->in_link && link->priv->uri)
-    gtk_item_factory_popup(link->priv->factory, event->x_root, event->y_root, event->button, event->time);
-  
-  return FALSE;
-}
-
 static gboolean
 st_link_button_release_event_h (STLink *link,
 				GdkEventButton *event,
@@ -347,9 +379,13 @@ st_link_new (void)
   return g_object_new(ST_TYPE_LINK, NULL);
 }
 
-/*
- * Set the label of the link. TEXT can contain Pango markup.
- */
+/**
+ * st_link_set_text:
+ * @link: a link.
+ * @text: the link label (can contain Pango markup).
+ *
+ * Sets the text label of @link.
+ **/
 void
 st_link_set_text (STLink *link, const char *text)
 {
@@ -371,4 +407,15 @@ st_link_set_uri (STLink *link, const char *uri)
 
   st_link_update_text(link);
   st_link_update_cursor(link);
+
+  if (link->priv->uri)
+    {
+      GTK_WIDGET_SET_FLAGS(link, GTK_CAN_FOCUS);
+      sgtk_widget_set_popup_menu(GTK_WIDGET(link), GTK_MENU(link->priv->menu));
+    }
+  else
+    {
+      GTK_WIDGET_UNSET_FLAGS(link, GTK_CAN_FOCUS);
+      sgtk_widget_set_popup_menu(GTK_WIDGET(link), NULL);
+    }
 }
