@@ -32,11 +32,12 @@
 #include "config.h"
 #include <stdlib.h>
 #include <unistd.h>
+#include <getopt.h>
+#include <locale.h>
 #include <errno.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include "sg-util.h"
-#include "sgtk-message-dialog.h"
 #include "sgtk-stock.h"
 #include "sgtk-util.h"
 #include "st-thread.h"
@@ -51,14 +52,11 @@
 #include "st-shell.h"
 #include "st-splash.h"
 #include "st-util.h"
-#include "st-dialog-api.h"
 
 /*** variable declarations ***************************************************/
 
 static char *custom_private_dir = NULL;
-static STShell *shell = NULL;
-
-static char **global_argv;
+char **st_main_argv;
 
 /*** function declarations ***************************************************/
 
@@ -68,7 +66,6 @@ static void       st_main_splash_set_text	(STSplash	*splash,
 static void       st_main_splash_set_progress	(STSplash	*splash,
 						 double		progress);
 
-static void       st_main_quit_real		(gboolean	restart);
 static void       st_main_parse_args		(int		argc,
 						 char		**argv);
 static void       st_main_print_help		(void);
@@ -80,7 +77,7 @@ int
 main (int argc, char **argv)
 {
   STSplash *splash = NULL;
-  GSList *l;
+  GtkWindow *window;
 
 #ifdef ENABLE_NLS
   bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
@@ -88,11 +85,22 @@ main (int argc, char **argv)
   textdomain(GETTEXT_PACKAGE);
 #endif
 
+  /*
+   * gtk_init() calls setlocale(), but we need the locale to be set
+   * right now, so that g_print() and friends correctly convert
+   * internationalized strings to the user's charset (g_print() and
+   * friends might be used from st_thread_init() and
+   * st_main_parse_args()).
+   */
+  setlocale(LC_ALL, "");
+
   st_main_parse_args(argc, argv);
 
   st_thread_init();		/* must be called before gtk_init() */
   gtk_init(&argc, &argv);
 
+  GDK_THREADS_ENTER();
+
   st_settings_init(custom_private_dir);
   if (st_settings.splash_enabled)
     splash = st_main_splash_create();
@@ -108,21 +116,19 @@ main (int argc, char **argv)
   st_plugins_load();		st_main_splash_set_progress(splash, (double) 6 / 8);
 
   st_main_splash_set_text(splash, _("Loading session..."));
-  st_session_load();		st_main_splash_set_progress(splash, (double) 7 / 8);
+  st_session_load();
+  g_slist_foreach(st_handlers_list, (GFunc) st_handler_select_category, NULL);
 
-  SG_LIST_FOREACH(l, st_handlers_list)
-    st_handler_select_category(l->data);
+  st_main_splash_set_progress(splash, (double) 7 / 8);
 
-  st_main_splash_set_progress(splash, (double) 8 / 8);
-
-  GDK_THREADS_ENTER();
-
-  shell = st_shell_new();
-  g_object_add_weak_pointer(G_OBJECT(shell), (gpointer *) &shell);
+  st_main_splash_set_text(splash, _("Creating user interface..."));
+  st_shell_new();		st_main_splash_set_progress(splash, (double) 8 / 8);
 
   if (splash)
     gtk_widget_destroy(GTK_WIDGET(splash));
-  st_shell_show(shell);
+
+  window = st_shell_get_window(st_shell);
+  gtk_widget_show(GTK_WIDGET(window));
 
   gtk_main();
 
@@ -136,15 +142,12 @@ st_main_splash_create (void)
 {
   GtkWidget *splash;
 
-  GDK_THREADS_ENTER();
-
   gtk_window_set_auto_startup_notification(FALSE);
   splash = st_splash_new();
   gtk_widget_show(splash);
 
   sgtk_flush();
   gtk_window_set_auto_startup_notification(TRUE);
-  GDK_THREADS_LEAVE();
 
   return ST_SPLASH(splash);
 }
@@ -156,10 +159,8 @@ st_main_splash_set_text (STSplash *splash, const char *str)
 
   if (splash)
     {
-      GDK_THREADS_ENTER();
       st_splash_set_text(splash, str);
       sgtk_flush();
-      GDK_THREADS_LEAVE();
     }
 }
 
@@ -168,110 +169,57 @@ st_main_splash_set_progress (STSplash *splash, double progress)
 {
   if (splash)
     {
-      GDK_THREADS_ENTER();
       st_splash_set_progress(splash, progress);
       sgtk_flush();
-      GDK_THREADS_LEAVE();
     }
 }
 
-GtkWindow *
-st_main_get_transient (void)
-{
-  return shell ? st_shell_get_transient(shell) : NULL;
-}
-
-void
-st_main_quit (void)
-{
-  st_main_quit_real(FALSE);
-}
-
-void
-st_main_restart (void)
-{
-  st_main_quit_real(TRUE);
-}
-
-static void
-st_main_quit_real (gboolean restart)
-{
-  /*
-   * HIG 2.0:
-   *
-   * "When a confirmation alert is needed, present it immediately. If
-   * the user confirms closing without saving, hide the alert and the
-   * document or application window immediately, before doing any
-   * necessary internal clean-up. If the user chooses to save before
-   * closing, hide the alert immediately but show the document window
-   * until the document is saved, in case an error occurs. Then hide
-   * the document window immediately after it has been saved
-   * successfuly."
-   */
-
-  st_shell_prepare_finalize(shell);
-  st_session_save();
-
-  st_shell_hide(shell);
-  g_object_unref(shell);
-
-  if (restart)
-    {
-      char *secondary;
-      char *normalized;
-
-      execvp(global_argv[0], global_argv);
-
-      secondary = g_strdup_printf(_("Unable to restart streamtuner: %s"), g_strerror(errno));
-      normalized = st_dialog_normalize(secondary);
-      g_free(secondary);
-
-      st_error_dialog(_("A fatal error has occurred"), "%s", normalized);
-      g_free(normalized);
-    }
-
-  gtk_main_quit();
-}
-
 static void
 st_main_parse_args (int argc, char **argv)
 {
+  const struct option options[] = {
+    { "directory",	required_argument,	NULL,		'd' },
+    { "help",		no_argument,		NULL,		'?' },
+    { "version",	no_argument,		NULL,		'v' },
+    { NULL,		0,			NULL,		0 },
+  };
   int i;
   int c;
 
-  global_argv = g_new0(char *, argc + 1);
+  st_main_argv = g_new0(char *, argc + 1);
   for (i = 0; i < argc; i++)
-    global_argv[i] = g_strdup(argv[i]);
-  global_argv[i] = NULL;
+    st_main_argv[i] = g_strdup(argv[i]);
+  st_main_argv[i] = NULL;
 
-  while ((c = getopt(argc, argv, "d:thv")) != -1)
+  while ((c = getopt_long(argc, argv, "d:?v", options, NULL)) != -1)
     switch (c)
       {
       case 'd':
 	custom_private_dir = g_strdup(optarg);
 	break;
-      case 'h':
+
+      case '?':
 	st_main_print_help();
 	exit(0);
+
       case 'v':
 	st_main_print_version();
 	exit(0);
-      case '?':
-	exit(1);
+
       default:
-	g_assert_not_reached();
+	g_return_if_reached();
       }
 }
 
 static void
 st_main_print_help (void)
 {
-  g_print(_("Usage: streamtuner [OPTIONS]\n"));
+  g_print("%s\n", _("Usage: streamtuner [OPTIONS]"));
   g_print("\n");
-  g_print(_("Options:\n"));
-  g_print(_("  -d DIR   Specify a configuration directory (default is ~/.streamtuner)\n"));
-  g_print(_("  -h       Show this help and exit\n"));
-  g_print(_("  -v       Show version information and exit\n"));
+  g_print("%s\n", _("Options:"));
+  g_print("  -d, --directory=DIR      %s\n", _("Specify a configuration directory [~/.streamtuner]"));
+  g_print("  -?, --help               %s\n", _("Show this help"));
+  g_print("  -v, --version            %s\n", _("Show version information"));
 }
 
 static void
