@@ -32,10 +32,6 @@
 #ifndef _ST_MAIN_H
 #define _ST_MAIN_H
 
-#include <gtk/gtk.h>
-
-GtkWindow	*st_main_get_transient	(void);
-void		st_main_quit		(void);
-void		st_main_restart         (void);
+extern char	**st_main_argv;
 
 #endif /* _ST_MAIN_H */
