@@ -34,6 +34,16 @@
 
 /*** API implementation ******************************************************/
 
+/**
+ * st_pls_parse:
+ * @playlist: the playlist data to parse.
+ *
+ * Parses a SHOUTcast PLS playlist.
+ *
+ * Return value: the list of URIs contained in @playlist. The list
+ * should be freed using g_slist_foreach(list, (GFunc) g_free, NULL);
+ * g_slist_free(list); when no longer needed.
+ **/
 GSList *
 st_pls_parse (const char *playlist)
 {
@@ -46,7 +56,7 @@ st_pls_parse (const char *playlist)
   lines = g_strsplit(playlist, "\n", 0);
 
   for (i = 0; lines[i]; i++)
-    if (! strncmp(lines[i], "File", 4))
+    if (g_str_has_prefix(lines[i], "File"))
       {
 	char *equal;
 	
