@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2003 Jean-Yves Lefort
+ * Copyright (c) 2002, 2003, 2004 Jean-Yves Lefort
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,13 +32,17 @@
 #ifndef _ST_PLUGIN_H
 #define _ST_PLUGIN_H
 
-#include <glib.h>
+#include "st-plugin-api.h"
 
-typedef gboolean	(STPluginInitCallback)	(GError		**err);
+extern STPlugin *st_plugin_current;
 
-extern const char	*st_plugin_current;
+STPlugin *st_plugin_new (const char *filename, gboolean only_info);
 
-gboolean		st_plugin_load		(const char	*filename,
-						 GError		**err);
+const char *st_plugin_get_name (STPlugin *plugin);
+const char *st_plugin_get_filename (STPlugin *plugin);
+const char *st_plugin_get_label (STPlugin *plugin);
+GdkPixbuf *st_plugin_get_pixbuf (STPlugin *plugin);
+
+const char *st_plugin_get_error (STPlugin *plugin);
 
 #endif /* _ST_PLUGIN_H */
