@@ -1,6 +1,4 @@
 /*
- * $Id: st-preferences.c,v 1.15 2004/03/15 19:43:15 jylefort Exp $
- *
  * Copyright (c) 2003, 2004 Jean-Yves Lefort
  * All rights reserved.
  *
@@ -35,6 +33,10 @@
 #include "sg-util.h"
 #include "st-settings.h"
 
+/*** cpp *********************************************************************/
+
+#define STRING_PTR		"st-preferences-string-ptr"
+
 /*** function declarations ***************************************************/
 
 static void st_preferences_toggle_toggled_h	  (GtkToggleButton  *toggle,
@@ -57,8 +59,7 @@ st_preferences_bind_boolean (GtkToggleButton *toggle, gboolean *ptr)
 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle), *ptr);
 
-  g_signal_connect(G_OBJECT(toggle), "toggled",
-		   G_CALLBACK(st_preferences_toggle_toggled_h), ptr);
+  g_signal_connect(toggle, "toggled", G_CALLBACK(st_preferences_toggle_toggled_h), ptr);
 }
 
 static void
@@ -81,9 +82,28 @@ st_preferences_bind_string (GtkEntry *entry, char **ptr)
   g_return_if_fail(ptr != NULL);
 
   gtk_entry_set_text(entry, *ptr ? *ptr : "");
+  g_object_set_data(G_OBJECT(entry), STRING_PTR, ptr);
 
-  g_signal_connect(G_OBJECT(entry), "focus-out-event",
-		   G_CALLBACK(st_preferences_entry_focus_out_h), ptr);
+  g_signal_connect(entry, "focus-out-event", G_CALLBACK(st_preferences_entry_focus_out_h), NULL);
+}
+
+void
+st_preferences_commit_string (GtkEntry *entry)
+{
+  char **ptr;
+  const char *str;
+
+  g_return_if_fail(GTK_IS_ENTRY(entry));
+
+  ptr = g_object_get_data(G_OBJECT(entry), STRING_PTR);
+  g_return_if_fail(ptr != NULL);
+
+  str = gtk_entry_get_text(entry);
+
+  G_LOCK(st_settings);
+  g_free(*ptr);
+  *ptr = str[0] ? g_strdup(str) : NULL;
+  G_UNLOCK(st_settings);
 }
 
 static gboolean
@@ -91,20 +111,9 @@ st_preferences_entry_focus_out_h (GtkWidget *widget,
 				  GdkEventFocus *event,
 				  gpointer data)
 {
-  char **ptr = data;
-  const char *str;
+  st_preferences_commit_string(GTK_ENTRY(widget));
 
-  g_return_val_if_fail(GTK_IS_ENTRY(widget), FALSE);
-  g_return_val_if_fail(ptr != NULL, FALSE);
-
-  str = gtk_entry_get_text(GTK_ENTRY(widget));
-
-  G_LOCK(st_settings);
-  g_free(*ptr);
-  *ptr = str[0] ? g_strdup(str) : NULL;
-  G_UNLOCK(st_settings);
-
-  return FALSE;
+  return FALSE;			/* propagate event */
 }
 
 void
@@ -114,8 +123,7 @@ st_preferences_bind_int_spin (GtkSpinButton *spin, int *ptr)
   g_return_if_fail(ptr != NULL);
 
   gtk_spin_button_set_value(spin, *ptr);
-  g_signal_connect(G_OBJECT(spin), "value-changed",
-		   G_CALLBACK(st_preferences_spin_value_changed_h), ptr);
+  g_signal_connect(spin, "value-changed", G_CALLBACK(st_preferences_spin_value_changed_h), ptr);
 }
 
 static void
@@ -139,8 +147,7 @@ st_preferences_bind_int_radio (GtkRadioButton *radio, int *ptr, int value)
 
   g_object_set_data(G_OBJECT(radio), "value", GINT_TO_POINTER(value));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), value == *ptr);
-  g_signal_connect(G_OBJECT(radio), "toggled",
-		   G_CALLBACK(st_preferences_radio_toggled_h), ptr);
+  g_signal_connect(radio, "toggled", G_CALLBACK(st_preferences_radio_toggled_h), ptr);
 }
 
 static void
