@@ -33,12 +33,13 @@
 #include <string.h>
 #include <glib.h>
 #include <glib/gi18n.h>
-#include "art/preselections.h"
 #include "sg-util.h"
 #include "st-action.h"
 #include "st-handler.h"
 #include "st-handlers-api.h"
 #include "st-stream-store.h"
+#include "st-util.h"
+#include "st-util-api.h"
 
 /*** cpp *********************************************************************/
 
@@ -207,15 +208,30 @@ st_preselections_stream_stock_field_get_cb (STPreselectionsStream *stream,
       g_value_set_string(value, stream->genre);
       break;
 
-    default:
-      /*
-       * This can not happen.
-       *
-       * However, handlers not shipped with streamtuner MUST ignore
-       * unknown stock fields, to stay compatible with future versions
-       * of streamtuner.
-       */
-      g_return_if_reached();
+    case ST_HANDLER_STOCK_FIELD_DESCRIPTION:
+      /* nop */
+      break;
+
+    case ST_HANDLER_STOCK_FIELD_HOMEPAGE:
+      g_value_set_string(value, stream->homepage);
+      break;
+
+    case ST_HANDLER_STOCK_FIELD_URI_LIST:
+      {
+	GValueArray *value_array;
+	GValue uri_value = { 0, };
+
+	value_array = g_value_array_new(1);
+
+	g_value_init(&uri_value, G_TYPE_STRING);
+	g_value_set_string(&uri_value, stream->url);
+	g_value_array_append(value_array, &uri_value);
+	g_value_unset(&uri_value);
+
+	g_value_take_boxed(value, value_array);
+
+	break;
+      }
     }
 }
 
@@ -303,13 +319,22 @@ st_preselections_stream_browse_cb (STPreselectionsStream *stream,
 static void
 st_preselections_init_handler (void)
 {
+  GdkPixbuf *pixbuf;
+  STHandlerField *field;
+
   st_preselections_handler = st_handler_new("preselections");
 
   st_handler_set_label(st_preselections_handler, _("Preselections"));
   st_handler_set_copyright(st_preselections_handler, "Copyright \302\251 2004 Jean-Yves Lefort");
   st_handler_set_description(st_preselections_handler, _("Manually Edited Streams"));
-  st_handler_set_icon_from_inline(st_preselections_handler, sizeof(art_preselections), art_preselections);
   st_handler_set_flags(st_preselections_handler, ST_HANDLER_NO_CATEGORIES);
+
+  pixbuf = st_pixbuf_new_from_file(ST_PNG("preselections"));
+  if (pixbuf)
+    {
+      st_handler_set_icon_from_pixbuf(st_preselections_handler, pixbuf);
+      g_object_unref(pixbuf);
+    }
   
   st_handler_bind(st_preselections_handler, ST_HANDLER_EVENT_STREAM_NEW, st_preselections_stream_new_cb, NULL);
   st_handler_bind(st_preselections_handler, ST_HANDLER_EVENT_STREAM_FIELD_GET, st_preselections_stream_field_get_cb, NULL);
@@ -323,32 +348,39 @@ st_preselections_init_handler (void)
   st_handler_bind(st_preselections_handler, ST_HANDLER_EVENT_STREAM_RECORD, st_preselections_stream_record_cb, NULL);
   st_handler_bind(st_preselections_handler, ST_HANDLER_EVENT_STREAM_BROWSE, st_preselections_stream_browse_cb, NULL);
 
-  st_handler_add_field(st_preselections_handler,
-		       st_handler_field_new(FIELD_NAME,
-					    _("Name"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE));
-  st_handler_add_field(st_preselections_handler,
-		       st_handler_field_new(FIELD_GENRE,
-					    _("Genre"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE));
-  st_handler_add_field(st_preselections_handler,
-		       st_handler_field_new(FIELD_HOMEPAGE,
-					    _("Homepage"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE
-					    | ST_HANDLER_FIELD_START_HIDDEN));
-  st_handler_add_field(st_preselections_handler,
-		       st_handler_field_new(FIELD_URL,
-					    _("URL"),
-					    G_TYPE_STRING,
-					    ST_HANDLER_FIELD_VISIBLE
-					    | ST_HANDLER_FIELD_EDITABLE
-					    | ST_HANDLER_FIELD_START_HIDDEN));
+  field = st_handler_field_new(FIELD_NAME,
+			       _("Name"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE);
+  st_handler_field_set_description(field, _("The preselection name"));
+  st_handler_add_field(st_preselections_handler, field);
+
+  field = st_handler_field_new(FIELD_GENRE,
+			       _("Genre"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE);
+  st_handler_field_set_description(field, _("The preselection genre"));
+  st_handler_add_field(st_preselections_handler, field);
+
+  field = st_handler_field_new(FIELD_HOMEPAGE,
+			       _("Homepage"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE
+			       | ST_HANDLER_FIELD_START_HIDDEN);
+  st_handler_field_set_description(field, _("The preselection homepage URL"));
+  st_handler_add_field(st_preselections_handler, field);
+
+  field = st_handler_field_new(FIELD_URL,
+			       _("URL"),
+			       G_TYPE_STRING,
+			       ST_HANDLER_FIELD_VISIBLE
+			       | ST_HANDLER_FIELD_EDITABLE
+			       | ST_HANDLER_FIELD_START_HIDDEN);
+  st_handler_field_set_description(field, _("The preselection listen URL"));
+  st_handler_add_field(st_preselections_handler, field);
 
   st_handlers_add(st_preselections_handler);
 }
@@ -359,7 +391,7 @@ st_preselections_init (void)
   st_preselections_init_handler();
   
   st_action_register("play-stream", _("Listen to a stream"), "xmms %q");
-  st_action_register("record-stream", _("Record a stream"), "xterm -hold -e streamripper %q");
+  st_action_register("record-stream", _("Record a stream"), "xterm -e streamripper %q");
   st_action_register("view-web", _("Open a web page"), "epiphany %q");
 }
 
@@ -383,37 +415,25 @@ st_preselections_add_stock (void)
 	  "Virgin Radio 1215AM",
 	  "Eclectic",
 	  "http://www.virginradio.co.uk/",
-	  "http://ogg.smgradio.com/vr96.ogg"
+	  "http://ogg.smgradio.com/vr160.ogg"
 	},
 	{
-	  "Virgin Radio Classic Tracks",
-	  "Eclectic",
+	  "Virgin Radio Classic Rock",
+	  "Classic Rock",
 	  "http://www.virginradio.co.uk/",
-	  "http://ogg.smgradio.com/vc96.ogg"
+	  "http://ogg.smgradio.com/vc160.ogg"
 	},
 	{
 	  "The Virgin Radio Groove",
 	  "Soul",
 	  "http://www.virginradio.co.uk/",
-	  "http://ogg.smgradio.com/gr96.ogg"
-	},
-	{
-	  "Liquid",
-	  "Alternative Pop",
-	  "http://www.virginradio.co.uk/",
-	  "http://ogg.smgradio.com/lq96.ogg"
+	  "http://ogg.smgradio.com/gr160.ogg"
 	},
 	{
 	  "Kill Radio",
 	  "Eclectic",
 	  "http://www.killradio.org/",
-	  "http://216.240.136.96:8002"
-	},
-	{
-	  "Basic.ch",
-	  "DJ mixes",
-	  "http://www.basic.ch/",
-	  "http://live.infomaniak.ch:8094/"
+	  "http://slaptech.net:8000/killradio64.mp3"
 	},
 	{
 	  "DFM Radio Television International",
@@ -421,12 +441,6 @@ st_preselections_add_stock (void)
 	  "http://dfm.nu/",
 	  "http://194.109.209.34:8000/dfm_1"
 	},
-	{
-	  "Power FM Radio",
-	  "Eclectic",
-	  "http://www.powerfmradio.com/",
-	  "http://live.str3am.com:2290"
-	},
 	{
 	  "Blazeradio",
 	  "Eclectic",
@@ -522,6 +536,7 @@ st_preselections_new (void)
   st_preselections_make_unique_names(stream_bag, preselections);
 
   st_stream_store_append(preselections, stream_bag);
+  st_stream_store_touch(preselections);
   g_object_unref(preselections);
 
   /* we don't unref stream_bag, that's caller's responsability */
