@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2003 Jean-Yves Lefort
+ * Copyright (c) 2002, 2003, 2004 Jean-Yves Lefort
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,10 +49,9 @@ st_programs_register (const char *action, const char *command)
   char *new_command;
 
   g_return_if_fail(action != NULL);
-  g_return_if_fail(command != NULL);
 
   label = st_plugin_current
-    ? g_strdup_printf(_("%s (deprecated, used by %s)"), action, st_plugin_current)
+    ? g_strdup_printf(_("%s (deprecated, used by %s)"), action, st_plugin_get_filename(st_plugin_current))
     : g_strdup_printf(_("%s (deprecated)"), action);
   
   new_command = st_action_convert_old_style_command(command);
