@@ -36,46 +36,68 @@
 
 /*** API implementation ******************************************************/
 
-/*
- * If STR matches REGEXP, returns TRUE and stores newly-allocated substrings
- * into the ... list. Otherwise, returns FALSE.
+/**
+ * st_re_parse:
+ * @regexp: a regular expression to match.
+ * @str: a string to match @regexp against.
+ * @...: a list of char ** pointers for storing substrings of
+ * @str. The number of char ** pointers provided must be equal to
+ * @regexp->re_nsub. The strings stored in the pointers should be freed
+ * when no longer needed.
  *
- * You should make sure the number of variable arguments is equal to
- * REGEXP->re_nsub.
- */
+ * Checks if @str matches @regexp, and if it does, distributes the
+ * substrings of @str matched by the parenthesized subexpressions of
+ * @regexp.
+ *
+ * <informalexample><programlisting>
+ * int status;
+ * regex_t re;
+ * gboolean matches;
+ * char *value1, *value2;
+ * &nbsp;
+ * status = regcomp(&amp;re, "name1=(.*) name2=(.*)", REG_EXTENDED);
+ * g_assert(status == 0);
+ * &nbsp;
+ * matches = st_re_parse(&amp;re, "name1=foo name2=bar", &amp;value1, &amp;value2);
+ * </programlisting></informalexample>
+ *
+ * Return value: %TRUE if @str matches @regexp.
+ **/
 gboolean
-st_re_parse (regex_t *regexp, const char *str, ...)
+st_re_parse (const regex_t *regexp, const char *str, ...)
 {
-  regmatch_t pmatch[regexp->re_nsub + 1];
-
   g_return_val_if_fail(regexp != NULL, FALSE);
   g_return_val_if_fail(str != NULL, FALSE);
-
-  if (regexec(regexp, str, regexp->re_nsub + 1, pmatch, 0) == 0)
-    {
-      va_list ap;
-
-      char **sub;
-      int i;
-
-      va_start(ap, str);
-      
-      for (i = 0; i < regexp->re_nsub; i++)
-	{
-	  int index = i + 1;
-
-	  sub = va_arg(ap, char **);
-
-	  g_return_val_if_fail(sub != NULL, FALSE);
-
-	  *sub = g_strndup(&str[pmatch[index].rm_so],
-			   pmatch[index].rm_eo - pmatch[index].rm_so);
-	}
-	  
-      va_end(ap);
-      
-      return TRUE;
-    }
+  
+  {
+    regmatch_t pmatch[regexp->re_nsub + 1];
+    
+    if (regexec(regexp, str, regexp->re_nsub + 1, pmatch, 0) == 0)
+      {
+	va_list ap;
+	
+	char **sub;
+	int i;
+	
+	va_start(ap, str);
+	
+	for (i = 0; i < regexp->re_nsub; i++)
+	  {
+	    int index = i + 1;
+	    
+	    sub = va_arg(ap, char **);
+	    
+	    g_return_val_if_fail(sub != NULL, FALSE);
+	    
+	    *sub = g_strndup(&str[pmatch[index].rm_so],
+			     pmatch[index].rm_eo - pmatch[index].rm_so);
+	  }
+	
+	va_end(ap);
+	
+	return TRUE;
+      }
+  }
 
   return FALSE;
 }
