@@ -43,15 +43,10 @@ struct _STSearchDialogPrivate
   GtkWidget	*entry;
 };
   
-/*** variable declarations ***************************************************/
-
-static GObjectClass *parent_class = NULL;
-
 /*** function declarations ***************************************************/
 
 static void st_search_dialog_class_init (STSearchDialogClass *class);
 static void st_search_dialog_init (STSearchDialog *dialog);
-static void st_search_dialog_finalize (GObject *object);
 
 static void st_search_dialog_update_sensitivity (STSearchDialog *dialog);
 static void st_search_dialog_entry_changed_h (GtkEditable *editable,
@@ -90,11 +85,7 @@ st_search_dialog_get_type (void)
 static void
 st_search_dialog_class_init (STSearchDialogClass *class)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS(class);
-
-  parent_class = g_type_class_peek_parent(class);
-
-  object_class->finalize = st_search_dialog_finalize;
+  g_type_class_add_private(class, sizeof(STSearchDialogPrivate));
 }
 
 static void
@@ -103,7 +94,7 @@ st_search_dialog_init (STSearchDialog *dialog)
   GtkWidget *label;
   GtkWidget *table;
 
-  dialog->priv = g_new0(STSearchDialogPrivate, 1);
+  dialog->priv = G_TYPE_INSTANCE_GET_PRIVATE(dialog, ST_TYPE_SEARCH_DIALOG, STSearchDialogPrivate);
 
   gtk_dialog_add_buttons(GTK_DIALOG(dialog),
 			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -148,16 +139,6 @@ st_search_dialog_init (STSearchDialog *dialog)
   st_search_dialog_update_sensitivity(dialog);
 }
 
-static void
-st_search_dialog_finalize (GObject *object)
-{
-  STSearchDialog *dialog = ST_SEARCH_DIALOG(object);
-
-  g_free(dialog->priv);
-
-  G_OBJECT_CLASS(parent_class)->finalize(object);
-}
-
 static void
 st_search_dialog_update_sensitivity (STSearchDialog *dialog)
 {
