@@ -30,6 +30,7 @@
  */
 
 #include "config.h"
+#include <string.h>
 #include <errno.h>
 #include <glib.h>
 #include <sys/types.h>
@@ -38,13 +39,17 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include "sg-util.h"
-#include "st-cache-load.h"
-#include "st-cache-save.h"
+#include "st-cache.h"
 #include "st-config-load.h"
 #include "st-config-save.h"
 #include "st-dialog-api.h"
 #include "st-settings.h"
 #include "st-preselections.h"
+#include "st-handlers.h"
+#include "st-handler.h"
+#include "st-category-store.h"
+#include "st-oldcache-load.h"
+#include "st-util-api.h"
 
 /*** function declarations ***************************************************/
 
@@ -56,6 +61,7 @@ void
 st_session_load (void)
 {
   GError *err = NULL;
+  GSList *l;
   
   if (g_file_test(st_settings.accels_file, G_FILE_TEST_IS_REGULAR))
     gtk_accel_map_load(st_settings.accels_file);
@@ -74,22 +80,94 @@ st_session_load (void)
 	  g_clear_error(&err);
 	}
     }
-  
-  if (g_file_test(st_settings.cache_file, G_FILE_TEST_IS_REGULAR))
-    {
-      if (! st_cache_load(st_settings.cache_file, &err))
+
+  if (g_file_test(st_settings.cache_dir, G_FILE_TEST_IS_REGULAR))
+    {				/* old cache */
+      if (! st_oldcache_load(st_settings.cache_dir, &err))
 	{
 	  char *normalized;
 
 	  normalized = st_dialog_normalize(err->message);
-	  
-	  st_error_dialog(_("Unable to load cache"), "%s", normalized);
+
+	  st_error_dialog(_("Unable to load old cache"), "%s", normalized);
 
 	  g_free(normalized);
 	  g_clear_error(&err);
 	}
     }
+  else
+    SG_LIST_FOREACH(l, st_handlers_list)
+      {
+	STHandler *handler = l->data;
+	const char *name;
 
+	if (ST_HANDLER_HAS_CATEGORIES(handler) && st_cache_has_categories(handler))
+	  {
+	    GNode *categories;
+	    
+	    categories = st_cache_load_categories(handler, &err);
+	    if (categories)
+	      {
+		STCategoryStore *category_store;
+		
+		category_store = st_handler_get_categories(handler);
+		st_category_store_append_node(category_store, categories);
+		
+		g_object_unref(category_store);
+	      }
+	    else
+	      {
+		char *primary;
+		char *normalized;
+		
+		primary = g_strdup_printf(_("Unable to load categories of handler \"%s\" from cache"), st_handler_get_label(handler));
+		normalized = st_dialog_normalize(err->message);
+		
+		st_error_dialog(primary, "%s", normalized);
+		
+		g_free(primary);
+		g_free(normalized);
+		
+		g_clear_error(&err);
+	      }
+	  }
+	
+	name = st_handler_get_name(handler);
+	if (! strcmp(name, "bookmarks") || ! strcmp(name, "preselections"))
+	  {
+	    if (st_cache_has_streams(handler, ST_CATEGORY_BAG_MAIN))
+	      {
+		STStreamStore *streams;
+		
+		streams = st_cache_load_streams(handler, ST_CATEGORY_BAG_MAIN, NULL, NULL, &err);
+		if (streams)
+		  {
+		    st_handler_set_streams(handler, ST_CATEGORY_BAG_MAIN, streams);
+		    g_object_unref(streams);
+		  }
+		else
+		  {
+		    const char *primary;
+		    char *normalized;
+		    
+		    if (! strcmp(name, "bookmarks"))
+		      primary = _("Unable to load bookmarks from cache");
+		    else if (! strcmp(name, "preselections"))
+		      primary = _("Unable to load preselections from cache");
+		    else
+		      g_return_if_reached();
+		    
+		    normalized = st_dialog_normalize(err->message);
+		    
+		    st_error_dialog(primary, "%s", normalized);
+		    
+		    g_free(normalized);
+		    g_clear_error(&err);
+		  }
+	      }
+	  }
+      }
+  
   st_preselections_add_stock();
 }
 
@@ -98,11 +176,53 @@ st_session_save (void)
 {
   GError *err = NULL;
 
+  /* remove old cache if present */
+
+  if (g_file_test(st_settings.cache_dir, G_FILE_TEST_IS_REGULAR))
+    unlink(st_settings.cache_dir);
+
+  if (g_file_test(st_settings.images_dir, G_FILE_TEST_IS_DIR))
+    {
+      GDir *dir;
+
+      dir = g_dir_open(st_settings.images_dir, 0, NULL);
+      if (dir)
+	{
+	  const char *filename;
+
+	  while ((filename = g_dir_read_name(dir)))
+	    {
+	      char *extension;
+
+	      extension = strrchr(filename, '.');
+	      if (extension && ! strcmp(extension, ".png"))
+		{
+		  char *base;
+
+		  base = g_strndup(filename, extension - filename);
+		  if (st_str_like(base, ST_NUMERIC))
+		    {
+		      char *pathname;
+
+		      pathname = g_build_filename(st_settings.images_dir, filename, NULL);
+		      unlink(pathname);
+		      g_free(pathname);
+		    }
+		  g_free(base);
+		}
+	    }
+
+	  g_dir_close(dir);
+	}
+
+      rmdir(st_settings.images_dir);
+    }
+  
   /* create directories */
 
   if (! st_session_mkdir(st_settings.private_dir))
     return;
-  if (! st_session_mkdir(st_settings.images_dir))
+  if (! st_session_mkdir(st_settings.cache_dir))
     return;
 
   /* save accel map */
@@ -125,7 +245,7 @@ st_session_save (void)
   
   /* save cache */
 
-  if (! st_cache_save(st_settings.cache_file, &err))
+  if (! st_cache_save(&err))
     {
       char *normalized;
       
