@@ -34,6 +34,7 @@
 #include <limits.h>
 #include <glib.h>
 #include "st-entities.h"
+#include "st-sgml-ref-api.h"
 
 /*** function declarations ***************************************************/
 
@@ -89,49 +90,76 @@ st_sgml_ref_get_unichar (const char *ref)
   return c;
 }
 
-/*
- * Parse STR, possibly containing SGML character references, and
- * return a newly-allocated UTF-8 string with the references expanded
- * to their Unicode character.
- */
+/**
+ * st_sgml_ref_expand:
+ * @str: the nul-terminated string to parse.
+ *
+ * Parses @str, expanding its SGML character references and XHTML
+ * character entities. See st_sgml_ref_expand_len().
+ *
+ * Return value: the expansion of @str. The string should be freed
+ * when no longer needed.
+ **/
 char *
 st_sgml_ref_expand (const char *str)
+{
+  g_return_val_if_fail(str != NULL, NULL);
+
+  return st_sgml_ref_expand_len(str, -1);
+}
+
+/**
+ * st_sgml_ref_expand_len:
+ * @str: the string to parse.
+ * @len: the length of @str, or -1 if @str is nul-terminated.
+ *
+ * Parses @str, expanding its SGML character references and XHTML
+ * character entities.
+ *
+ * Numerical SGML character references as well as XHTML entities are
+ * supported. Unsupported entities will be inserted verbatim into the
+ * result.
+ *
+ * Return value: the expansion of @str. The string should be freed
+ * when no longer needed.
+ **/
+char *
+st_sgml_ref_expand_len (const char *str, int len)
 {
   GString *unescaped;
   char *ampersand;
-  char *start;
+  char *semicolon;
 
   g_return_val_if_fail(str != NULL, NULL);
 
+  if (len == -1)
+    len = strlen(str);
+
   unescaped = g_string_new(NULL);
-  start = (char *) str;
 
-  while ((ampersand = strchr(start, '&')))
-    {
-      char *semicolon = strchr(ampersand, ';');
+  while ((ampersand = g_strstr_len(str, len, "&")))
+    if ((semicolon = g_strstr_len(ampersand, len - (ampersand - str), ";")))
+      {
+	char *ref;
+	gunichar c;
+	
+	ref = g_strndup(ampersand + 1, semicolon - ampersand - 1);
+	c = st_sgml_ref_get_unichar(ref);
+	g_free(ref);
+	
+	g_string_append_len(unescaped, str, ampersand - str);
+	if (c)
+	  g_string_append_unichar(unescaped, c);
+	else			/* invalid reference, append it raw */
+	  g_string_append_len(unescaped, ampersand, semicolon - ampersand + 1);
+	
+	len -= semicolon - str + 1;
+	str = semicolon + 1;
+      }
+    else
+      break;
 
-      if (semicolon)
-	{
-	  char *ref;
-	  gunichar c;
-
-	  ref = g_strndup(ampersand + 1, semicolon - ampersand - 1);
-	  c = st_sgml_ref_get_unichar(ref);
-	  g_free(ref);
-
-	  g_string_append_len(unescaped, start, ampersand - start);
-	  if (c)
-	    g_string_append_unichar(unescaped, c);
-	  else			/* invalid reference, append it raw */
-	    g_string_append_len(unescaped, ampersand, semicolon - ampersand + 1);
-
-	  start = semicolon + 1;
-	}
-      else
-	break;
-    }
-
-  g_string_append(unescaped, start);
+  g_string_append_len(unescaped, str, len);
 
   return g_string_free(unescaped, FALSE);
 }
