@@ -31,9 +31,10 @@
 
 #include "config.h"
 #include <string.h>
+#include <unistd.h>
+#include <errno.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include "art/streamtuner.h"
 #include "sg-util.h"
 #include "sgtk-hig.h"
 #include "sgtk-util.h"
@@ -57,80 +58,88 @@
 #include "st-main.h"
 #include "st-browser-tab-label.h"
 #include "st-stream-store.h"
-#include "st-stream-task.h"
-#include "st-bookmarks.h"
 #include "st-util.h"
+#include "st-stream-columns-dialog.h"
+#include "st-session.h"
+
+/*** cpp *********************************************************************/
+
+#define FIND_NAVIGATION_SENSITIVE \
+  (st_settings.find_token && *st_settings.find_token)
 
 /*** type definitions ********************************************************/
 
 struct _STShellPrivate
 {
-  GtkWidget		*window;
-  GtkAccelGroup		*accel_group;
-  GtkWidget		*box;
+  GtkWidget			*window;
+  GtkAccelGroup			*accel_group;
+  GtkWidget			*box;
 
   /* menubar */
-  GtkItemFactory	*factory;
-  GtkWidget		*menubar;
-  STStreamMenuItems	*stream_items;
-  GtkWidget		*select_all_item;
-  GtkWidget		*find_item;
-  GtkWidget		*find_next_item;
-  GtkWidget		*find_previous_item;
-  GtkWidget		*search_in_all_categories_item;
-  GtkWidget		*stop_item;
-  GtkWidget		*reload_item;
-  GtkWidget		*view_tab_icons_item;
-  GtkWidget		*stream_columns_item;
-  GtkWidget		*directories_menu;
-  GtkWidget		*previous_directory_item;
-  GtkWidget		*next_directory_item;
-  GtkWidget		*move_directory_left_item;
-  GtkWidget		*move_directory_right_item;
-  GtkWidget		*directory_homepage_item;
-  GSList		*directory_items;
+  GtkItemFactory		*factory;
+  GtkWidget			*menubar;
+  STStreamMenuItems		*stream_items;
+  GtkWidget			*select_all_item;
+  GtkWidget			*find_item;
+  GtkWidget			*find_next_item;
+  GtkWidget			*find_previous_item;
+  GtkWidget			*search_in_all_categories_item;
+  GtkWidget			*stop_item;
+  GtkWidget			*reload_item;
+  GtkWidget			*view_tab_icons_item;
+  GtkWidget			*directories_menu;
+  GtkWidget			*directory_preferences_item;
+  GtkWidget			*directory_homepage_item;
+  GtkWidget			*previous_directory_item;
+  GtkWidget			*next_directory_item;
+  GtkWidget			*move_directory_left_item;
+  GtkWidget			*move_directory_right_item;
+  GSList			*directory_items;
 
   /* toolbar */
-  GtkWidget		*toolbar;
-  GtkWidget		*tune_in_button;
-  GtkWidget		*record_button;
-  GtkWidget		*browse_button;
-  GtkWidget		*stop_button;
-  GtkWidget		*reload_button;
-  GtkWidget		*link;
+  GtkWidget			*toolbar;
+  GtkWidget			*tune_in_button;
+  GtkWidget			*record_button;
+  GtkWidget			*browse_button;
+  GtkWidget			*stop_button;
+  GtkWidget			*reload_button;
+  GtkWidget			*link;
 
   /* notebook */
-  GtkWidget		*notebook;
-  unsigned int		switch_page_hid;
+  GtkWidget			*notebook;
+  unsigned int			switch_page_hid;
 
   /* statusbox */
-  GtkWidget		*statusbox;
+  GtkWidget			*statusbox;
 
-  GSList		*tabs;
-  char			*find_token;
-  gboolean		find_case_sensitive;
+  GSList			*tabs;
+  char				*find_token;
+  gboolean			find_case_sensitive;
 
-  GSList		*tasks;
-  STThread		*tune_in_thread;
+  STBrowserTabStreamTask	*tune_in_task;
+
+  gboolean			restart;
 
   /* dialogs */
-  GtkWidget		*stream_properties;
-  GtkWidget		*find;
-  GtkWidget		*preferences;
-  GtkWidget		*about;
+  GtkWidget			*stream_properties;
+  GtkWidget			*stream_columns;
+  GtkWidget			*find;
+  GtkWidget			*preferences;
+  GtkWidget			*about;
 
   /* fullscreen mode */
-  gboolean		fullscreen;
-  GtkWidget		*leave_fullscreen_popup;
-  gboolean		leave_fullscreen_popup_visible;
-  unsigned int		leave_fullscreen_timeout_id;
-  unsigned int		screen_size_changed_hid;
-  unsigned int		key_press_event_hid;
+  gboolean			fullscreen;
+  GtkWidget			*leave_fullscreen_popup;
+  gboolean			leave_fullscreen_popup_visible;
+  unsigned int			leave_fullscreen_timeout_id;
+  unsigned int			screen_size_changed_hid;
+  unsigned int			key_press_event_hid;
 };
 
 /*** variable declarations ***************************************************/
 
 static GObjectClass *parent_class = NULL;
+STShell *st_shell = NULL;
 
 /*** function declarations ***************************************************/
 
@@ -217,6 +226,8 @@ static void st_shell_tab_moved (STShell *shell);
 static void st_shell_update_visibility (STShell *shell);
 static void st_shell_update_toolbar_style (STShell *shell);
 static void st_shell_update_toolbar_size (STShell *shell);
+static void st_shell_update_title (STShell *shell);
+static void st_shell_update_sensitivity (STShell *shell);
 
 static STBrowserTab *st_shell_get_selected_tab (STShell *shell);
 
@@ -262,6 +273,9 @@ static void st_shell_dialog_response_h (GtkDialog *dialog,
 					int response,
 					gpointer data);
 
+static gboolean st_shell_can_present_directory_preferences (STShell *shell);
+static void st_shell_present_directory_preferences (STShell *shell);
+
 static gboolean st_shell_can_visit_directory_homepage (STShell *shell);
 static void st_shell_visit_directory_homepage (STShell *shell);
 
@@ -285,13 +299,15 @@ static void st_shell_move_tab_right (STShell *shell);
 static void st_shell_help (STShell *shell);
 static void st_shell_homepage (STShell *shell);
 
+static void st_shell_new_preselection (STShell *shell);
+
 static STBrowserTab *st_shell_get_tab_with_handler (STShell *shell,
 						    STHandler *handler);
 static void st_shell_select_tab (STShell *shell,
 				 STBrowserTab *tab,
 				 gboolean force_update);
 
-static void st_shell_launch_stream_task (STShell *shell, STStreamTask task);
+static void st_shell_set_statusbar_of_tab (STShell *shell, STBrowserTab *tab);
 
 static void st_shell_category_selection_changed_h (STCategoryView *view,
 						   gpointer user_data);
@@ -301,9 +317,15 @@ static void st_shell_stream_activated_h (GtkTreeView *treeview,
 					 GtkTreePath *arg1,
 					 GtkTreeViewColumn *arg2,
 					 gpointer user_data);
-static void st_shell_tab_reloading_changed_h (STBrowserTab *tab,
-					      gboolean reloading,
+static void st_shell_tab_notify_running_h (GObject *object,
+					   GParamSpec *pspec,
+					   gpointer user_data);
+static void st_shell_tab_stream_task_added_h (STBrowserTab *tab,
+					      STBrowserTabStreamTask *task,
 					      gpointer user_data);
+static void st_shell_tab_stream_task_removed_h (STBrowserTab *tab,
+						STBrowserTabStreamTask *task,
+						gpointer user_data);
 
 /*** implementation **********************************************************/
 
@@ -342,6 +364,8 @@ st_shell_class_init (STShellClass *class)
 
   parent_class = g_type_class_peek_parent(class);
 
+  g_type_class_add_private(class, sizeof(STShellPrivate));
+
   object_class->finalize = st_shell_finalize;
 }
 
@@ -350,7 +374,10 @@ st_shell_init (STShell *shell)
 {
   STHandler *handler = NULL;
 
-  shell->priv = g_new0(STShellPrivate, 1);
+  g_return_if_fail(st_shell == NULL);
+  st_shell = shell;
+
+  shell->priv = G_TYPE_INSTANCE_GET_PRIVATE(shell, ST_TYPE_SHELL, STShellPrivate);
 
   st_shell_make_window(shell);
   st_shell_make_menubar(shell);
@@ -386,39 +413,65 @@ static void
 st_shell_finalize (GObject *object)
 {
   STShell *shell = ST_SHELL(object);
+  GSList *l;
+  STBrowserTab *selected_tab;
 
-  /* destroy the window */
+  /* stop everything */
   
+  SG_LIST_FOREACH(l, shell->priv->tabs)
+    if (st_browser_tab_can_stop(l->data))
+      st_browser_tab_stop(l->data);
+
+  /* store some settings */
+  
+  selected_tab = st_shell_get_selected_tab(shell);
+  if (selected_tab)
+    {
+      g_free(st_settings.selected_handler_name);
+      st_settings.selected_handler_name = g_strdup(st_handler_get_name(selected_tab->handler));
+    }
+
+  /* destroy the main window before the final processing (HIG 2.0) */
+
   gtk_widget_destroy(shell->priv->window);
 
-  /* cleanup */
+  while (gtk_events_pending())
+    gtk_main_iteration();
 
-  g_object_unref(shell->priv->accel_group);
-  g_object_unref(shell->priv->factory);
-  st_stream_menu_items_free(shell->priv->stream_items);
-  g_slist_free(shell->priv->directory_items);
-  g_slist_free(shell->priv->tabs);
+  /* save session and restart or quit */
 
-  g_free(shell->priv);
+  st_session_save();
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  if (shell->priv->restart)
+    {
+      char *secondary;
+      char *normalized;
+
+      execvp(st_main_argv[0], st_main_argv);
+
+      secondary = g_strdup_printf(_("Unable to restart streamtuner: %s"), g_strerror(errno));
+      normalized = st_dialog_normalize(secondary);
+      g_free(secondary);
+
+      st_error_dialog(_("A fatal error has occurred"), "%s", normalized);
+      g_free(normalized);
+    }
+  
+  gtk_main_quit();
+
+  parent_class->finalize(object);
 }
 
 static void
 st_shell_make_window (STShell *shell)
 {
-  GdkPixbuf *icon;
-
   shell->priv->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  g_object_add_weak_pointer(G_OBJECT(shell->priv->window), (gpointer *) &shell->priv->window);
+
   shell->priv->accel_group = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(shell->priv->window), shell->priv->accel_group);
 
-  icon = gdk_pixbuf_new_from_inline(sizeof(art_streamtuner),
-				    art_streamtuner,
-				    FALSE,
-				    NULL);
-  gtk_window_set_icon(GTK_WINDOW(shell->priv->window), icon);
-  g_object_unref(icon);
+  st_window_set_icon(GTK_WINDOW(shell->priv->window));
 
   shell->priv->box = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(shell->priv->window), shell->priv->box);
@@ -430,7 +483,7 @@ st_shell_make_window (STShell *shell)
 
   g_object_connect(shell->priv->window,
 		   "signal::window-state-event", st_shell_window_state_event_h, shell,
-		   "signal::delete-event", st_shell_window_delete_event_h, NULL,
+		   "signal::delete-event", st_shell_window_delete_event_h, shell,
 		   NULL);
 }
 
@@ -457,9 +510,11 @@ st_shell_window_delete_event_h (GtkWidget *widget,
 				GdkEvent *event,
 				gpointer user_data)
 {
-  st_main_quit();
+  STShell *shell = user_data;
 
-  return TRUE;
+  g_object_unref(shell);
+
+  return TRUE;			/* do not propagate event */
 }
 
 static void
@@ -645,6 +700,7 @@ st_shell_window_key_press_event_h (GtkWidget *widget,
 static void
 st_shell_make_menubar (STShell *shell)
 {
+  /* keep in sync with the stock items in st-stock.c */
   static GtkItemFactoryEntry entries[] = {
     {
       N_("/_Stream"),				NULL,
@@ -652,7 +708,7 @@ st_shell_make_menubar (STShell *shell)
       "<Branch>",				NULL
     },
     {
-      N_("/Stream/_New Preselection"),		"<Control>N",
+      N_("/Stream/_New Preselection..."),	"<Control>N",
       st_shell_new_preselection,		0,
       "<StockItem>",				ST_STOCK_NEW_PRESELECTION
     },
@@ -668,7 +724,7 @@ st_shell_make_menubar (STShell *shell)
     },
     {
       N_("/Stream/_Quit"),			NULL,
-      st_main_quit,				0,
+      g_object_unref,				0,
       "<StockItem>",				GTK_STOCK_QUIT
     },
     {
@@ -687,7 +743,7 @@ st_shell_make_menubar (STShell *shell)
       "<Separator>",				NULL
     },
     {
-      N_("/Edit/_Find"),			NULL,
+      N_("/Edit/_Find..."),			NULL,
       st_shell_find,				0,
       "<StockItem>",				GTK_STOCK_FIND
     },
@@ -707,7 +763,7 @@ st_shell_make_menubar (STShell *shell)
       "<Separator>",				NULL
     },
     {
-      N_("/Edit/_Search in All Categories"),	"<Shift><Control>S",
+      N_("/Edit/_Search in All Categories..."),	"<Shift><Control>S",
       st_shell_search_in_all_categories,	0,
       "<Item>",					NULL
     },
@@ -818,8 +874,8 @@ st_shell_make_menubar (STShell *shell)
     },
     {
       N_("/View/Stream _Columns"),		NULL,
-      NULL,					0,
-      "<Branch>",				NULL
+      st_shell_present_stream_columns,		0,
+      "<StockItem>",				ST_STOCK_STREAM_COLUMNS
     },
     {
       "/View/separator3",			NULL,
@@ -841,6 +897,11 @@ st_shell_make_menubar (STShell *shell)
       NULL,					0,
       "<Branch>",				NULL
     },
+    {
+      N_("/Directories/_Directory Preferences"), NULL,
+      st_shell_present_directory_preferences,	0,
+      "<StockItem>",				ST_STOCK_DIRECTORY_PREFERENCES
+    },
     {
       N_("/Directories/_Visit Directory Homepage"), NULL,
       st_shell_visit_directory_homepage,	0,
@@ -924,7 +985,7 @@ st_shell_make_menubar (STShell *shell)
 
   menu = gtk_item_factory_get_widget(shell->priv->factory, N_("/Stream"));
 
-  shell->priv->stream_items = st_stream_menu_items_new(shell, shell->priv->accel_group);
+  shell->priv->stream_items = st_stream_menu_items_new(shell->priv->accel_group);
   st_stream_menu_items_insert_into_shell(shell->priv->stream_items, GTK_MENU_SHELL(menu), 2);
 
   accel_path = g_strdup_printf("<streamtuner-Browser>/Stream/%s", _("Tune in"));
@@ -943,20 +1004,20 @@ st_shell_make_menubar (STShell *shell)
 
   shell->priv->menubar = gtk_item_factory_get_widget(shell->priv->factory, "<streamtuner-Browser>");
   shell->priv->select_all_item = gtk_item_factory_get_item(shell->priv->factory, N_("/Edit/Select All"));
-  shell->priv->find_item = gtk_item_factory_get_item(shell->priv->factory, N_("/Edit/Find"));
+  shell->priv->find_item = gtk_item_factory_get_item(shell->priv->factory, N_("/Edit/Find..."));
   shell->priv->find_next_item = gtk_item_factory_get_item(shell->priv->factory, N_("/Edit/Find Next"));
   shell->priv->find_previous_item = gtk_item_factory_get_item(shell->priv->factory, N_("/Edit/Find Previous"));
-  shell->priv->search_in_all_categories_item = gtk_item_factory_get_item(shell->priv->factory, N_("/Edit/Search in All Categories"));
+  shell->priv->search_in_all_categories_item = gtk_item_factory_get_item(shell->priv->factory, N_("/Edit/Search in All Categories..."));
   shell->priv->stop_item = gtk_item_factory_get_item(shell->priv->factory, N_("/View/Stop"));
   shell->priv->reload_item = gtk_item_factory_get_item(shell->priv->factory, N_("/View/Reload"));
   shell->priv->view_tab_icons_item = gtk_item_factory_get_item(shell->priv->factory, N_("/View/Tab Icons"));
-  shell->priv->stream_columns_item = gtk_item_factory_get_item(shell->priv->factory, N_("/View/Stream Columns"));
   shell->priv->directories_menu = gtk_item_factory_get_widget(shell->priv->factory, N_("/Directories"));
+  shell->priv->directory_preferences_item = gtk_item_factory_get_item(shell->priv->factory, N_("/Directories/Directory Preferences"));
+  shell->priv->directory_homepage_item = gtk_item_factory_get_item(shell->priv->factory, N_("/Directories/Visit Directory Homepage"));
   shell->priv->previous_directory_item = gtk_item_factory_get_widget(shell->priv->factory, N_("/Directories/Previous Directory"));
   shell->priv->next_directory_item = gtk_item_factory_get_widget(shell->priv->factory, N_("/Directories/Next Directory"));
   shell->priv->move_directory_left_item = gtk_item_factory_get_widget(shell->priv->factory, N_("/Directories/Move Directory Left"));
   shell->priv->move_directory_right_item = gtk_item_factory_get_widget(shell->priv->factory, N_("/Directories/Move Directory Right"));
-  shell->priv->directory_homepage_item = gtk_item_factory_get_item(shell->priv->factory, N_("/Directories/Visit Directory Homepage"));
 
   /* add directory items */
 
@@ -996,8 +1057,7 @@ st_shell_menubar_make_directory_items (STShell *shell)
   g_return_if_fail(ST_IS_SHELL(shell));
 
   /* destroy the old items */
-  SG_LIST_FOREACH(l, shell->priv->directory_items)
-    gtk_widget_destroy(l->data);
+  g_slist_foreach(shell->priv->directory_items, (GFunc) gtk_widget_destroy, NULL);
 
   g_slist_free(shell->priv->directory_items);
   shell->priv->directory_items = NULL;
@@ -1209,7 +1269,7 @@ st_shell_make_toolbar (STShell *shell)
 							-1);
   shell->priv->stop_button = gtk_toolbar_insert_stock(GTK_TOOLBAR(shell->priv->toolbar),
 						      GTK_STOCK_STOP,
-						      _("Stop the currently running tasks"),
+						      _("Stop the selected tab"),
 						      NULL,
 						      (GtkSignalFunc) st_shell_toolbar_button_clicked_h,
 						      shell,
@@ -1260,6 +1320,7 @@ st_shell_make_notebook (STShell *shell)
   GdkPixbuf *drag_pixbuf;
 
   shell->priv->notebook = gtk_notebook_new();
+  gtk_notebook_set_scrollable(GTK_NOTEBOOK(shell->priv->notebook), TRUE);
 
   /*
    * The tab labels will be moved in realtime by
@@ -1282,7 +1343,7 @@ st_shell_make_notebook (STShell *shell)
       tab = st_browser_tab_new(handler);
       tab_label = ST_BROWSER_TAB(tab)->label;
       
-      stream_menu_items = st_stream_menu_items_new(shell, shell->priv->accel_group);
+      stream_menu_items = st_stream_menu_items_new(shell->priv->accel_group);
       st_stream_view_set_menu_items(ST_BROWSER_TAB(tab)->stream_view, stream_menu_items);
 
       gtk_drag_source_set(tab_label,
@@ -1303,7 +1364,11 @@ st_shell_make_notebook (STShell *shell)
 		       "signal::drag-motion", st_shell_tab_label_drag_motion_h, shell,
 		       NULL);
 
-      g_signal_connect(tab, "reloading-changed", G_CALLBACK(st_shell_tab_reloading_changed_h), shell);
+      g_object_connect(tab,
+		       "signal::notify::running", st_shell_tab_notify_running_h, shell,
+		       "signal::stream-task-added", st_shell_tab_stream_task_added_h, shell,
+		       "signal::stream-task-removed", st_shell_tab_stream_task_removed_h, shell,
+		       NULL);
 
       if (ST_BROWSER_TAB(tab)->category_view)
 	g_signal_connect(ST_BROWSER_TAB(tab)->category_view,
@@ -1369,10 +1434,10 @@ st_shell_handlers_compare (gconstpointer a,
   STShell *shell = user_data;
   STBrowserTab *tab1 = st_shell_get_tab_with_handler(shell, (STHandler *) a);
   STBrowserTab *tab2 = st_shell_get_tab_with_handler(shell, (STHandler *) b);
-  int num1 = gtk_notebook_page_num(GTK_NOTEBOOK(shell->priv->notebook), GTK_WIDGET(tab1));
-  int num2 = gtk_notebook_page_num(GTK_NOTEBOOK(shell->priv->notebook), GTK_WIDGET(tab2));
+  int num1 = gtk_notebook_page_num(GTK_NOTEBOOK(shell->priv->notebook), (GtkWidget *) tab1);
+  int num2 = gtk_notebook_page_num(GTK_NOTEBOOK(shell->priv->notebook), (GtkWidget *) tab2);
 
-  return (num1 < num2) ? -1 : ((num1 == num2) ? 0 : 1);
+  return num1 - num2;
 }
 
 static void
@@ -1435,28 +1500,26 @@ st_shell_tab_selected (STShell *shell)
   /* update menubar */
 
   st_shell_menubar_select_directory_item(shell);
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(shell->priv->stream_columns_item), tab->stream_view->columns_menu);
 
-  /* pack tab's statusbar */
+  /* pack statusbar */
 
-  if (GTK_BIN(shell->priv->statusbox)->child)
-    gtk_container_remove(GTK_CONTAINER(shell->priv->statusbox), GTK_BIN(shell->priv->statusbox)->child);
-  gtk_container_add(GTK_CONTAINER(shell->priv->statusbox), tab->statusbar);
+  st_shell_set_statusbar_of_tab(shell, tab);
 
-  /* work-around a bug in the Bluecurve theme engine */
-  gtk_widget_realize(GTK_WIDGET(ST_STATUSBAR(tab->statusbar)->progress_bar));
+  /* misc */
 
-  if (! gtk_tree_view_get_model(GTK_TREE_VIEW(tab->stream_view)) && ! tab->reloading)
+  if (! gtk_tree_view_get_model(GTK_TREE_VIEW(tab->stream_view)))
     st_browser_tab_update(tab);
   
   st_shell_update_sensitivity(shell);
 
   if (shell->priv->stream_properties)
     st_stream_properties_dialog_update_sensitivity(ST_STREAM_PROPERTIES_DIALOG(shell->priv->stream_properties));
+  if (shell->priv->stream_columns)
+    st_stream_columns_dialog_set_stream_view(ST_STREAM_COLUMNS_DIALOG(shell->priv->stream_columns), tab->stream_view);
   if (shell->priv->find)
     {
-      sgtk_find_dialog_set_previous_sensitive(SGTK_FIND_DIALOG(shell->priv->find), TRUE);
-      sgtk_find_dialog_set_next_sensitive(SGTK_FIND_DIALOG(shell->priv->find), TRUE);
+      sgtk_find_dialog_set_previous_sensitive(SGTK_FIND_DIALOG(shell->priv->find), FIND_NAVIGATION_SENSITIVE);
+      sgtk_find_dialog_set_next_sensitive(SGTK_FIND_DIALOG(shell->priv->find), FIND_NAVIGATION_SENSITIVE);
     }
 }
 
@@ -1465,10 +1528,13 @@ st_shell_tab_moved (STShell *shell)
 {
   g_return_if_fail(ST_IS_SHELL(shell));
 
-  /* reorder the handlers list */
+  /* reorder the handlers lists */
   st_handlers_list = g_slist_sort_with_data(st_handlers_list,
 					    st_shell_handlers_compare,
 					    shell);
+  st_handlers_full_list = g_slist_sort_with_data(st_handlers_full_list,
+						 st_shell_handlers_compare,
+						 shell);
 
   /* reorder the directories menu by recreating it */
   st_shell_menubar_make_directory_items(shell);
@@ -1481,6 +1547,7 @@ static void
 st_shell_make_statusbox (STShell *shell)
 {
   shell->priv->statusbox = gtk_event_box_new();
+  gtk_event_box_set_visible_window(GTK_EVENT_BOX(shell->priv->statusbox), FALSE);
 
   gtk_box_pack_start(GTK_BOX(shell->priv->box), shell->priv->statusbox, FALSE, FALSE, 0);
   gtk_widget_show(shell->priv->statusbox);
@@ -1550,45 +1617,7 @@ st_shell_new (void)
   return g_object_new(ST_TYPE_SHELL, NULL);
 }
 
-void
-st_shell_show (STShell *shell)
-{
-  g_return_if_fail(ST_IS_SHELL(shell));
-
-  gtk_widget_show(shell->priv->window);
-}
-
-void
-st_shell_hide (STShell *shell)
-{
-  g_return_if_fail(ST_IS_SHELL(shell));
-
-  gtk_widget_hide(shell->priv->window);
-}
-
-void
-st_shell_prepare_finalize (STShell *shell)
-{
-  STBrowserTab *selected_tab;
-
-  g_return_if_fail(ST_IS_SHELL(shell));
-
-  /* stop everything */
-  
-  if (st_shell_can_stop(shell))
-    st_shell_stop(shell);
-
-  /* store some settings */
-  
-  selected_tab = st_shell_get_selected_tab(shell);
-  if (selected_tab)
-    {
-      g_free(st_settings.selected_handler_name);
-      st_settings.selected_handler_name = g_strdup(st_handler_get_name(selected_tab->handler));
-    }
-}
-
-void
+static void
 st_shell_update_title (STShell *shell)
 {
   STBrowserTab *selected_tab;
@@ -1619,11 +1648,9 @@ st_shell_update_title (STShell *shell)
   g_string_free(title, TRUE);
 }
 
-void
+static void
 st_shell_update_sensitivity (STShell *shell)
 {
-  GSList *l;
-
   g_return_if_fail(ST_IS_SHELL(shell));
 
   /* menubar */
@@ -1635,11 +1662,12 @@ st_shell_update_sensitivity (STShell *shell)
   gtk_widget_set_sensitive(shell->priv->search_in_all_categories_item, st_shell_can_search_in_all_categories(shell));
   gtk_widget_set_sensitive(shell->priv->stop_item, st_shell_can_stop(shell));
   gtk_widget_set_sensitive(shell->priv->reload_item, st_shell_can_reload(shell));
+  gtk_widget_set_sensitive(shell->priv->directory_preferences_item, st_shell_can_present_directory_preferences(shell));
+  gtk_widget_set_sensitive(shell->priv->directory_homepage_item, st_shell_can_visit_directory_homepage(shell));
   gtk_widget_set_sensitive(shell->priv->previous_directory_item, st_shell_can_select_previous_tab(shell));
   gtk_widget_set_sensitive(shell->priv->next_directory_item, st_shell_can_select_next_tab(shell));
   gtk_widget_set_sensitive(shell->priv->move_directory_left_item, st_shell_can_move_tab_left(shell));
   gtk_widget_set_sensitive(shell->priv->move_directory_right_item, st_shell_can_move_tab_right(shell));
-  gtk_widget_set_sensitive(shell->priv->directory_homepage_item, st_shell_can_visit_directory_homepage(shell));
 
   /* toolbar */
   gtk_widget_set_sensitive(shell->priv->tune_in_button, st_shell_can_tune_in(shell));
@@ -1649,8 +1677,7 @@ st_shell_update_sensitivity (STShell *shell)
   gtk_widget_set_sensitive(shell->priv->reload_button, st_shell_can_reload(shell));
 
   /* tabs */
-  SG_LIST_FOREACH(l, shell->priv->tabs)
-    st_browser_tab_update_sensitivity(l->data);
+  g_slist_foreach(shell->priv->tabs, (GFunc) st_browser_tab_update_sensitivity, NULL);
 }
 
 static STBrowserTab *
@@ -1675,31 +1702,19 @@ st_shell_can_stop (STShell *shell)
 
   selected_tab = st_shell_get_selected_tab(shell);
 
-  return (selected_tab && st_browser_tab_can_stop(selected_tab))
-    || shell->priv->tasks != NULL;
+  return selected_tab && st_browser_tab_can_stop(selected_tab);
 }
 
 static void
 st_shell_stop (STShell *shell)
 {
-  GSList *tasks;
-  GSList *l;
+  STBrowserTab *selected_tab;
 
   g_return_if_fail(ST_IS_SHELL(shell));
   g_return_if_fail(st_shell_can_stop(shell));
 
-  SG_LIST_FOREACH(l, shell->priv->tabs)
-    if (st_browser_tab_can_stop(l->data))
-      st_browser_tab_stop(l->data);
-
-  /*
-   * We use a copy, because shell->priv->tasks will be modified in
-   * st_thread_abort().
-   */
-  tasks = g_slist_copy(shell->priv->tasks);
-  SG_LIST_FOREACH(l, tasks)
-    st_thread_abort(l->data);
-  g_slist_free(tasks);
+  selected_tab = st_shell_get_selected_tab(shell);
+  st_browser_tab_stop(selected_tab);
 }
 
 static gboolean
@@ -1771,8 +1786,6 @@ st_shell_find (STShell *shell)
     gtk_window_present(GTK_WINDOW(shell->priv->find));
   else
     {
-      gboolean navigation_sensitive;
-
       shell->priv->find = sgtk_find_dialog_new(GTK_WINDOW(shell->priv->window));
       g_object_add_weak_pointer(G_OBJECT(shell->priv->find), (gpointer *) &shell->priv->find);
 
@@ -1781,9 +1794,8 @@ st_shell_find (STShell *shell)
       sgtk_find_dialog_set_case_sensitive(SGTK_FIND_DIALOG(shell->priv->find), st_settings.find_case_sensitive);
       sgtk_find_dialog_set_wrap_around(SGTK_FIND_DIALOG(shell->priv->find), st_settings.find_wrap_around);
 
-      navigation_sensitive = st_settings.find_token && *st_settings.find_token;
-      sgtk_find_dialog_set_previous_sensitive(SGTK_FIND_DIALOG(shell->priv->find), navigation_sensitive);
-      sgtk_find_dialog_set_next_sensitive(SGTK_FIND_DIALOG(shell->priv->find), navigation_sensitive);
+      sgtk_find_dialog_set_previous_sensitive(SGTK_FIND_DIALOG(shell->priv->find), FIND_NAVIGATION_SENSITIVE);
+      sgtk_find_dialog_set_next_sensitive(SGTK_FIND_DIALOG(shell->priv->find), FIND_NAVIGATION_SENSITIVE);
       
       g_object_connect(shell->priv->find,
 		       "signal::notify", st_shell_find_notify_h, NULL,
@@ -1805,16 +1817,14 @@ st_shell_find_notify_h (GObject *object,
   if (! strcmp(name, "token"))
     {
       const char *token;
-      gboolean navigation_sensitive;
 
       token = sgtk_find_dialog_get_token(dialog);
 
       g_free(st_settings.find_token);
       st_settings.find_token = *token ? g_strdup(token) : NULL;
 
-      navigation_sensitive = st_settings.find_token && *st_settings.find_token;
-      sgtk_find_dialog_set_previous_sensitive(dialog, navigation_sensitive);
-      sgtk_find_dialog_set_next_sensitive(dialog, navigation_sensitive);
+      sgtk_find_dialog_set_previous_sensitive(dialog, FIND_NAVIGATION_SENSITIVE);
+      sgtk_find_dialog_set_next_sensitive(dialog, FIND_NAVIGATION_SENSITIVE);
     }
   else if (! strcmp(name, "history"))
     {
@@ -1837,16 +1847,16 @@ st_shell_find_notify_h (GObject *object,
   else if (! strcmp(name, "case-sensitive"))
     {
       st_settings.find_case_sensitive = sgtk_find_dialog_get_case_sensitive(dialog);
-      sgtk_find_dialog_set_previous_sensitive(dialog, TRUE);
-      sgtk_find_dialog_set_next_sensitive(dialog, TRUE);
+      sgtk_find_dialog_set_previous_sensitive(dialog, FIND_NAVIGATION_SENSITIVE);
+      sgtk_find_dialog_set_next_sensitive(dialog, FIND_NAVIGATION_SENSITIVE);
     }
   else if (! strcmp(name, "wrap-around"))
     {
       st_settings.find_wrap_around = sgtk_find_dialog_get_wrap_around(dialog);
       if (st_settings.find_wrap_around)
 	{
-	  sgtk_find_dialog_set_previous_sensitive(dialog, TRUE);
-	  sgtk_find_dialog_set_next_sensitive(dialog, TRUE);
+	  sgtk_find_dialog_set_previous_sensitive(dialog, FIND_NAVIGATION_SENSITIVE);
+	  sgtk_find_dialog_set_next_sensitive(dialog, FIND_NAVIGATION_SENSITIVE);
 	}
     }
 }
@@ -1993,26 +2003,10 @@ st_shell_search_in_all_categories (STShell *shell)
   search_category = st_handler_get_stock_category(selected_tab->handler, ST_CATEGORY_BAG_SEARCH);
   g_return_if_fail(search_category != NULL);
 
-  st_category_bag_select(search_category);
   st_category_view_select_category(selected_tab->category_view, search_category);
   st_browser_tab_reload(selected_tab);
 }
 
-static void
-st_shell_launch_stream_task (STShell *shell, STStreamTask task)
-{
-  STBrowserTab *selected_tab;
-  GSList *selected_streams;
-
-  g_return_if_fail(ST_IS_SHELL(shell));
-
-  selected_tab = st_shell_get_selected_tab(shell);
-  selected_streams = st_stream_view_get_selected_streams(selected_tab->stream_view);
-
-  /* selected_streams's ownership is taken by st_stream_task_launch() */
-  st_stream_task_launch(shell, selected_tab->handler, task, selected_streams);
-}
-
 gboolean
 st_shell_can_tune_in (STShell *shell)
 {
@@ -2022,18 +2016,19 @@ st_shell_can_tune_in (STShell *shell)
 
   selected_tab = st_shell_get_selected_tab(shell);
 
-  return selected_tab
-    && ST_HANDLER_EVENT_HAS_TUNE_IN(selected_tab->handler)
-    && st_stream_view_has_selected_streams(selected_tab->stream_view);
+  return selected_tab && st_browser_tab_can_run_stream_task(selected_tab, ST_BROWSER_TAB_STREAM_TASK_TUNE_IN);
 }
 
 void
 st_shell_tune_in (STShell *shell)
 {
+  STBrowserTab *selected_tab;
+
   g_return_if_fail(ST_IS_SHELL(shell));
   g_return_if_fail(st_shell_can_tune_in(shell));
 
-  st_shell_launch_stream_task(shell, ST_STREAM_TASK_TUNE_IN);
+  selected_tab = st_shell_get_selected_tab(shell);
+  st_browser_tab_run_stream_task(selected_tab, ST_BROWSER_TAB_STREAM_TASK_TUNE_IN);
 }
 
 gboolean
@@ -2045,18 +2040,19 @@ st_shell_can_record (STShell *shell)
 
   selected_tab = st_shell_get_selected_tab(shell);
 
-  return selected_tab
-    && st_handler_event_is_bound(selected_tab->handler, ST_HANDLER_EVENT_STREAM_RECORD)
-    && st_stream_view_has_selected_streams(selected_tab->stream_view);
+  return selected_tab && st_browser_tab_can_run_stream_task(selected_tab, ST_BROWSER_TAB_STREAM_TASK_RECORD);
 }
 
 void
 st_shell_record (STShell *shell)
 {
+  STBrowserTab *selected_tab;
+
   g_return_if_fail(ST_IS_SHELL(shell));
   g_return_if_fail(st_shell_can_record(shell));
 
-  st_shell_launch_stream_task(shell, ST_STREAM_TASK_RECORD);
+  selected_tab = st_shell_get_selected_tab(shell);
+  st_browser_tab_run_stream_task(selected_tab, ST_BROWSER_TAB_STREAM_TASK_RECORD);
 }
 
 gboolean
@@ -2068,18 +2064,19 @@ st_shell_can_browse (STShell *shell)
 
   selected_tab = st_shell_get_selected_tab(shell);
 
-  return selected_tab
-    && st_handler_event_is_bound(selected_tab->handler, ST_HANDLER_EVENT_STREAM_BROWSE)
-    && st_stream_view_has_selected_streams(selected_tab->stream_view);
+  return selected_tab && st_browser_tab_can_run_stream_task(selected_tab, ST_BROWSER_TAB_STREAM_TASK_BROWSE);
 }
 
 void
 st_shell_browse (STShell *shell)
 {
+  STBrowserTab *selected_tab;
+
   g_return_if_fail(ST_IS_SHELL(shell));
   g_return_if_fail(st_shell_can_browse(shell));
 
-  st_shell_launch_stream_task(shell, ST_STREAM_TASK_BROWSE);
+  selected_tab = st_shell_get_selected_tab(shell);
+  st_browser_tab_run_stream_task(selected_tab, ST_BROWSER_TAB_STREAM_TASK_BROWSE);
 }
 
 gboolean
@@ -2104,7 +2101,7 @@ st_shell_present_stream_properties (STShell *shell)
     gtk_window_present(GTK_WINDOW(shell->priv->stream_properties));
   else
     {
-      shell->priv->stream_properties = st_stream_properties_dialog_new(shell);
+      shell->priv->stream_properties = st_stream_properties_dialog_new(GTK_WINDOW(shell->priv->window));
       g_object_add_weak_pointer(G_OBJECT(shell->priv->stream_properties), (gpointer *) &shell->priv->stream_properties);
 
       g_signal_connect(shell->priv->stream_properties,
@@ -2154,20 +2151,39 @@ st_shell_stream_properties_response_h (GtkDialog *dialog,
     }
 }
 
+void
+st_shell_present_stream_columns (STShell *shell)
+{
+  g_return_if_fail(ST_IS_SHELL(shell));
+
+  if (shell->priv->stream_columns)
+    gtk_window_present(GTK_WINDOW(shell->priv->stream_columns));
+  else
+    {
+      STBrowserTab *selected_tab;
+
+      shell->priv->stream_columns = st_stream_columns_dialog_new(GTK_WINDOW(shell->priv->window));
+      g_object_add_weak_pointer(G_OBJECT(shell->priv->stream_columns), (gpointer *) &shell->priv->stream_columns);
+
+      selected_tab = st_shell_get_selected_tab(shell);
+      if (selected_tab)
+	st_stream_columns_dialog_set_stream_view(ST_STREAM_COLUMNS_DIALOG(shell->priv->stream_columns), selected_tab->stream_view);
+
+      g_signal_connect(shell->priv->stream_columns,
+		       "response",
+		       G_CALLBACK(st_shell_dialog_response_h),
+		       NULL);
+
+      gtk_widget_show(shell->priv->stream_columns);
+    }
+}
+
 static void
 st_shell_present_preferences (STShell *shell)
 {
   g_return_if_fail(ST_IS_SHELL(shell));
 
-  if (shell->priv->preferences)
-    gtk_window_present(GTK_WINDOW(shell->priv->preferences));
-  else
-    {
-      shell->priv->preferences = st_preferences_dialog_new(GTK_WINDOW(shell->priv->window));
-      g_object_add_weak_pointer(G_OBJECT(shell->priv->preferences), (gpointer *) &shell->priv->preferences);
-
-      gtk_widget_show(shell->priv->preferences);
-    }
+  st_shell_present_handler_preferences(shell, NULL);
 }
 
 static void
@@ -2197,6 +2213,47 @@ st_shell_dialog_response_h (GtkDialog *dialog, int response, gpointer data)
   gtk_widget_destroy(GTK_WIDGET(dialog));
 }
 
+static gboolean
+st_shell_can_present_directory_preferences (STShell *shell)
+{
+  STBrowserTab *selected_tab;
+
+  g_return_val_if_fail(ST_IS_SHELL(shell), FALSE);
+
+  selected_tab = st_shell_get_selected_tab(shell);
+
+  return selected_tab && st_browser_tab_can_present_preferences(selected_tab);
+}
+
+static void
+st_shell_present_directory_preferences (STShell *shell)
+{
+  STBrowserTab *selected_tab;
+
+  g_return_if_fail(ST_IS_SHELL(shell));
+  g_return_if_fail(st_shell_can_present_directory_preferences(shell));
+
+  selected_tab = st_shell_get_selected_tab(shell);
+  st_browser_tab_present_preferences(selected_tab);
+}
+
+void
+st_shell_present_handler_preferences (STShell *shell, STHandler *handler)
+{
+  g_return_if_fail(ST_IS_SHELL(shell));
+
+  if (! shell->priv->preferences)
+    {
+      shell->priv->preferences = st_preferences_dialog_new(GTK_WINDOW(shell->priv->window));
+      g_object_add_weak_pointer(G_OBJECT(shell->priv->preferences), (gpointer *) &shell->priv->preferences);
+    }
+
+  if (handler)
+    st_preferences_dialog_select_handler(ST_PREFERENCES_DIALOG(shell->priv->preferences), handler);
+
+  gtk_window_present(GTK_WINDOW(shell->priv->preferences));
+}
+
 static gboolean
 st_shell_can_visit_directory_homepage (STShell *shell)
 {
@@ -2230,18 +2287,19 @@ st_shell_can_add_bookmarks (STShell *shell)
 
   selected_tab = st_shell_get_selected_tab(shell);
 
-  return selected_tab
-    && selected_tab->handler != st_bookmarks_handler
-    && st_stream_view_has_selected_streams(selected_tab->stream_view);
+  return selected_tab && st_browser_tab_can_run_stream_task(selected_tab, ST_BROWSER_TAB_STREAM_TASK_ADD_BOOKMARKS);
 }
 
 void
 st_shell_add_bookmarks (STShell *shell)
 {
+  STBrowserTab *selected_tab;
+
   g_return_if_fail(ST_IS_SHELL(shell));
   g_return_if_fail(st_shell_can_add_bookmarks(shell));
 
-  st_shell_launch_stream_task(shell, ST_STREAM_TASK_ADD_BOOKMARKS);
+  selected_tab = st_shell_get_selected_tab(shell);
+  st_browser_tab_run_stream_task(selected_tab, ST_BROWSER_TAB_STREAM_TASK_ADD_BOOKMARKS);
 }
 
 gboolean
@@ -2517,37 +2575,7 @@ st_shell_homepage (STShell *shell)
   st_visit_homepage(ST_HOME);
 }
 
-void
-st_shell_add_stream_task (STShell *shell, STThread *thread, gboolean tune_in)
-{
-  g_return_if_fail(ST_IS_SHELL(shell));
-  g_return_if_fail(thread != NULL);
-
-  if (tune_in)
-    {
-      if (shell->priv->tune_in_thread)
-	st_thread_abort(shell->priv->tune_in_thread);
-      shell->priv->tune_in_thread = thread;
-    }
-  
-  shell->priv->tasks = g_slist_append(shell->priv->tasks, thread);
-  st_shell_update_sensitivity(shell);
-}
-
-void
-st_shell_remove_stream_task (STShell *shell, STThread *thread)
-{
-  g_return_if_fail(ST_IS_SHELL(shell));
-  g_return_if_fail(thread != NULL);
-
-  shell->priv->tasks = g_slist_remove(shell->priv->tasks, thread);
-  if (thread == shell->priv->tune_in_thread)
-    shell->priv->tune_in_thread = NULL;
-
-  st_shell_update_sensitivity(shell);
-}
-
-void
+static void
 st_shell_new_preselection (STShell *shell)
 {
   STBrowserTab *tab;
@@ -2573,11 +2601,32 @@ st_shell_new_preselection (STShell *shell)
 }
 
 GtkWindow *
-st_shell_get_transient (STShell *shell)
+st_shell_get_window (STShell *shell)
 {
   g_return_val_if_fail(ST_IS_SHELL(shell), NULL);
 
-  return GTK_WINDOW(shell->priv->window);
+  return (GtkWindow *) shell->priv->window;
+}
+
+static void
+st_shell_set_statusbar_of_tab (STShell *shell, STBrowserTab *tab)
+{
+  STCategoryBag *category_bag;
+
+  g_return_if_fail(ST_IS_SHELL(shell));
+  g_return_if_fail(ST_IS_BROWSER_TAB(tab));
+
+  category_bag = st_handler_get_selected_category(tab->handler);
+  g_return_if_fail(category_bag != NULL);
+
+  if (GTK_BIN(shell->priv->statusbox)->child)
+    gtk_container_remove(GTK_CONTAINER(shell->priv->statusbox), GTK_BIN(shell->priv->statusbox)->child);
+
+  gtk_container_add(GTK_CONTAINER(shell->priv->statusbox), category_bag->statusbar);
+  gtk_widget_show(category_bag->statusbar);
+
+  /* work-around a bug in the Bluecurve theme engine */
+  gtk_widget_realize(GTK_WIDGET(ST_STATUSBAR(category_bag->statusbar)->progress_bar));
 }
 
 static void
@@ -2585,9 +2634,13 @@ st_shell_category_selection_changed_h (STCategoryView *view,
 				       gpointer user_data)
 {
   STShell *shell = user_data;
+  STBrowserTab *tab;
 
   st_shell_update_title(shell);
   st_shell_update_sensitivity(shell);
+
+  tab = st_shell_get_selected_tab(shell);
+  st_shell_set_statusbar_of_tab(shell, tab);
 }
 
 static void
@@ -2605,8 +2658,8 @@ st_shell_stream_selection_changed_h (STStreamView *view, gpointer user_data)
     }
   if (shell->priv->find)
     {
-      sgtk_find_dialog_set_previous_sensitive(SGTK_FIND_DIALOG(shell->priv->find), TRUE);
-      sgtk_find_dialog_set_next_sensitive(SGTK_FIND_DIALOG(shell->priv->find), TRUE);
+      sgtk_find_dialog_set_previous_sensitive(SGTK_FIND_DIALOG(shell->priv->find), FIND_NAVIGATION_SENSITIVE);
+      sgtk_find_dialog_set_next_sensitive(SGTK_FIND_DIALOG(shell->priv->find), FIND_NAVIGATION_SENSITIVE);
     }
 }
 
@@ -2625,13 +2678,14 @@ st_shell_stream_activated_h (GtkTreeView *treeview,
 }
 
 static void
-st_shell_tab_reloading_changed_h (STBrowserTab *tab,
-				  gboolean reloading,
-				  gpointer user_data)
+st_shell_tab_notify_running_h (GObject *object,
+			       GParamSpec *pspec,
+			       gpointer user_data)
 {
+  STBrowserTab *tab = ST_BROWSER_TAB(object);
   STShell *shell = user_data;
 
-  if (reloading)
+  if (tab->running)
     /*
      * The selected_category label might have been changed by url_cb,
      * so we update the shell title.
@@ -2640,3 +2694,37 @@ st_shell_tab_reloading_changed_h (STBrowserTab *tab,
 
   st_shell_update_sensitivity(shell);
 }
+
+static void
+st_shell_tab_stream_task_added_h (STBrowserTab *tab,
+				  STBrowserTabStreamTask *task,
+				  gpointer user_data)
+{
+  STShell *shell = user_data;
+
+  if (task->type == ST_BROWSER_TAB_STREAM_TASK_TUNE_IN)
+    {
+      if (shell->priv->tune_in_task)
+	st_thread_abort(shell->priv->tune_in_task->thread);
+      shell->priv->tune_in_task = task;
+    }
+}
+
+static void
+st_shell_tab_stream_task_removed_h (STBrowserTab *tab,
+				    STBrowserTabStreamTask *task,
+				    gpointer user_data)
+{
+  STShell *shell = user_data;
+
+  if (task == shell->priv->tune_in_task)
+    shell->priv->tune_in_task = NULL;
+}
+
+void
+st_shell_set_restart (STShell *shell, gboolean restart)
+{
+  g_return_if_fail(ST_IS_SHELL(shell));
+  
+  shell->priv->restart = restart;
+}
