@@ -32,7 +32,6 @@
 #include "config.h"
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include "art/streamtuner.h"
 #include "sgtk-hig.h"
 #include "st-splash.h"
 #include "st-util.h"
@@ -45,15 +44,10 @@ struct _STSplashPrivate
   GtkWidget	*progress_bar;
 };
 
-/*** variable declarations ***************************************************/
-
-static GObjectClass *parent_class = NULL;
-
 /*** function declarations ***************************************************/
 
 static void st_splash_class_init (STSplashClass *class);
 static void st_splash_init (STSplash *splash);
-static void st_splash_finalize (GObject *object);
 
 /*** implementation **********************************************************/
 
@@ -88,29 +82,19 @@ st_splash_get_type (void)
 static void
 st_splash_class_init (STSplashClass *class)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS(class);
-
-  parent_class = g_type_class_peek_parent(class);
-
-  object_class->finalize = st_splash_finalize;
+  g_type_class_add_private(class, sizeof(STSplashPrivate));
 }
 
 static void
 st_splash_init (STSplash *splash)
 {
-  GdkPixbuf *icon;
   GtkWidget *logo;
   GtkWidget *vbox;
   GtkWidget *frame;
 
-  splash->priv = g_new0(STSplashPrivate, 1);
+  splash->priv = G_TYPE_INSTANCE_GET_PRIVATE(splash, ST_TYPE_SPLASH, STSplashPrivate);
   
-  icon = gdk_pixbuf_new_from_inline(sizeof(art_streamtuner),
-				    art_streamtuner,
-				    FALSE,
-				    NULL);
-  gtk_window_set_icon(GTK_WINDOW(splash), icon);
-  g_object_unref(icon);
+  st_window_set_icon(GTK_WINDOW(splash));
 
   gtk_window_set_title(GTK_WINDOW(splash), NULL);
   gtk_window_set_resizable(GTK_WINDOW(splash), FALSE);
@@ -137,16 +121,6 @@ st_splash_init (STSplash *splash)
   gtk_widget_show_all(frame);
 }
 
-static void
-st_splash_finalize (GObject *object)
-{
-  STSplash *splash = ST_SPLASH(object);
-
-  g_free(splash->priv);
-
-  G_OBJECT_CLASS(parent_class)->finalize(object);
-}
-
 GtkWidget *
 st_splash_new (void)
 {
