@@ -34,6 +34,10 @@
 
 #include <glib.h>
 
+G_BEGIN_DECLS
+
 gboolean st_is_aborted (void);
 
+G_END_DECLS
+
 #endif /* _ST_STATE_API_H */
