@@ -32,27 +32,33 @@
 #ifndef _ST_STOCK_H
 #define _ST_STOCK_H
 
-#define ST_STOCK_TUNE_IN			"st-tune-in"
-#define ST_STOCK_RECORD				"st-record"
-#define ST_STOCK_BROWSE				"st-browse"
-
-#define ST_STOCK_RELOAD				"st-reload"
-
-#define ST_STOCK_QUIT_WITHOUT_SAVING		"st-quit-without-saving"
-#define ST_STOCK_SEARCH				"st-search"
-#define ST_STOCK_OPEN_LINK			"st-open-link"
-#define ST_STOCK_COPY_LINK_ADDRESS		"st-copy-link-address"
-
-#define ST_STOCK_NEW_PRESELECTION		"st-new-preselection"
 #define ST_STOCK_ADD_BOOKMARK			"st-add-bookmark"
-
-#define ST_STOCK_CATEGORY			"st-category"
+#define ST_STOCK_APPLICATIONS			"st-applications"
+#define ST_STOCK_BROWSE				"st-browse"
+#define ST_STOCK_CATEGORY_OPEN_RUNNING		"st-category-open-running"
 #define ST_STOCK_CATEGORY_OPEN			"st-category-open"
-
+#define ST_STOCK_CATEGORY_RUNNING		"st-category-running"
+#define ST_STOCK_CATEGORY			"st-category"
+#define ST_STOCK_COPY_LINK_ADDRESS		"st-copy-link-address"
+#define ST_STOCK_DIRECTORY_PREFERENCES		"st-directory-preferences"
+#define ST_STOCK_GENERAL			"st-general"
+#define ST_STOCK_HIDE_THIS_COLUMN		"st-hide-this-column"
 #define ST_STOCK_LEAVE_FULLSCREEN		"st-leave-fullscreen"
-
+#define ST_STOCK_MAIN_CATEGORY_RUNNING		"st-main-category-running"
+#define ST_STOCK_MAIN_CATEGORY			"st-main-category"
+#define ST_STOCK_NEW_PRESELECTION		"st-new-preselection"
+#define ST_STOCK_OPEN_LINK			"st-open-link"
+#define ST_STOCK_PLUGIN				"st-plugin"
+#define ST_STOCK_QUIT_WITHOUT_SAVING		"st-quit-without-saving"
+#define ST_STOCK_RECORD				"st-record"
+#define ST_STOCK_RELOAD				"st-reload"
+#define ST_STOCK_SEARCH_CATEGORY_RUNNING	"st-search-category-running"
+#define ST_STOCK_SEARCH_CATEGORY		"st-search-category"
+#define ST_STOCK_SEARCH				"st-search"
 #define ST_STOCK_SELECT_ALL			"st-select-all"
-
+#define ST_STOCK_SHOW_ALL_COLUMNS		"st-show-all-columns"
+#define ST_STOCK_STREAM_COLUMNS			"st-stream-columns"
+#define ST_STOCK_TUNE_IN			"st-tune-in"
 #define ST_STOCK_VISIT_DIRECTORY_HOMEPAGE	"st-visit-directory-homepage"
 
 void st_stock_init (void);
