@@ -50,12 +50,6 @@ enum {
     
 struct _STStreamBagPrivate
 {
-  /*
-   * This mutex only protects the STStreamBag object. Handlers are
-   * responsible for protecting the underlying STStream object.
-   */
-  GMutex		*mutex;
-
   int			running_count;
 };
 
@@ -107,6 +101,8 @@ st_stream_bag_class_init (STStreamBagClass *class)
 
   parent_class = g_type_class_peek_parent(class);
 
+  g_type_class_add_private(class, sizeof(STStreamBagPrivate));
+
   object_class->finalize = st_stream_bag_finalize;
 
   stream_bag_signals[CHANGED] = g_signal_new("changed",
@@ -146,8 +142,7 @@ st_stream_bag_init (STStreamBag *bag)
   bag->handler = NULL;
   memset(&bag->iter, 0, sizeof(bag->iter));
   
-  bag->priv = g_new0(STStreamBagPrivate, 1);
-  bag->priv->mutex = g_mutex_new();
+  bag->priv = G_TYPE_INSTANCE_GET_PRIVATE(bag, ST_TYPE_STREAM_BAG, STStreamBagPrivate);
 }
 
 static void
@@ -156,10 +151,8 @@ st_stream_bag_finalize (GObject *object)
   STStreamBag *bag = ST_STREAM_BAG(object);
 
   st_handler_event_stream_free(bag->handler, bag->stream);
-  g_mutex_free(bag->priv->mutex);
-  g_free(bag->priv);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  parent_class->finalize(object);
 }
 
 STStreamBag *
@@ -187,19 +180,22 @@ st_stream_bag_new_from_stream (STHandler *handler, STStream *stream)
 }
 
 void
-st_stream_bag_set_running (STStreamBag *bag, gboolean running)
+st_stream_bag_running_ref (STStreamBag *bag)
 {
   g_return_if_fail(ST_IS_STREAM_BAG(bag));
 
-  g_mutex_lock(bag->priv->mutex);
-  if (running)
-    bag->priv->running_count++;
-  else if (bag->priv->running_count > 0)
-    bag->priv->running_count--;
-  running = bag->priv->running_count > 0;
-  g_mutex_unlock(bag->priv->mutex);
+  if (++bag->priv->running_count == 1)
+    g_signal_emit(bag, stream_bag_signals[STATE_CHANGED], 0, TRUE);
+}
 
-  g_signal_emit(bag, stream_bag_signals[STATE_CHANGED], 0, running);
+void
+st_stream_bag_running_unref (STStreamBag *bag)
+{
+  g_return_if_fail(ST_IS_STREAM_BAG(bag));
+  g_return_if_fail(bag->priv->running_count > 0);
+
+  if (--bag->priv->running_count == 0)
+    g_signal_emit(bag, stream_bag_signals[STATE_CHANGED], 0, FALSE);
 }
 
 /*** event wrappers **********************************************************/
@@ -241,11 +237,46 @@ st_stream_bag_get_stock_field (STStreamBag *bag,
     {
     case ST_HANDLER_STOCK_FIELD_NAME:
     case ST_HANDLER_STOCK_FIELD_GENRE:
+    case ST_HANDLER_STOCK_FIELD_DESCRIPTION:
+    case ST_HANDLER_STOCK_FIELD_HOMEPAGE:
       g_value_init(value, G_TYPE_STRING);
       break;
 
+    case ST_HANDLER_STOCK_FIELD_URI_LIST:
+      g_value_init(value, G_TYPE_VALUE_ARRAY);
+      break;
+
     default:
-      g_assert_not_reached();
+      g_return_if_reached();
+    }
+  
+  st_stream_bag_get_stock_field_initialized(bag, stock_field, value);
+}
+
+void
+st_stream_bag_get_stock_field_initialized (STStreamBag *bag,
+					   STHandlerStockField stock_field,
+					   GValue *value)
+{
+  g_return_if_fail(ST_IS_STREAM_BAG(bag));
+  g_return_if_fail(st_handler_event_is_bound(bag->handler, ST_HANDLER_EVENT_STREAM_STOCK_FIELD_GET));
+  g_return_if_fail(G_IS_VALUE(value));
+
+  switch (stock_field)
+    {
+    case ST_HANDLER_STOCK_FIELD_NAME:
+    case ST_HANDLER_STOCK_FIELD_GENRE:
+    case ST_HANDLER_STOCK_FIELD_DESCRIPTION:
+    case ST_HANDLER_STOCK_FIELD_HOMEPAGE:
+      g_return_if_fail(G_VALUE_HOLDS_STRING(value));
+      break;
+
+    case ST_HANDLER_STOCK_FIELD_URI_LIST:
+      g_return_if_fail(G_VALUE_HOLDS(value, G_TYPE_VALUE_ARRAY));
+      break;
+
+    default:
+      g_return_if_reached();
     }
   
   st_handler_event_stream_stock_field_get(bag->handler, bag->stream, stock_field, value);
