@@ -38,7 +38,6 @@
 
 struct _STStreamMenuItems
 {
-  STShell	*shell;
   GtkWidget	*tune_in;
   GtkWidget	*record;
   GtkWidget	*browse;
@@ -52,13 +51,12 @@ struct _STStreamMenuItems
 /*** implementation **********************************************************/
 
 STStreamMenuItems *
-st_stream_menu_items_new (STShell *shell, GtkAccelGroup *accel_group)
+st_stream_menu_items_new (GtkAccelGroup *accel_group)
 {
   STStreamMenuItems *items;
 
   items = g_new(STStreamMenuItems, 1);
 
-  items->shell = shell;
   items->tune_in = gtk_image_menu_item_new_from_stock(ST_STOCK_TUNE_IN, accel_group);
   items->record = gtk_image_menu_item_new_from_stock(ST_STOCK_RECORD, accel_group);
   items->browse = gtk_image_menu_item_new_from_stock(ST_STOCK_BROWSE, accel_group);
@@ -71,27 +69,27 @@ st_stream_menu_items_new (STShell *shell, GtkAccelGroup *accel_group)
   g_signal_connect_swapped(items->tune_in,
 			   "activate",
 			   G_CALLBACK(st_shell_tune_in),
-			   items->shell);
+			   st_shell);
   g_signal_connect_swapped(items->record,
 			   "activate",
 			   G_CALLBACK(st_shell_record),
-			   items->shell);
+			   st_shell);
   g_signal_connect_swapped(items->browse,
 			   "activate",
 			   G_CALLBACK(st_shell_browse),
-			   items->shell);
+			   st_shell);
   g_signal_connect_swapped(items->add_bookmark,
 			   "activate",
 			   G_CALLBACK(st_shell_add_bookmarks),
-			   items->shell);
+			   st_shell);
   g_signal_connect_swapped(items->delete,
 			   "activate",
 			   G_CALLBACK(st_shell_delete_streams),
-			   items->shell);
+			   st_shell);
   g_signal_connect_swapped(items->properties,
 			   "activate",
 			   G_CALLBACK(st_shell_present_stream_properties),
-			   items->shell);
+			   st_shell);
 
   gtk_widget_show(items->tune_in);
   gtk_widget_show(items->record);
@@ -118,12 +116,12 @@ st_stream_menu_items_update_sensitivity (STStreamMenuItems *items)
 {
   g_return_if_fail(items != NULL);
 
-  gtk_widget_set_sensitive(items->tune_in, st_shell_can_tune_in(items->shell));
-  gtk_widget_set_sensitive(items->record, st_shell_can_record(items->shell));
-  gtk_widget_set_sensitive(items->browse, st_shell_can_browse(items->shell));
-  gtk_widget_set_sensitive(items->add_bookmark, st_shell_can_add_bookmarks(items->shell));
-  gtk_widget_set_sensitive(items->delete, st_shell_can_delete_streams(items->shell));
-  gtk_widget_set_sensitive(items->properties, st_shell_can_present_stream_properties(items->shell));
+  gtk_widget_set_sensitive(items->tune_in, st_shell_can_tune_in(st_shell));
+  gtk_widget_set_sensitive(items->record, st_shell_can_record(st_shell));
+  gtk_widget_set_sensitive(items->browse, st_shell_can_browse(st_shell));
+  gtk_widget_set_sensitive(items->add_bookmark, st_shell_can_add_bookmarks(st_shell));
+  gtk_widget_set_sensitive(items->delete, st_shell_can_delete_streams(st_shell));
+  gtk_widget_set_sensitive(items->properties, st_shell_can_present_stream_properties(st_shell));
 }
 
 void
