@@ -33,12 +33,10 @@
 #define _ST_STREAM_MENU_ITEMS_H
 
 #include <gtk/gtk.h>
-#include "st-shell.h"
 
 typedef struct _STStreamMenuItems STStreamMenuItems;
 
-STStreamMenuItems *st_stream_menu_items_new (STShell		*shell,
-					     GtkAccelGroup	*accel_group);
+STStreamMenuItems *st_stream_menu_items_new (GtkAccelGroup	*accel_group);
 void              st_stream_menu_items_free (STStreamMenuItems	*items);
 
 void st_stream_menu_items_update_sensitivity	(STStreamMenuItems  *items);
