@@ -43,6 +43,7 @@
 #include "st-shell.h"
 #include "st-dialog-api.h"
 #include "st-handler-field.h"
+#include "st-util-api.h"
 
 /*** cpp *********************************************************************/
 
@@ -52,8 +53,6 @@
 
 struct _STStreamPropertiesDialogPrivate
 {
-  STShell		*shell;
-
   STStreamBag		*stream_bag;
 
   GtkWidget		*previous_button;
@@ -96,10 +95,11 @@ static void st_stream_properties_dialog_set_widget_data (GtkWidget *widget,
 static void st_stream_properties_dialog_get_fields_and_values (STStreamPropertiesDialog *dialog,
 							       GSList **fields,
 							       GSList **values);
+static gboolean st_stream_properties_dialog_value_equal (const GValue *value1,
+							 const GValue *value2);
 static void st_stream_properties_dialog_optimize_fields_and_values (STStreamBag *stream_bag,
 								    GSList **fields,
 								    GSList **values);
-static void st_stream_properties_dialog_values_free (GSList *values);
 
 static CancelInfo *st_stream_properties_dialog_cancel_info_new (STStreamPropertiesDialog *dialog);
 static void       st_stream_properties_dialog_cancel_info_free (CancelInfo *info);
@@ -149,13 +149,15 @@ st_stream_properties_dialog_class_init (STStreamPropertiesDialogClass *class)
 
   parent_class = g_type_class_peek_parent(class);
 
+  g_type_class_add_private(class, sizeof(STStreamPropertiesDialogPrivate));
+
   object_class->finalize = st_stream_properties_dialog_finalize;
 }
 
 static void
 st_stream_properties_dialog_init (STStreamPropertiesDialog *dialog)
 {
-  dialog->priv = g_new0(STStreamPropertiesDialogPrivate, 1);
+  dialog->priv = G_TYPE_INSTANCE_GET_PRIVATE(dialog, ST_TYPE_STREAM_PROPERTIES_DIALOG, STStreamPropertiesDialogPrivate);
 
   dialog->priv->previous_button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_GO_BACK, SGTK_RESPONSE_PREVIOUS);
   dialog->priv->next_button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_GO_FORWARD, SGTK_RESPONSE_NEXT);
@@ -176,9 +178,8 @@ st_stream_properties_dialog_finalize (GObject *object)
   st_stream_properties_dialog_unset_stream(dialog);
   g_slist_free(dialog->priv->widgets);
   st_stream_properties_dialog_cancel_list_free(dialog);
-  g_free(dialog->priv);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  parent_class->finalize(object);
 }
 
 static void
@@ -200,7 +201,7 @@ static void
 st_stream_properties_dialog_construct_table (STStreamPropertiesDialog *dialog)
 {
   GSList *l;
-  int fields;
+  int n_fields;
   int vi = 0;
   int width;
   int height;
@@ -221,8 +222,8 @@ st_stream_properties_dialog_construct_table (STStreamPropertiesDialog *dialog)
       dialog->priv->widgets = NULL;
     }
   
-  fields = st_handler_count_fields(dialog->priv->table_handler, ST_HANDLER_FIELD_VISIBLE | ST_HANDLER_FIELD_EDITABLE);
-  dialog->priv->table = gtk_table_new(fields, 2, FALSE);
+  n_fields = st_handler_count_fields(dialog->priv->table_handler, ST_HANDLER_FIELD_VISIBLE | ST_HANDLER_FIELD_EDITABLE);
+  dialog->priv->table = gtk_table_new(n_fields, 2, FALSE);
  
   gtk_table_set_row_spacings(GTK_TABLE(dialog->priv->table), SGTK_HIG_CONTROL_SPACING);
   gtk_table_set_col_spacings(GTK_TABLE(dialog->priv->table), SGTK_HIG_CONTROL_LABEL_SPACING);
@@ -237,6 +238,8 @@ st_stream_properties_dialog_construct_table (STStreamPropertiesDialog *dialog)
 
 	  if (ST_HANDLER_FIELD_IS_EDITABLE(field))
 	    {
+	      const char *description;
+
 	      switch (st_handler_field_get_type(field))
 		{
 		case G_TYPE_BOOLEAN:
@@ -247,12 +250,20 @@ st_stream_properties_dialog_construct_table (STStreamPropertiesDialog *dialog)
 		  widget = gtk_spin_button_new_with_range(G_MININT, G_MAXINT, 1);
 		  break;
 
+		case G_TYPE_UINT:
+		  widget = gtk_spin_button_new_with_range(0, G_MAXUINT, 1);
+		  break;
+
+		case G_TYPE_DOUBLE:
+		  widget = gtk_spin_button_new_with_range(G_MINDOUBLE, G_MAXDOUBLE, 1);
+		  break;
+
 		case G_TYPE_STRING:
 		  widget = gtk_entry_new();
 		  break;
 
 		default:
-		  g_assert_not_reached();
+		  g_return_if_reached();
 		}
 
 	      if (! focused_widget)
@@ -260,9 +271,13 @@ st_stream_properties_dialog_construct_table (STStreamPropertiesDialog *dialog)
 
 	      if (! first_widget)
 		first_widget = widget;
-	      if (prev_widget)
-		sgtk_widget_set_next_widget(prev_widget, widget);
+	      if (prev_widget && GTK_IS_ENTRY(prev_widget))
+		sgtk_entry_set_next_widget(GTK_ENTRY(prev_widget), widget);
 	      prev_widget = widget;
+
+	      description = st_handler_field_get_description(field);
+	      if (description)
+		st_set_tooltip(widget, description);
 	    }
 	  else
 	    {
@@ -335,8 +350,8 @@ st_stream_properties_dialog_construct_table (STStreamPropertiesDialog *dialog)
   if (! focused_widget)
     gtk_widget_grab_focus(dialog->priv->ok_button);
 
-  if (prev_widget)
-    sgtk_widget_set_next_widget(prev_widget, first_widget);
+  if (prev_widget && GTK_IS_ENTRY(prev_widget) && prev_widget != first_widget)
+    sgtk_entry_set_next_widget(GTK_ENTRY(prev_widget), first_widget);
 
   gtk_widget_show(dialog->priv->table);
   gtk_container_add(GTK_CONTAINER(SGTK_DIALOG(dialog)->contents), dialog->priv->table);
@@ -420,6 +435,14 @@ st_stream_properties_dialog_set_widget_data (GtkWidget *widget,
 	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), g_value_get_int(&value));
 	  break;
 	  
+	case G_TYPE_UINT:
+	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), g_value_get_uint(&value));
+	  break;
+
+	case G_TYPE_DOUBLE:
+	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), g_value_get_double(&value));
+	  break;
+
 	case G_TYPE_STRING:
 	  {
 	    const char *str = g_value_get_string(&value);
@@ -441,6 +464,18 @@ st_stream_properties_dialog_set_widget_data (GtkWidget *widget,
 	  gtk_label_set_text(GTK_LABEL(widget), str);
 	  g_free(str);
 	}
+      else if (G_VALUE_HOLDS_UINT(&value))
+	{
+	  char *str = g_strdup_printf("%u", g_value_get_uint(&value));
+	  gtk_label_set_text(GTK_LABEL(widget), str);
+	  g_free(str);
+	}
+      else if (G_VALUE_HOLDS_DOUBLE(&value))
+	{
+	  char *str = g_strdup_printf("%f", g_value_get_double(&value));
+	  gtk_label_set_text(GTK_LABEL(widget), str);
+	  g_free(str);
+	}
       else if (G_VALUE_HOLDS_STRING(&value))
 	gtk_label_set_text(GTK_LABEL(widget), g_value_get_string(&value));
       else if (G_VALUE_HOLDS(&value, GDK_TYPE_PIXBUF))
@@ -451,9 +486,9 @@ st_stream_properties_dialog_set_widget_data (GtkWidget *widget,
 	  char *str;
 
 	  value_array = g_value_get_boxed(&value);
-	  str = sg_value_array_get_string(value_array);
+	  str = value_array ? sg_value_array_get_string(value_array) : NULL;
 
-	  gtk_label_set_text(GTK_LABEL(widget), str);
+	  gtk_label_set_text(GTK_LABEL(widget), str ? str : "");
 	  g_free(str);
 	}
       else
@@ -464,14 +499,14 @@ st_stream_properties_dialog_set_widget_data (GtkWidget *widget,
 }
 
 GtkWidget *
-st_stream_properties_dialog_new (STShell *shell)
+st_stream_properties_dialog_new (GtkWindow *parent)
 {
   STStreamPropertiesDialog *dialog;
 
   dialog = g_object_new(ST_TYPE_STREAM_PROPERTIES_DIALOG, NULL);
 
-  dialog->priv->shell = shell;
-  gtk_window_set_transient_for(GTK_WINDOW(dialog), st_shell_get_transient(shell));
+  if (parent)
+    gtk_window_set_transient_for(GTK_WINDOW(dialog), parent);
   st_stream_properties_dialog_update_sensitivity(dialog);
 
   return GTK_WIDGET(dialog);
@@ -486,14 +521,13 @@ st_stream_properties_dialog_set_stream (STStreamPropertiesDialog *dialog,
 
   st_stream_properties_dialog_unset_stream(dialog);
 
-  g_object_ref(stream_bag);
   if (stream_bag->handler != dialog->priv->table_handler)
     {
       dialog->priv->table_handler = stream_bag->handler;
       st_stream_properties_dialog_construct_table(dialog);
     }
 
-  dialog->priv->stream_bag = stream_bag;
+  dialog->priv->stream_bag = g_object_ref(stream_bag);
 
   g_object_connect(dialog->priv->stream_bag,
 		   "signal::changed", st_stream_properties_dialog_stream_changed_h, dialog,
@@ -528,8 +562,8 @@ st_stream_properties_dialog_update_sensitivity (STStreamPropertiesDialog *dialog
 {
   g_return_if_fail(ST_IS_STREAM_PROPERTIES_DIALOG(dialog));
 
-  gtk_widget_set_sensitive(dialog->priv->previous_button, st_shell_can_select_previous_stream(dialog->priv->shell));
-  gtk_widget_set_sensitive(dialog->priv->next_button, st_shell_can_select_next_stream(dialog->priv->shell));
+  gtk_widget_set_sensitive(dialog->priv->previous_button, st_shell_can_select_previous_stream(st_shell));
+  gtk_widget_set_sensitive(dialog->priv->next_button, st_shell_can_select_next_stream(st_shell));
 }
 
 static void
@@ -554,10 +588,10 @@ st_stream_properties_dialog_get_fields_and_values (STStreamPropertiesDialog *dia
       if (ST_HANDLER_FIELD_IS_EDITABLE(field))
 	{
 	  GValue current_value = { 0, };
-	  GValue *value = g_new0(GValue, 1);
+	  GValue *value;
 	  
 	  st_stream_bag_get_field(dialog->priv->stream_bag, field, &current_value);
-	  g_value_init(value, st_handler_field_get_type(field));
+	  value = sg_value_new(st_handler_field_get_type(field));
 
 	  switch (G_VALUE_TYPE(value))
 	    {
@@ -569,6 +603,14 @@ st_stream_properties_dialog_get_fields_and_values (STStreamPropertiesDialog *dia
 	      g_value_set_int(value, gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)));
 	      break;
 
+	    case G_TYPE_UINT:
+	      g_value_set_uint(value, gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)));
+	      break;
+
+	    case G_TYPE_DOUBLE:
+	      g_value_set_double(value, gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget)));
+	      break;
+
 	    case G_TYPE_STRING:
 	      {
 		const char *str = gtk_entry_get_text(GTK_ENTRY(widget));
@@ -577,7 +619,7 @@ st_stream_properties_dialog_get_fields_and_values (STStreamPropertiesDialog *dia
 	      }
 
 	    default:
-	      g_assert_not_reached();
+	      g_return_if_reached();
 	    }
 
 	  *fields = g_slist_append(*fields, field);
@@ -588,6 +630,50 @@ st_stream_properties_dialog_get_fields_and_values (STStreamPropertiesDialog *dia
     }
 }
 
+static gboolean
+st_stream_properties_dialog_value_equal (const GValue *value1,
+					 const GValue *value2)
+{
+  gboolean equal;
+
+  g_return_val_if_fail(G_IS_VALUE(value1), FALSE);
+  g_return_val_if_fail(G_IS_VALUE(value2), FALSE);
+  g_return_val_if_fail(G_VALUE_TYPE(value1) == G_VALUE_TYPE(value2), FALSE);
+
+  switch (G_VALUE_TYPE(value1))
+    {
+    case G_TYPE_BOOLEAN:
+      equal = g_value_get_boolean(value1) == g_value_get_boolean(value2);
+      break;
+
+    case G_TYPE_INT:
+      equal = g_value_get_int(value1) == g_value_get_int(value2);
+      break;
+
+    case G_TYPE_UINT:
+      equal = g_value_get_uint(value1) == g_value_get_uint(value2);
+      break;
+
+    case G_TYPE_DOUBLE:
+      equal = sg_double_equal(g_value_get_double(value1), g_value_get_double(value2));
+      break;
+
+    case G_TYPE_STRING:
+      {
+	const char *str1 = g_value_get_string(value1);
+	const char *str2 = g_value_get_string(value2);
+
+	equal = (str1 == NULL && str2 == NULL) || (str1 && str2 && ! strcmp(str1, str2));
+	break;
+      }
+
+    default:
+      g_return_val_if_reached(FALSE);
+    }
+
+  return equal;
+}
+
 /*
  * Modify FIELDS and VALUES so that only values differing from those
  * in STREAM_BAG are included.
@@ -614,11 +700,11 @@ st_stream_properties_dialog_optimize_fields_and_values (STStreamBag *stream_bag,
 
       st_stream_bag_get_field(stream_bag, field, &current_value);
 
-      if (! sg_value_equal(value, &current_value))
+      if (! st_stream_properties_dialog_value_equal(value, &current_value))
 	{
-	  GValue *new_value = g_new0(GValue, 1);
+	  GValue *new_value;
 
-	  g_value_init(new_value, G_VALUE_TYPE(value));
+	  new_value = sg_value_new(G_VALUE_TYPE(value));
 	  g_value_copy(value, new_value);
 
 	  new_fields = g_slist_append(new_fields, field);
@@ -629,26 +715,14 @@ st_stream_properties_dialog_optimize_fields_and_values (STStreamBag *stream_bag,
     }
 
   g_slist_free(*fields);
-  st_stream_properties_dialog_values_free(*values);
+
+  g_slist_foreach(*values, (GFunc) sg_value_free, NULL);
+  g_slist_free(*values);
 
   *fields = new_fields;
   *values = new_values;
 }
 
-static void
-st_stream_properties_dialog_values_free (GSList *values)
-{
-  GSList *l;
-
-  SG_LIST_FOREACH(l, values)
-    {
-      g_value_unset(l->data);
-      g_free(l->data);
-    }
-
-  g_slist_free(values);
-}
-
 static CancelInfo *
 st_stream_properties_dialog_cancel_info_new (STStreamPropertiesDialog *dialog)
 {
@@ -657,8 +731,7 @@ st_stream_properties_dialog_cancel_info_new (STStreamPropertiesDialog *dialog)
 
   g_return_val_if_fail(ST_IS_STREAM_PROPERTIES_DIALOG(dialog), NULL);
 
-  g_object_ref(dialog->priv->stream_bag);
-  info->stream_bag = dialog->priv->stream_bag;
+  info->stream_bag = g_object_ref(dialog->priv->stream_bag);
 
   SG_LIST_FOREACH(l, dialog->priv->widgets)
     {
@@ -685,7 +758,10 @@ st_stream_properties_dialog_cancel_info_free (CancelInfo *info)
 
   g_object_unref(info->stream_bag);
   g_slist_free(info->fields);
-  st_stream_properties_dialog_values_free(info->values);
+
+  g_slist_foreach(info->values, (GFunc) sg_value_free, NULL);
+  g_slist_free(info->values);
+
   g_free(info);
 }
 
@@ -714,13 +790,9 @@ st_stream_properties_dialog_cancel_list_append (STStreamPropertiesDialog *dialog
 static void
 st_stream_properties_dialog_cancel_list_free (STStreamPropertiesDialog *dialog)
 {
-  GSList *l;
-
   g_return_if_fail(ST_IS_STREAM_PROPERTIES_DIALOG(dialog));
 
-  SG_LIST_FOREACH(l, dialog->priv->cancel_list)
-    st_stream_properties_dialog_cancel_info_free(l->data);
-
+  g_slist_foreach(dialog->priv->cancel_list, (GFunc) st_stream_properties_dialog_cancel_info_free, NULL);
   g_slist_free(dialog->priv->cancel_list);
   dialog->priv->cancel_list = NULL;
 }
@@ -758,7 +830,9 @@ st_stream_properties_dialog_apply (STStreamPropertiesDialog *dialog)
     }
 
   g_slist_free(fields);
-  st_stream_properties_dialog_values_free(values);
+
+  g_slist_foreach(values, (GFunc) sg_value_free, NULL);
+  g_slist_free(values);
   
   st_stream_properties_dialog_cancel_list_append(dialog, info);
   st_stream_properties_dialog_update_title(dialog);
