@@ -29,12 +29,21 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include "config.h"
+#include <glib/gi18n.h>
 #include "sg-util.h"
 #include "sgtk-util.h"
 #include "st-stream-store.h"
 #include "st-handler-field.h"
 #include "st-handler.h"
 
+/*** type definitions ********************************************************/
+
+struct _STStreamStorePrivate
+{
+  gboolean touched;
+};
+
 /*** variable declarations ***************************************************/
 
 static GdkPixbuf *blank_pixbuf = NULL;
@@ -42,6 +51,7 @@ static GdkPixbuf *blank_pixbuf = NULL;
 /*** function declarations ***************************************************/
 
 static void st_stream_store_class_init (STStreamStoreClass *class);
+static void st_stream_store_init (STStreamStore *store);
 
 static void st_stream_store_sort_column_changed_h (GtkTreeSortable *treesortable,
 						   gpointer user_data);
@@ -77,7 +87,7 @@ st_stream_store_get_type (void)
 	NULL,
 	sizeof(STStreamStore),
 	0,
-	NULL
+	(GInstanceInitFunc) st_stream_store_init
       };
       
       stream_store_type = g_type_register_static(GTK_TYPE_LIST_STORE,
@@ -96,6 +106,8 @@ st_stream_store_class_init (STStreamStoreClass *class)
   int width;
   int height;
 
+  g_type_class_add_private(class, sizeof(STStreamStorePrivate));
+
   /* create a blank pixbuf for ST_STREAM_BAG_STATE_NORMAL */
 
   status = gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
@@ -105,6 +117,12 @@ st_stream_store_class_init (STStreamStoreClass *class)
   gdk_pixbuf_fill(blank_pixbuf, 0); /* fill with transparent black */
 }
 
+static void
+st_stream_store_init (STStreamStore *store)
+{
+  store->priv = G_TYPE_INSTANCE_GET_PRIVATE(store, ST_TYPE_STREAM_STORE, STStreamStorePrivate);
+}
+
 STStreamStore *
 st_stream_store_new (STHandler *handler)
 {
@@ -134,7 +152,12 @@ st_stream_store_new (STHandler *handler)
       if (ST_HANDLER_FIELD_IS_VISIBLE(field))
 	{
 	  GType type = st_handler_field_get_type(field);
-	  types[i++] = type == G_TYPE_VALUE_ARRAY ? G_TYPE_STRING : type;
+
+	  /* keep in sync with st_stream_store_update() */
+	  if (type == G_TYPE_BOOLEAN || type == G_TYPE_VALUE_ARRAY)
+	    type = G_TYPE_STRING;
+	    
+	  types[i++] = type;
 	}
     }
 
@@ -231,6 +254,7 @@ st_stream_store_stream_changed_h (STStreamBag *bag,
   STStreamStore *store = user_data;
 
   st_stream_store_update(store, bag);
+  st_stream_store_touch(store);
 }
 
 static void
@@ -259,6 +283,7 @@ st_stream_store_stream_deleted_h (STStreamBag *bag,
 		      NULL);
 
   gtk_list_store_remove(GTK_LIST_STORE(store), &bag->iter);
+  st_stream_store_touch(store);
 }
 
 void
@@ -290,17 +315,26 @@ st_stream_store_update (STStreamStore *store, STStreamBag *bag)
 	  GValue value = { 0, };
 
 	  st_stream_bag_get_field(bag, field, &value);
-	  if (G_VALUE_HOLDS(&value, G_TYPE_VALUE_ARRAY))
+
+	  if (G_VALUE_HOLDS_BOOLEAN(&value))
+	    {
+	      gboolean b = g_value_get_boolean(&value);
+
+	      g_value_unset(&value);
+	      g_value_init(&value, G_TYPE_STRING);
+	      g_value_set_string(&value, b ? _("Yes") : _("No"));
+	    }
+	  else if (G_VALUE_HOLDS(&value, G_TYPE_VALUE_ARRAY))
 	    {
 	      GValueArray *value_array;
 	      char *str;
 
 	      value_array = g_value_get_boxed(&value);
-	      str = sg_value_array_get_string(value_array);
+	      str = value_array ? sg_value_array_get_string(value_array) : NULL;
 
 	      g_value_unset(&value);
 	      g_value_init(&value, G_TYPE_STRING);
-	      g_value_set_string_take_ownership(&value, str);
+	      g_value_take_string(&value, str);
 	    }
 	  
 	  gtk_list_store_set_value(GTK_LIST_STORE(store), &bag->iter, i++, &value);
@@ -311,7 +345,7 @@ st_stream_store_update (STStreamStore *store, STStreamBag *bag)
 
 void
 st_stream_store_foreach (STStreamStore *store,
-			 STStreamStoreForeachCallback *cb,
+			 STStreamStoreForeachCallback cb,
 			 gpointer data)
 {
   GtkTreeIter iter;
@@ -352,3 +386,19 @@ st_stream_store_search_equal_func (GtkTreeModel *model,
 
   return ! matches;
 }
+
+void
+st_stream_store_touch (STStreamStore *store)
+{
+  g_return_if_fail(ST_IS_STREAM_STORE(store));
+
+  store->priv->touched = TRUE;
+}
+
+gboolean
+st_stream_store_is_touched (STStreamStore *store)
+{
+  g_return_val_if_fail(ST_IS_STREAM_STORE(store), FALSE);
+
+  return store->priv->touched;
+}
