@@ -61,6 +61,7 @@ enum {
 struct _STStreamViewPrivate
 {
   STHandler		*handler;
+  GtkWidget		*menu;
   STStreamMenuItems	*menu_items;
 };
   
@@ -83,18 +84,29 @@ static void st_stream_view_set_property (GObject	*object,
 					 const GValue	*value,
 					 GParamSpec	*pspec);
 
-static void st_stream_view_construct_columns_menu	(STStreamView *view);
+static gboolean st_stream_view_column_drop_cb (GtkTreeView *view,
+					       GtkTreeViewColumn *column,
+					       GtkTreeViewColumn *prev_column,
+					       GtkTreeViewColumn *next_column,
+					       gpointer data);
+
 static void st_stream_view_construct_columns		(STStreamView *view);
 static void st_stream_view_column_add_field	(GtkTreeViewColumn *column,
 						 STHandlerField    *field,
 						 int               column_id);
-static void st_stream_view_update_columns_menu		(STStreamView *view);
 
 static void st_stream_view_construct_column_menu  (STStreamView       *view,
 						   GtkTreeViewColumn  *column);
 
-static void st_stream_view_column_visible_toggled_h (GtkCheckMenuItem *item,
-						     gpointer	      data);
+static gboolean st_stream_view_column_popup_menu_h (GtkWidget *widget,
+						    gpointer user_data);
+static gboolean st_stream_view_column_button_press_event_h (GtkWidget *widget,
+							    GdkEventButton *button,
+							    gpointer user_data);
+
+static void st_stream_view_column_notify_visible_h	(GObject      *object,
+							 GParamSpec   *pspec,
+							 gpointer     data);
 static void st_stream_view_column_notify_width_h	(GObject      *object,
 							 GParamSpec   *pspec,
 							 gpointer     data);
@@ -102,12 +114,25 @@ static void st_stream_view_column_hide_activate_h	(GtkMenuItem  *item,
 							 gpointer     data);
 static void st_stream_view_column_show_all_activate_h	(GtkMenuItem  *item,
 							 gpointer     data);
+static void st_stream_view_column_stream_columns_activate_h (GtkMenuItem *item,
+							     gpointer    data);
 static void st_stream_view_selection_changed_h	(GtkTreeSelection     *selection,
 						 gpointer	      user_data);
+static void st_stream_view_columns_changed_h	(GtkTreeView          *view,
+						 gpointer             user_data);
+static void st_stream_view_destroy_h		(GtkObject            *object,
+						 gpointer             user_data);
 
-static gboolean st_stream_view_get_first_selection (STStreamView      *view,
+static void	st_stream_view_order_columns       (STStreamView      *view);
+static gboolean	st_stream_view_get_first_selection (STStreamView      *view,
 						    GtkTreeIter       *iter);
-     
+
+static gboolean	st_stream_view_model_iter_move     (GtkTreeModel      *model,
+						    GtkTreeIter       *iter,
+						    sGtkDirection     direction,
+						    gboolean          wrap_around,
+						    gboolean          *edge_hit);
+
 /*** implementation **********************************************************/
 
 GType
@@ -145,6 +170,8 @@ st_stream_view_class_init (STStreamViewClass *class)
 
   parent_class = g_type_class_peek_parent(class);
 
+  g_type_class_add_private(class, sizeof(STStreamViewPrivate));
+
   object_class->finalize = st_stream_view_finalize;
   object_class->constructor = st_stream_view_constructor;
   object_class->set_property = st_stream_view_set_property;
@@ -152,8 +179,8 @@ st_stream_view_class_init (STStreamViewClass *class)
   g_object_class_install_property(object_class,
 				  PROP_HANDLER,
 				  g_param_spec_pointer("handler",
-						       _("Handler"),
-						       _("The view's STHandler object"),
+						       NULL,
+						       NULL,
 						       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 
   stream_view_signals[SELECTION_CHANGED] = g_signal_new("selection-changed",
@@ -170,8 +197,7 @@ st_stream_view_class_init (STStreamViewClass *class)
 static void
 st_stream_view_init (STStreamView *view)
 {
-  view->priv = g_new0(STStreamViewPrivate, 1);
-  view->columns_menu = NULL;
+  view->priv = G_TYPE_INSTANCE_GET_PRIVATE(view, ST_TYPE_STREAM_VIEW, STStreamViewPrivate);
 }
 
 static void
@@ -179,13 +205,12 @@ st_stream_view_finalize (GObject *object)
 {
   STStreamView *view = ST_STREAM_VIEW(object);
 
-  g_object_unref(view->columns_menu);
-
+  if (view->priv->menu)
+    gtk_widget_destroy(view->priv->menu);
   if (view->priv->menu_items)
     st_stream_menu_items_free(view->priv->menu_items);
-  g_free(view->priv);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  parent_class->finalize(object);
 }
 
 static GObject *
@@ -197,9 +222,7 @@ st_stream_view_constructor (GType type,
   STStreamView *view;
   GtkTreeSelection *selection;
 
-  object = G_OBJECT_CLASS(parent_class)->constructor(type,
-						     n_construct_properties,
-						     construct_params);
+  object = parent_class->constructor(type, n_construct_properties, construct_params);
   view = ST_STREAM_VIEW(object);
 
   gtk_tree_view_set_enable_search(GTK_TREE_VIEW(view), TRUE);
@@ -217,10 +240,17 @@ st_stream_view_constructor (GType type,
 		   view);
 
   st_stream_view_construct_columns(view);
+  st_stream_view_order_columns(view);
 
-  /* the following should be called after the columns have been created */
-  st_stream_view_construct_columns_menu(view);
-  st_stream_view_update_columns_menu(view);
+  g_object_connect(view,
+		   "signal::columns-changed", st_stream_view_columns_changed_h, NULL,
+		   "signal::destroy", st_stream_view_destroy_h, NULL,
+		   NULL);
+
+  gtk_tree_view_set_column_drag_function(GTK_TREE_VIEW(view),
+					 st_stream_view_column_drop_cb,
+					 NULL,
+					 NULL);
 
   return object;
 }
@@ -231,9 +261,7 @@ st_stream_view_set_property (GObject *object,
 			     const GValue *value,
 			     GParamSpec *pspec)
 {
-  STStreamView *view;
-
-  view = ST_STREAM_VIEW(object);
+  STStreamView *view = ST_STREAM_VIEW(object);
 
   switch (prop_id)
     {
@@ -247,71 +275,25 @@ st_stream_view_set_property (GObject *object,
     }
 }
 
-static void
-st_stream_view_construct_columns_menu (STStreamView *view)
+static gboolean
+st_stream_view_column_drop_cb (GtkTreeView *view,
+			       GtkTreeViewColumn *column,
+			       GtkTreeViewColumn *prev_column,
+			       GtkTreeViewColumn *next_column,
+			       gpointer data)
 {
-  GtkTreeViewColumn *column;
-  int i = N_VISIBLE_STOCK_COLUMNS;
+  STHandlerField *prev_field;
+  STHandlerField *next_field;
 
-  g_return_if_fail(ST_IS_STREAM_VIEW(view));
-  
-  view->columns_menu = gtk_menu_new();
+  prev_field = prev_column ? st_stream_view_column_get_field(prev_column) : NULL;
+  next_field = next_column ? st_stream_view_column_get_field(next_column) : NULL;
 
-  g_object_ref(view->columns_menu);
-  gtk_object_sink(GTK_OBJECT(view->columns_menu));
-
-  while ((column = gtk_tree_view_get_column(GTK_TREE_VIEW(view), i++)))
-    {
-      STHandlerField *field = g_object_get_data(G_OBJECT(column), "field");
-      GtkWidget *item;
-      unsigned int hid;
-
-      item = gtk_check_menu_item_new_with_label(st_handler_field_get_label(field));
-      gtk_widget_show(item);
-
-      gtk_menu_shell_append(GTK_MENU_SHELL(view->columns_menu), item);
-
-      hid = g_signal_connect(item,
-			     "toggled",
-			     G_CALLBACK(st_stream_view_column_visible_toggled_h),
-			     NULL);
-
-      g_object_set_data(G_OBJECT(item), "hid", GINT_TO_POINTER(hid));
-      g_object_set_data(G_OBJECT(item), "field", field);
-      g_object_set_data(G_OBJECT(item), "column", column);
-    }
-}
-
-static void
-st_stream_view_column_visible_toggled_h (GtkCheckMenuItem *item, gpointer data)
-{
-  STHandlerField *field = g_object_get_data(G_OBJECT(item), "field");
-  GtkTreeViewColumn *column = g_object_get_data(G_OBJECT(item), "column");
-  gboolean visible;
-
-  visible = gtk_check_menu_item_get_active(item);
-  st_handler_field_set_user_visible(field, visible);
-  gtk_tree_view_column_set_visible(column, visible);
-}
-
-static void
-st_stream_view_update_columns_menu (STStreamView *view)
-{
-  GList *l;
-
-  g_return_if_fail(ST_IS_STREAM_VIEW(view));
-  g_return_if_fail(GTK_IS_MENU_SHELL(view->columns_menu));
-
-  SG_LIST_FOREACH(l, GTK_MENU_SHELL(view->columns_menu)->children)
-    {
-      GtkWidget *item = l->data;
-      STHandlerField *field = g_object_get_data(G_OBJECT(item), "field");
-      unsigned int hid = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(item), "hid"));
-
-      g_signal_handler_block(item, hid);
-      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), st_handler_field_get_user_visible(field));
-      g_signal_handler_unblock(item, hid);
-    }
+  /*
+   * Do not allow to move between two stock columns, or between a
+   * stock column and an edge (we recognize stock columns because they
+   * have no associated STHandlerField).
+   */
+  return prev_field || next_field;
 }
 
 static void
@@ -358,19 +340,20 @@ st_stream_view_construct_columns (STStreamView *view)
 	      gtk_tree_view_column_set_sort_column_id(column, vi);
 	      gtk_tree_view_column_set_resizable(column, TRUE);
 	      gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+	      gtk_tree_view_column_set_reorderable(column, TRUE);
 
 	      g_object_set_data(G_OBJECT(column), "field", field);
 
 	      width = st_handler_field_get_width(field);
 	      /* width == 0 only if loaded from an old config file */
 	      gtk_tree_view_column_set_fixed_width(column, width > 0 ? width : ST_HANDLER_FIELD_DEFAULT_WIDTH);
-      
+
 	      gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
 	      st_stream_view_construct_column_menu(view, column);
 
-	      g_signal_connect(column,
-			       "notify::width",
-			       G_CALLBACK(st_stream_view_column_notify_width_h),
+	      g_object_connect(column,
+			       "signal::notify::visible", st_stream_view_column_notify_visible_h, NULL,
+			       "signal::notify::width", st_stream_view_column_notify_width_h, NULL,
 			       NULL);
 	    }
 	  else
@@ -398,13 +381,7 @@ st_stream_view_column_add_field (GtkTreeViewColumn *column,
   g_return_if_fail(field != NULL);
 
   type = st_handler_field_get_type(field);
-  if (type == G_TYPE_BOOLEAN)
-    {
-      renderer = gtk_cell_renderer_toggle_new();
-      property = "active";
-      expand = FALSE;
-    }
-  else if (type == GDK_TYPE_PIXBUF)
+  if (type == GDK_TYPE_PIXBUF)
     {
       renderer = gtk_cell_renderer_pixbuf_new();
       property = "pixbuf";
@@ -426,11 +403,10 @@ st_stream_view_construct_column_menu (STStreamView *view,
 				      GtkTreeViewColumn *column)
 {
   GtkWidget *menu;
-
   GtkWidget *hide;
-  GtkWidget *image;
-  GtkWidget *separator;
   GtkWidget *show_all;
+  GtkWidget *separator;
+  GtkWidget *stream_columns;
   
   g_return_if_fail(ST_IS_STREAM_VIEW(view));
   g_return_if_fail(GTK_IS_TREE_VIEW_COLUMN(column));
@@ -438,21 +414,22 @@ st_stream_view_construct_column_menu (STStreamView *view,
 
   menu = gtk_menu_new();
 
-  hide = gtk_image_menu_item_new_with_mnemonic(_("_Hide This Column"));
-  image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(hide), image);
-  
+  hide = gtk_image_menu_item_new_from_stock(ST_STOCK_HIDE_THIS_COLUMN, NULL);
+  show_all = gtk_image_menu_item_new_from_stock(ST_STOCK_SHOW_ALL_COLUMNS, NULL);
   separator = gtk_separator_menu_item_new();
-  show_all = gtk_menu_item_new_with_mnemonic(_("_Show All Columns"));
+  stream_columns = gtk_image_menu_item_new_from_stock(ST_STOCK_STREAM_COLUMNS, NULL);
 
   gtk_widget_show(hide);
-  gtk_widget_show(image);
-  gtk_widget_show(separator);
   gtk_widget_show(show_all);
+  gtk_widget_show(separator);
+  gtk_widget_show(stream_columns);
   
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), hide);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), separator);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), show_all);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), separator);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), stream_columns);
+
+  g_object_set_data_full(G_OBJECT(column), "menu", menu, (GDestroyNotify) gtk_widget_destroy);
 
   g_signal_connect(hide,
 		   "activate",
@@ -462,8 +439,60 @@ st_stream_view_construct_column_menu (STStreamView *view,
 		   "activate",
 		   G_CALLBACK(st_stream_view_column_show_all_activate_h),
 		   view);
+  g_signal_connect(stream_columns,
+		   "activate",
+		   G_CALLBACK(st_stream_view_column_stream_columns_activate_h),
+		   view);
 
-  sgtk_widget_set_popup_menu(column->button, GTK_MENU(menu));
+  g_object_connect(column->button,
+		   "signal::popup-menu", st_stream_view_column_popup_menu_h, column,
+		   "signal::button-press-event", st_stream_view_column_button_press_event_h, column,
+		   NULL);
+}
+
+static gboolean
+st_stream_view_column_popup_menu_h (GtkWidget *widget, gpointer user_data)
+{
+  GtkTreeViewColumn *column = user_data;
+  GtkMenu *menu = g_object_get_data(G_OBJECT(column), "menu");
+
+  gtk_menu_popup(menu, NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
+  
+  return TRUE;			/* a menu was activated */
+}
+
+static gboolean
+st_stream_view_column_button_press_event_h (GtkWidget *widget,
+					    GdkEventButton *event,
+					    gpointer user_data)
+{
+  if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
+    {
+      GtkTreeViewColumn *column = user_data;
+      GtkMenu *menu = g_object_get_data(G_OBJECT(column), "menu");
+
+      /* workaround for http://bugzilla.gnome.org/show_bug.cgi?id=159640 */
+      column->maybe_reordered = FALSE;
+
+      gtk_menu_popup(menu,
+		     NULL,
+		     NULL,
+		     NULL,
+		     NULL,
+		     event->button,
+		     event->time);
+    }
+
+  return FALSE;			/* propagate event */
+}
+
+static void
+st_stream_view_column_notify_visible_h (GObject *object,
+					GParamSpec *pspec,
+					gpointer data)
+{
+  STHandlerField *field = st_stream_view_column_get_field(GTK_TREE_VIEW_COLUMN(object));
+  st_handler_field_set_user_visible(field, gtk_tree_view_column_get_visible(GTK_TREE_VIEW_COLUMN(object)));
 }
 
 static void
@@ -471,7 +500,7 @@ st_stream_view_column_notify_width_h (GObject *object,
 				      GParamSpec *pspec,
 				      gpointer data)
 {
-  STHandlerField *field = g_object_get_data(object, "field");
+  STHandlerField *field = st_stream_view_column_get_field(GTK_TREE_VIEW_COLUMN(object));
   st_handler_field_set_width(field, gtk_tree_view_column_get_width(GTK_TREE_VIEW_COLUMN(object)));
 }
 
@@ -479,12 +508,8 @@ static void
 st_stream_view_column_hide_activate_h (GtkMenuItem *item, gpointer data)
 {
   GtkTreeViewColumn *column = data;
-  STHandlerField *field = g_object_get_data(G_OBJECT(column), "field");
 
-  st_handler_field_set_user_visible(field, FALSE);
   gtk_tree_view_column_set_visible(column, FALSE);
-
-  st_stream_view_update_columns_menu(ST_STREAM_VIEW(column->tree_view));
 }
 
 static void
@@ -495,14 +520,13 @@ st_stream_view_column_show_all_activate_h (GtkMenuItem *item, gpointer data)
   int i = N_VISIBLE_STOCK_COLUMNS;
 
   while ((column = gtk_tree_view_get_column(GTK_TREE_VIEW(view), i++)))
-    {
-      STHandlerField *field = g_object_get_data(G_OBJECT(column), "field");
-	  
-      st_handler_field_set_user_visible(field, TRUE);
-      gtk_tree_view_column_set_visible(column, TRUE);
-    }
+    gtk_tree_view_column_set_visible(column, TRUE);
+}
 
-  st_stream_view_update_columns_menu(view);
+static void
+st_stream_view_column_stream_columns_activate_h (GtkMenuItem *item, gpointer data)
+{
+  st_shell_present_stream_columns(st_shell);
 }
 
 static void
@@ -514,6 +538,56 @@ st_stream_view_selection_changed_h (GtkTreeSelection *selection,
   g_signal_emit(view, stream_view_signals[SELECTION_CHANGED], 0);
 }
 
+static void
+st_stream_view_columns_changed_h (GtkTreeView *view, gpointer user_data)
+{
+  GtkTreeViewColumn *column;
+  int i = N_VISIBLE_STOCK_COLUMNS;
+  int pos = 0;
+  
+  while ((column = gtk_tree_view_get_column(view, i++)))
+    {
+      STHandlerField *field = st_stream_view_column_get_field(column);
+      st_handler_field_set_position(field, pos++);
+    }
+}
+
+static void
+st_stream_view_destroy_h (GtkObject *object, gpointer user_data)
+{
+  g_signal_handlers_disconnect_by_func(object, st_stream_view_columns_changed_h, NULL);
+}
+
+static void
+st_stream_view_order_columns (STStreamView *view)
+{
+  GList *columns;
+  GList *l;
+
+  g_return_if_fail(ST_IS_STREAM_VIEW(view));
+
+  columns = gtk_tree_view_get_columns(GTK_TREE_VIEW(view));
+  SG_LIST_FOREACH(l, g_list_nth(columns, N_VISIBLE_STOCK_COLUMNS))
+    {
+      GtkTreeViewColumn *column = l->data;
+      STHandlerField *field;
+      int position;
+
+      field = st_stream_view_column_get_field(column);
+      position = st_handler_field_get_position(field);
+
+      if (position != -1)
+	{
+	  GtkTreeViewColumn *after;
+
+	  after = gtk_tree_view_get_column(GTK_TREE_VIEW(view), position + N_VISIBLE_STOCK_COLUMNS - 1);
+	  if (after != column)
+	    gtk_tree_view_move_column_after(GTK_TREE_VIEW(view), column, after);
+	}
+    }
+  g_list_free(columns);
+}
+
 GtkWidget *
 st_stream_view_new (STHandler *handler)
 {
@@ -522,6 +596,14 @@ st_stream_view_new (STHandler *handler)
 		      NULL);
 }
 
+STHandler *
+st_stream_view_get_handler (STStreamView *view)
+{
+  g_return_val_if_fail(ST_IS_STREAM_VIEW(view), NULL);
+
+  return view->priv->handler;
+}
+
 void
 st_stream_view_update_sensitivity (STStreamView *view)
 {
@@ -675,15 +757,13 @@ st_stream_view_get_first_selection (STStreamView *view, GtkTreeIter *iter)
       paths = gtk_tree_selection_get_selected_rows(selection, NULL);
       if (paths)
 	{
-	  GList *l;
 	  GtkTreePath *path = paths->data;
 	  gboolean status;
 	  
 	  status = gtk_tree_model_get_iter(model, iter, path);
 	  g_return_val_if_fail(status == TRUE, FALSE);
 
-	  SG_LIST_FOREACH(l, paths)
-	    gtk_tree_path_free(l->data);
+	  g_list_foreach(paths, (GFunc) gtk_tree_path_free, NULL);
 	  g_list_free(paths);
 
 	  return TRUE;
@@ -699,17 +779,15 @@ st_stream_view_get_first_selection (STStreamView *view, GtkTreeIter *iter)
 void
 st_stream_view_set_menu_items (STStreamView *view, STStreamMenuItems *items)
 {
-  GtkWidget *menu;
-
   g_return_if_fail(ST_IS_STREAM_VIEW(view));
+  g_return_if_fail(view->priv->menu_items == NULL);
   g_return_if_fail(items != NULL);
 
+  view->priv->menu = gtk_menu_new();
   view->priv->menu_items = items;
   
-  menu = gtk_menu_new();
-  st_stream_menu_items_insert_into_shell(view->priv->menu_items, GTK_MENU_SHELL(menu), 0);
-
-  sgtk_widget_set_popup_menu(GTK_WIDGET(view), GTK_MENU(menu));
+  st_stream_menu_items_insert_into_shell(view->priv->menu_items, GTK_MENU_SHELL(view->priv->menu), 0);
+  sgtk_widget_set_popup_menu(GTK_WIDGET(view), GTK_MENU(view->priv->menu));
 }
 
 void
@@ -791,6 +869,42 @@ st_stream_view_select_all (STStreamView *view)
   gtk_tree_selection_select_all(selection);
 }
 
+static gboolean
+st_stream_view_model_iter_move (GtkTreeModel *model,
+				GtkTreeIter *iter,
+				sGtkDirection direction,
+				gboolean wrap_around,
+				gboolean *edge_hit)
+{
+  gboolean valid;
+
+  g_return_val_if_fail(GTK_IS_TREE_MODEL(model), FALSE);
+  g_return_val_if_fail(iter != NULL, FALSE);
+
+  valid = sgtk_tree_model_iter_move(model, iter, direction);
+  if (! valid && wrap_around)
+    {
+      if (edge_hit)
+	*edge_hit = TRUE;
+
+      switch (direction)
+	{
+	case SGTK_PREVIOUS:
+	  valid = sgtk_tree_model_get_iter_last(model, iter);
+	  break;
+	  
+	case SGTK_NEXT:
+	  valid = gtk_tree_model_get_iter_first(model, iter);
+	  break;
+	  
+	default:
+	  g_return_val_if_reached(FALSE);
+	}
+    }
+
+  return valid;
+}
+
 gboolean
 st_stream_view_find (STStreamView *view,
 		     sGtkDirection direction,
@@ -819,7 +933,7 @@ st_stream_view_find (STStreamView *view,
 	  STStreamBag *bag = last->data;
 
 	  iter = bag->iter;
-	  valid = sgtk_tree_model_iter_move(model, &iter, direction);
+	  valid = st_stream_view_model_iter_move(model, &iter, direction, wrap_around, &edge_hit);
 
 	  sg_objects_free(selected_streams);
 	}
@@ -842,26 +956,32 @@ st_stream_view_find (STStreamView *view,
 	  if (found)
 	    break;
 
-	  valid = sgtk_tree_model_iter_move(model, &iter, direction);
-	  if (! valid && wrap_around && ! edge_hit)
-	    {
-	      edge_hit = TRUE;
-	      switch (direction)
-		{
-		case SGTK_PREVIOUS:
-		  valid = sgtk_tree_model_get_iter_last(model, &iter);
-		  break;
-
-		case SGTK_NEXT:
-		  valid = gtk_tree_model_get_iter_first(model, &iter);
-		  break;
-
-		default:
-		  g_return_val_if_reached(FALSE);
-		}
-	    }
+	  valid = st_stream_view_model_iter_move(model, &iter, direction, wrap_around && ! edge_hit, &edge_hit);
 	}
     }
 
   return found;
 }
+
+GSList *
+st_stream_view_get_columns (STStreamView *view)
+{
+  GSList *columns = NULL;
+  int i = N_VISIBLE_STOCK_COLUMNS;
+  GtkTreeViewColumn *column;
+
+  g_return_val_if_fail(ST_IS_STREAM_VIEW(view), NULL);
+
+  while ((column = gtk_tree_view_get_column(GTK_TREE_VIEW(view), i++)))
+    columns = g_slist_append(columns, column);
+
+  return columns;
+}
+
+STHandlerField *
+st_stream_view_column_get_field (GtkTreeViewColumn *column)
+{
+  g_return_val_if_fail(GTK_IS_TREE_VIEW_COLUMN(column), NULL);
+
+  return g_object_get_data(G_OBJECT(column), "field");
+}
