@@ -51,7 +51,6 @@ typedef struct
   GtkTreeView		view;
 
   STStreamViewPrivate	*priv;
-  GtkWidget		*columns_menu;
 } STStreamView;
 
 typedef struct
@@ -64,6 +63,7 @@ typedef struct
 
 GType		st_stream_view_get_type		   (void);
 GtkWidget	*st_stream_view_new		   (STHandler     *handler);
+STHandler	*st_stream_view_get_handler        (STStreamView  *view);
 void		st_stream_view_set_menu_items      (STStreamView  *view,
 						    STStreamMenuItems *items);
 void		st_stream_view_update_sensitivity  (STStreamView  *view);
@@ -92,4 +92,7 @@ gboolean	st_stream_view_find                (STStreamView  *view,
 						    gboolean      case_sensitive,
 						    gboolean      wrap_around);
 
+GSList		*st_stream_view_get_columns        (STStreamView  *view);
+STHandlerField	*st_stream_view_column_get_field   (GtkTreeViewColumn *column);
+
 #endif /* _ST_STREAM_VIEW_H */
