@@ -37,8 +37,14 @@
 #define ST_COPYRIGHT	"Copyright \302\251 2002, 2003, 2004 Jean-Yves Lefort"
 #define ST_HOME		"http://www.nongnu.org/streamtuner/"
 
+#define ST_PNG(name)	UIDIR G_DIR_SEPARATOR_S name ".png"
+
 GtkWidget	*st_logo_new		(void);
 void		st_show_help		(const char	*link_id);
 void		st_visit_homepage	(const char	*url);
+void		st_parser_msg_cb	(GScanner	*scanner,
+					 char		*message,
+					 gboolean	error);
+void		st_window_set_icon	(GtkWindow	*window);
 
 #endif /* _ST_UTIL_H */
